# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: apps/WorkingMemory.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='apps/WorkingMemory.proto',
  package='com.mammut.interpreter.memory',
  syntax='proto3',
  serialized_options=_b('B\023WorkingMemoryProtos'),
  serialized_pb=_b('\n\x18\x61pps/WorkingMemory.proto\x12\x1d\x63om.mammut.interpreter.memory\"\xc2\x05\n\x12WorkingMemoryValue\x12\x45\n\x04type\x18\x01 \x01(\x0e\x32\x37.com.mammut.interpreter.memory.WorkingMemoryValue.types\x12U\n\x0c\x63\x61ridanility\x18\x02 \x01(\x0e\x32?.com.mammut.interpreter.memory.WorkingMemoryValue.cardinalities\x12\x12\n\x08valueInt\x18\x03 \x01(\x05H\x00\x12\x13\n\tvalueLong\x18\x04 \x01(\x03H\x00\x12\x15\n\x0bvalueString\x18\x05 \x01(\tH\x00\x12\x13\n\tvalueBool\x18\x06 \x01(\x08H\x00\x12\x15\n\x0bvalueDouble\x18\x07 \x01(\x01H\x00\x12\x14\n\nvalueOther\x18\x08 \x01(\tH\x00\x12\x0f\n\x07valueRS\x18\t \x03(\t\x12\x0f\n\x07valueRI\x18\n \x03(\x03\x12\x0f\n\x07valueRL\x18\x0b \x03(\x03\x12\x0f\n\x07valueRB\x18\x0c \x03(\x08\x12\x0f\n\x07valueRD\x18\r \x03(\x01\x12\x0f\n\x07valueRO\x18\x0e \x03(\t\"\xeb\x01\n\x05types\x12\n\n\x06STRING\x10\x00\x12\x07\n\x03INT\x10\x01\x12\x08\n\x04LONG\x10\x02\x12\x08\n\x04\x42OOL\x10\x03\x12\n\n\x06\x44OUBLE\x10\x04\x12\x11\n\rZONEDDATETIME\x10\x05\x12\r\n\tLOCALDATE\x10\x06\x12\r\n\tLOCALTIME\x10\x07\x12\x0e\n\nATTACHMENT\x10\x10\x12\x0f\n\x0bGATTACHMENT\x10\x11\x12\x0f\n\x0bGEOLOCATION\x10\x12\x12\x10\n\x0cOPENINGHOURS\x10\x13\x12\x11\n\rSTRINGTOLOWER\x10\x14\x12\x11\n\rSTRINGTOUPPER\x10\x15\x12\x12\n\x0eTYPEDESCRIPTOR\x10\x16\".\n\rcardinalities\x12\n\n\x06SINGLE\x10\x00\x12\x08\n\x04LIST\x10\x01\x12\x07\n\x03SET\x10\x02\x42\x08\n\x06valuesB\x15\x42\x13WorkingMemoryProtosb\x06proto3')
)



_WORKINGMEMORYVALUE_TYPES = _descriptor.EnumDescriptor(
  name='types',
  full_name='com.mammut.interpreter.memory.WorkingMemoryValue.types',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STRING', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INT', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LONG', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BOOL', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DOUBLE', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ZONEDDATETIME', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCALDATE', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCALTIME', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ATTACHMENT', index=8, number=16,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GATTACHMENT', index=9, number=17,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GEOLOCATION', index=10, number=18,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPENINGHOURS', index=11, number=19,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STRINGTOLOWER', index=12, number=20,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STRINGTOUPPER', index=13, number=21,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPEDESCRIPTOR', index=14, number=22,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=473,
  serialized_end=708,
)
_sym_db.RegisterEnumDescriptor(_WORKINGMEMORYVALUE_TYPES)

_WORKINGMEMORYVALUE_CARDINALITIES = _descriptor.EnumDescriptor(
  name='cardinalities',
  full_name='com.mammut.interpreter.memory.WorkingMemoryValue.cardinalities',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SINGLE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LIST', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SET', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=710,
  serialized_end=756,
)
_sym_db.RegisterEnumDescriptor(_WORKINGMEMORYVALUE_CARDINALITIES)


_WORKINGMEMORYVALUE = _descriptor.Descriptor(
  name='WorkingMemoryValue',
  full_name='com.mammut.interpreter.memory.WorkingMemoryValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='caridanility', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.caridanility', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueInt', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueInt', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueLong', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueLong', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueString', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueString', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueBool', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueBool', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueDouble', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueDouble', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueOther', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueOther', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueRS', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueRS', index=8,
      number=9, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueRI', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueRI', index=9,
      number=10, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueRL', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueRL', index=10,
      number=11, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueRB', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueRB', index=11,
      number=12, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueRD', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueRD', index=12,
      number=13, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueRO', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.valueRO', index=13,
      number=14, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _WORKINGMEMORYVALUE_TYPES,
    _WORKINGMEMORYVALUE_CARDINALITIES,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='values', full_name='com.mammut.interpreter.memory.WorkingMemoryValue.values',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=60,
  serialized_end=766,
)

_WORKINGMEMORYVALUE.fields_by_name['type'].enum_type = _WORKINGMEMORYVALUE_TYPES
_WORKINGMEMORYVALUE.fields_by_name['caridanility'].enum_type = _WORKINGMEMORYVALUE_CARDINALITIES
_WORKINGMEMORYVALUE_TYPES.containing_type = _WORKINGMEMORYVALUE
_WORKINGMEMORYVALUE_CARDINALITIES.containing_type = _WORKINGMEMORYVALUE
_WORKINGMEMORYVALUE.oneofs_by_name['values'].fields.append(
  _WORKINGMEMORYVALUE.fields_by_name['valueInt'])
_WORKINGMEMORYVALUE.fields_by_name['valueInt'].containing_oneof = _WORKINGMEMORYVALUE.oneofs_by_name['values']
_WORKINGMEMORYVALUE.oneofs_by_name['values'].fields.append(
  _WORKINGMEMORYVALUE.fields_by_name['valueLong'])
_WORKINGMEMORYVALUE.fields_by_name['valueLong'].containing_oneof = _WORKINGMEMORYVALUE.oneofs_by_name['values']
_WORKINGMEMORYVALUE.oneofs_by_name['values'].fields.append(
  _WORKINGMEMORYVALUE.fields_by_name['valueString'])
_WORKINGMEMORYVALUE.fields_by_name['valueString'].containing_oneof = _WORKINGMEMORYVALUE.oneofs_by_name['values']
_WORKINGMEMORYVALUE.oneofs_by_name['values'].fields.append(
  _WORKINGMEMORYVALUE.fields_by_name['valueBool'])
_WORKINGMEMORYVALUE.fields_by_name['valueBool'].containing_oneof = _WORKINGMEMORYVALUE.oneofs_by_name['values']
_WORKINGMEMORYVALUE.oneofs_by_name['values'].fields.append(
  _WORKINGMEMORYVALUE.fields_by_name['valueDouble'])
_WORKINGMEMORYVALUE.fields_by_name['valueDouble'].containing_oneof = _WORKINGMEMORYVALUE.oneofs_by_name['values']
_WORKINGMEMORYVALUE.oneofs_by_name['values'].fields.append(
  _WORKINGMEMORYVALUE.fields_by_name['valueOther'])
_WORKINGMEMORYVALUE.fields_by_name['valueOther'].containing_oneof = _WORKINGMEMORYVALUE.oneofs_by_name['values']
DESCRIPTOR.message_types_by_name['WorkingMemoryValue'] = _WORKINGMEMORYVALUE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

WorkingMemoryValue = _reflection.GeneratedProtocolMessageType('WorkingMemoryValue', (_message.Message,), dict(
  DESCRIPTOR = _WORKINGMEMORYVALUE,
  __module__ = 'apps.WorkingMemory_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.memory.WorkingMemoryValue)
  ))
_sym_db.RegisterMessage(WorkingMemoryValue)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
