# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: apps/InterpreterMain.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='apps/InterpreterMain.proto',
  package='com.mammut.interpreter',
  syntax='proto3',
  serialized_options=_b('B\025InterpreterMainProtos'),
  serialized_pb=_b('\n\x1a\x61pps/InterpreterMain.proto\x12\x16\x63om.mammut.interpreter\"\xdd\x01\n\x14InterpreterMainInput\x12\x10\n\x08mammutId\x18\x01 \x01(\x03\x12\x0f\n\x07slideId\x18\x02 \x01(\t\x12\x1d\n\x15\x65xecuteSamplingCorpus\x18\x03 \x01(\x08\x12\x11\n\tcorpusIds\x18\x04 \x03(\x05\x12\x1e\n\x16metadataTokenSeparator\x18\x05 \x01(\t\x12\x18\n\x10regionalSettings\x18\x06 \x01(\t\x12\x0e\n\x06userId\x18\x07 \x01(\x03\x12\x0e\n\x06roomId\x18\x08 \x01(\x03\x12\x16\n\x0emammutLoggerId\x18\t \x01(\x03\"\x17\n\x15ImportantMessageProto\"Y\n\x17UnsupportedFeatureProto\x12>\n\x0e\x65rrorComponent\x18\x01 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"\'\n\x12\x43ompilerPhaseProto\x12\x11\n\textraInfo\x18\x01 \x01(\t\":\n\x15InterpreterPhaseProto\x12\x0e\n\x06roomId\x18\x01 \x01(\x04\x12\x11\n\textraInfo\x18\x02 \x01(\t\"w\n\x1fPropertyDescriptorNotFoundProto\x12\x14\n\x0cpropertyName\x18\x01 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x02 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"o\n\x1b\x45\x64geDescriptorNotFoundProto\x12\x10\n\x08\x65\x64geName\x18\x01 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x02 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"\x97\x01\n\x1aPropertyValueNotFoundProto\x12\x14\n\x0cpropertyName\x18\x01 \x01(\t\x12\x12\n\nvertexName\x18\x02 \x01(\t\x12\x0f\n\x07sheetId\x18\x03 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x04 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"j\n\x16\x45\x64geValueNotFoundProto\x12\x10\n\x08traverse\x18\x01 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x04 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"\x85\x01\n\x1aUnknownPropertyOrEdgeProto\x12\x13\n\x0b\x65lementName\x18\x01 \x01(\t\x12\x12\n\nvertexName\x18\x02 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x03 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"m\n\x19OntologyTypeNotFoundProto\x12\x10\n\x08typeName\x18\x01 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x02 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"\x8b\x01\n\x14VertexUndefinedProto\x12\x11\n\tsrcVertex\x18\x01 \x01(\t\x12\x0f\n\x07srcEdge\x18\x02 \x01(\t\x12\x0f\n\x07sheetId\x18\x03 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x04 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"e\n\x12SheetNotFoundProto\x12\x0f\n\x07sheetId\x18\x01 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x02 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"r\n\x19ReservedKeywordFoundProto\x12\x15\n\rreserverdWord\x18\x01 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x02 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"t\n\x1cUnexpectedCompilerErrorProto\x12\x14\n\x0c\x63\x61useMessage\x18\x01 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x02 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"\x85\x01\n\x14ObjectUndefinedProto\x12\x15\n\robjectElement\x18\x01 \x01(\t\x12\x16\n\x0epackageElement\x18\x02 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x03 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"x\n\"DefaultVertexPropertyNotFoundProto\x12\x12\n\nvertexName\x18\x01 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x03 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"z\n$MultipleDefaultVertexPropertiesProto\x12\x12\n\nvertexName\x18\x01 \x01(\t\x12>\n\x0e\x65rrorComponent\x18\x03 \x01(\x0e\x32&.com.mammut.interpreter.ErrorComponent\"\xad\"\n\x15InterpreterMainOutput\x12\x41\n\x04type\x18\x01 \x01(\x0e\x32\x33.com.mammut.interpreter.InterpreterMainOutput.Types\x12\x15\n\runknownMammut\x18\x02 \x01(\x08\x12\x13\n\x0bunknownRoom\x18\x66 \x01(\x08\x12\x13\n\x0bunknownUser\x18g \x01(\x08\x12\x10\n\x08mammutId\x18\x03 \x01(\x03\x12\x0e\n\x06roomId\x18\x64 \x01(\x03\x12\x0e\n\x06userId\x18\x65 \x01(\x03\x12\x43\n\x05level\x18\x04 \x01(\x0e\x32\x34.com.mammut.interpreter.InterpreterMainOutput.Levels\x12\x0f\n\x07message\x18\x05 \x01(\t\x12\x11\n\tthrowable\x18\x06 \x01(\t\x12\x42\n\x05phase\x18\x07 \x01(\x0e\x32\x33.com.mammut.interpreter.InterpreterMainOutput.Phase\x12\x1b\n\x13\x63orpusCompilationId\x18\x08 \x01(\t\x12L\n\ncorpusType\x18\t \x01(\x0e\x32\x38.com.mammut.interpreter.InterpreterMainOutput.CorpusType\x12I\n\x10importantMessage\x18\n \x01(\x0b\x32-.com.mammut.interpreter.ImportantMessageProtoH\x00\x12M\n\x12unsupportedFeature\x18\x0b \x01(\x0b\x32/.com.mammut.interpreter.UnsupportedFeatureProtoH\x00\x12\x43\n\rcompilerPhase\x18\x0c \x01(\x0b\x32*.com.mammut.interpreter.CompilerPhaseProtoH\x00\x12I\n\x10interpreterPhase\x18\r \x01(\x0b\x32-.com.mammut.interpreter.InterpreterPhaseProtoH\x00\x12S\n\x15propertyValueNotFound\x18\x0f \x01(\x0b\x32\x32.com.mammut.interpreter.PropertyValueNotFoundProtoH\x00\x12K\n\x11\x65\x64geValueNotFound\x18\x10 \x01(\x0b\x32..com.mammut.interpreter.EdgeValueNotFoundProtoH\x00\x12S\n\x15unknownPropertyOrEdge\x18\x11 \x01(\x0b\x32\x32.com.mammut.interpreter.UnknownPropertyOrEdgeProtoH\x00\x12Q\n\x14ontologyTypeNotFound\x18\x12 \x01(\x0b\x32\x31.com.mammut.interpreter.OntologyTypeNotFoundProtoH\x00\x12G\n\x0fvertexUndefined\x18\x13 \x01(\x0b\x32,.com.mammut.interpreter.VertexUndefinedProtoH\x00\x12\x43\n\rsheetNotFound\x18\x16 \x01(\x0b\x32*.com.mammut.interpreter.SheetNotFoundProtoH\x00\x12Q\n\x14reservedKeywordFound\x18\x17 \x01(\x0b\x32\x31.com.mammut.interpreter.ReservedKeywordFoundProtoH\x00\x12]\n\x1apropertyDescriptorNotFound\x18\x18 \x01(\x0b\x32\x37.com.mammut.interpreter.PropertyDescriptorNotFoundProtoH\x00\x12U\n\x16\x65\x64geDescriptorNotFound\x18\x1b \x01(\x0b\x32\x33.com.mammut.interpreter.EdgeDescriptorNotFoundProtoH\x00\x12W\n\x17unexpectedCompilerError\x18\x19 \x01(\x0b\x32\x34.com.mammut.interpreter.UnexpectedCompilerErrorProtoH\x00\x12G\n\x0fobjectUndefined\x18\x1a \x01(\x0b\x32,.com.mammut.interpreter.ObjectUndefinedProtoH\x00\x12h\n\"defaultVertexPropertyNotFoundProto\x18\x1c \x01(\x0b\x32:.com.mammut.interpreter.DefaultVertexPropertyNotFoundProtoH\x00\x12l\n$multipleDefaultVertexPropertiesProto\x18\x1d \x01(\x0b\x32<.com.mammut.interpreter.MultipleDefaultVertexPropertiesProtoH\x00\"\xe0\x0b\n\x05Types\x12\x1a\n\x16ImportantMessageOutput\x10\x00\x12\x1c\n\x18UnsupportedFeatureOutput\x10\x01\x12\x1a\n\x16GettingCorpusMapOutput\x10\x02\x12\x19\n\x15GettingOntologyOutput\x10\x03\x12\x1b\n\x17\x45xpandingOntologyOutput\x10\x04\x12\x1d\n\x19\x43omputingEntryPointOutput\x10\x05\x12\"\n\x1eGettingOntologyInstancesOutput\x10\x06\x12)\n%LoadingInstancesToWorkingMemoryOutput\x10\x07\x12#\n\x1fGeneratingOntologyClassesOutput\x10\x08\x12\x1a\n\x16GettingVariablesOutput\x10\t\x12\x1a\n\x16GettingSceneriesOutput\x10\n\x12\x1b\n\x17GettingExtensionsOutput\x10\x0b\x12\x1c\n\x18\x43ompilingVariablesOutput\x10\x0c\x12\x1c\n\x18\x43ompilingSceneriesOutput\x10\r\x12\x1d\n\x19\x43ompilingExtensionsOutput\x10\x0e\x12$\n GroupingCompiledExtensionsOutput\x10\x0f\x12#\n\x1fGroupingCompiledSceneriesOutput\x10\x10\x12$\n CompilingInformationTuplesOutput\x10\x11\x12\x16\n\x12StartPlayingOutput\x10\x12\x12!\n\x1dUnpreparedMammutDetectedOuput\x10\x13\x12\'\n#ResolvingMissingIterableIndexOutput\x10\x14\x12\"\n\x1eResolvingMissingVertexIdOutput\x10\x15\x12\x31\n-ResolvingMissingVertexIdForSetOperationOutput\x10\x16\x12(\n$ResolvingOverrideValueConflictOutput\x10\x17\x12\x1c\n\x18LtBBInterpretationOutput\x10\x18\x12$\n LtBBDesambiguatorExpansionOutput\x10\x19\x12\x18\n\x14\x41tCTGenerationOutput\x10\x1a\x12#\n\x1f\x41tCTDesambiguatorChoosingOutput\x10\x1b\x12#\n\x1fGettingIssueChannelOutputOutput\x10\x1c\x12\x1c\n\x18GettingThinkStatusOutput\x10\x1d\x12\x1f\n\x1bPropertyValueNotFoundOutput\x10\x1f\x12\x1f\n\x1bUnknownPropertyOrEdgeOutput\x10 \x12\x1e\n\x1aOntologyTypeNotFoundOutput\x10!\x12\x19\n\x15VertexUndefinedOutput\x10\"\x12\x1b\n\x17VariableUndefinedOutput\x10#\x12\x18\n\x14\x46ieldUndefinedOutput\x10$\x12\x1b\n\x17\x45\x64geValueNotFoundOutput\x10%\x12\x11\n\rSuccessOutput\x10&\x12\x11\n\rFailureOutput\x10\'\x12\x11\n\rSheetNotFound\x10(\x12\x1e\n\x1aReservedKeywordFoundOutput\x10)\x12$\n PropertyDescriptorNotFoundOutput\x10*\x12\x1a\n\x16\x45\x64geDescriptorNotFound\x10\x34\x12\x1b\n\x17UnexpectedCompilerError\x10\x32\x12\x19\n\x15ObjectUndefinedOutput\x10\x33\x12!\n\x1d\x44\x65\x66\x61ultVertexPropertyNotFound\x10\x37\x12#\n\x1fMultipleDefaultVertexProperties\x10\x38\"O\n\x06Levels\x12\x07\n\x03\x41ll\x10\x00\x12\t\n\x05Trace\x10\x01\x12\t\n\x05\x44\x65\x62ug\x10\x02\x12\x08\n\x04Info\x10\x03\x12\x08\n\x04Warn\x10\x04\x12\t\n\x05\x45rror\x10\x05\x12\x07\n\x03Off\x10\x06\"\xa5\x06\n\x05Phase\x12\x10\n\x0cWithoutPhase\x10\x00\x12\x14\n\x10GettingCorpusMap\x10\x01\x12\x13\n\x0fGettingOntology\x10\x02\x12\x15\n\x11\x45xpandingOntology\x10\x03\x12\x17\n\x13\x43omputingEntrypoint\x10\x04\x12\x1c\n\x18GettingOntologyInstances\x10\x05\x12#\n\x1fLoadingInstancesToWorkingMemory\x10\x06\x12\x1d\n\x19GeneratingOntologyClasses\x10\x07\x12\x14\n\x10GettingVariables\x10\x08\x12\x14\n\x10GettingSceneries\x10\t\x12\x15\n\x11GettingExtensions\x10\n\x12\x16\n\x12\x43ompilingVariables\x10\x0b\x12\x16\n\x12\x43ompilingSceneries\x10\x0c\x12\x17\n\x13\x43ompilingExtensions\x10\r\x12\x1e\n\x1aGroupingCompiledExtensions\x10\x0e\x12\x1d\n\x19GroupingCompiledSceneries\x10\x0f\x12\x1e\n\x1a\x43ompilingInformationTuples\x10\x10\x12\x10\n\x0cStartPlaying\x10\x1e\x12\x1c\n\x18UnpreparedMammutDetected\x10\x1f\x12!\n\x1dResolvingMissingIterableIndex\x10 \x12\x1c\n\x18ResolvingMissingVertexId\x10!\x12+\n\'ResolvingMissingVertexIdForSetOperation\x10\"\x12\"\n\x1eResolvingOverrideValueConflict\x10#\x12\x16\n\x12LtBBInterpretation\x10$\x12\x1e\n\x1aLtBBDesambiguatorExpansion\x10%\x12\x12\n\x0e\x41tCTGeneration\x10&\x12\x1d\n\x19\x41tCTDesambiguatorChoosing\x10\'\x12\x1d\n\x19GettingIssueChannelOutput\x10(\x12\x16\n\x12GettingThinkStatus\x10)\"T\n\nCorpusType\x12\x15\n\x11WithoutCorpusType\x10\x00\x12\n\n\x06Mammut\x10\x01\x12\x0b\n\x07\x43rawler\x10\x02\x12\x0b\n\x07Natural\x10\x03\x12\t\n\x05Issue\x10\x04\x42\x0f\n\rmessage_oneof*\x91\x01\n\x0e\x45rrorComponent\x12\x0f\n\x0bKttCompiler\x10\x00\x12\x17\n\x13KttConversationtime\x10\x01\x12\r\n\tKttMemory\x10\x02\x12\x0f\n\x0bKttOntology\x10\x03\x12\x12\n\x0eKttInterpreter\x10\x04\x12\x14\n\x10KttGoogleStorage\x10\x05\x12\x0b\n\x07KttRoot\x10\x06\x42\x17\x42\x15InterpreterMainProtosb\x06proto3')
)

_ERRORCOMPONENT = _descriptor.EnumDescriptor(
  name='ErrorComponent',
  full_name='com.mammut.interpreter.ErrorComponent',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='KttCompiler', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KttConversationtime', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KttMemory', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KttOntology', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KttInterpreter', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KttGoogleStorage', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KttRoot', index=6, number=6,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=6500,
  serialized_end=6645,
)
_sym_db.RegisterEnumDescriptor(_ERRORCOMPONENT)

ErrorComponent = enum_type_wrapper.EnumTypeWrapper(_ERRORCOMPONENT)
KttCompiler = 0
KttConversationtime = 1
KttMemory = 2
KttOntology = 3
KttInterpreter = 4
KttGoogleStorage = 5
KttRoot = 6


_INTERPRETERMAINOUTPUT_TYPES = _descriptor.EnumDescriptor(
  name='Types',
  full_name='com.mammut.interpreter.InterpreterMainOutput.Types',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ImportantMessageOutput', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UnsupportedFeatureOutput', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingCorpusMapOutput', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingOntologyOutput', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ExpandingOntologyOutput', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ComputingEntryPointOutput', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingOntologyInstancesOutput', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LoadingInstancesToWorkingMemoryOutput', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GeneratingOntologyClassesOutput', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingVariablesOutput', index=9, number=9,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingSceneriesOutput', index=10, number=10,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingExtensionsOutput', index=11, number=11,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CompilingVariablesOutput', index=12, number=12,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CompilingSceneriesOutput', index=13, number=13,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CompilingExtensionsOutput', index=14, number=14,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GroupingCompiledExtensionsOutput', index=15, number=15,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GroupingCompiledSceneriesOutput', index=16, number=16,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CompilingInformationTuplesOutput', index=17, number=17,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='StartPlayingOutput', index=18, number=18,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UnpreparedMammutDetectedOuput', index=19, number=19,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ResolvingMissingIterableIndexOutput', index=20, number=20,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ResolvingMissingVertexIdOutput', index=21, number=21,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ResolvingMissingVertexIdForSetOperationOutput', index=22, number=22,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ResolvingOverrideValueConflictOutput', index=23, number=23,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LtBBInterpretationOutput', index=24, number=24,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LtBBDesambiguatorExpansionOutput', index=25, number=25,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AtCTGenerationOutput', index=26, number=26,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AtCTDesambiguatorChoosingOutput', index=27, number=27,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingIssueChannelOutputOutput', index=28, number=28,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingThinkStatusOutput', index=29, number=29,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PropertyValueNotFoundOutput', index=30, number=31,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UnknownPropertyOrEdgeOutput', index=31, number=32,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OntologyTypeNotFoundOutput', index=32, number=33,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VertexUndefinedOutput', index=33, number=34,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VariableUndefinedOutput', index=34, number=35,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FieldUndefinedOutput', index=35, number=36,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EdgeValueNotFoundOutput', index=36, number=37,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SuccessOutput', index=37, number=38,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FailureOutput', index=38, number=39,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SheetNotFound', index=39, number=40,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ReservedKeywordFoundOutput', index=40, number=41,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PropertyDescriptorNotFoundOutput', index=41, number=42,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EdgeDescriptorNotFound', index=42, number=52,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UnexpectedCompilerError', index=43, number=50,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ObjectUndefinedOutput', index=44, number=51,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DefaultVertexPropertyNotFound', index=45, number=55,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MultipleDefaultVertexProperties', index=46, number=56,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4001,
  serialized_end=5505,
)
_sym_db.RegisterEnumDescriptor(_INTERPRETERMAINOUTPUT_TYPES)

_INTERPRETERMAINOUTPUT_LEVELS = _descriptor.EnumDescriptor(
  name='Levels',
  full_name='com.mammut.interpreter.InterpreterMainOutput.Levels',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='All', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Trace', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Debug', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Info', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Warn', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Error', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Off', index=6, number=6,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=5507,
  serialized_end=5586,
)
_sym_db.RegisterEnumDescriptor(_INTERPRETERMAINOUTPUT_LEVELS)

_INTERPRETERMAINOUTPUT_PHASE = _descriptor.EnumDescriptor(
  name='Phase',
  full_name='com.mammut.interpreter.InterpreterMainOutput.Phase',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='WithoutPhase', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingCorpusMap', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingOntology', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ExpandingOntology', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ComputingEntrypoint', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingOntologyInstances', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LoadingInstancesToWorkingMemory', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GeneratingOntologyClasses', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingVariables', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingSceneries', index=9, number=9,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingExtensions', index=10, number=10,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CompilingVariables', index=11, number=11,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CompilingSceneries', index=12, number=12,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CompilingExtensions', index=13, number=13,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GroupingCompiledExtensions', index=14, number=14,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GroupingCompiledSceneries', index=15, number=15,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CompilingInformationTuples', index=16, number=16,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='StartPlaying', index=17, number=30,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UnpreparedMammutDetected', index=18, number=31,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ResolvingMissingIterableIndex', index=19, number=32,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ResolvingMissingVertexId', index=20, number=33,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ResolvingMissingVertexIdForSetOperation', index=21, number=34,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ResolvingOverrideValueConflict', index=22, number=35,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LtBBInterpretation', index=23, number=36,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LtBBDesambiguatorExpansion', index=24, number=37,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AtCTGeneration', index=25, number=38,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AtCTDesambiguatorChoosing', index=26, number=39,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingIssueChannelOutput', index=27, number=40,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GettingThinkStatus', index=28, number=41,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=5589,
  serialized_end=6394,
)
_sym_db.RegisterEnumDescriptor(_INTERPRETERMAINOUTPUT_PHASE)

_INTERPRETERMAINOUTPUT_CORPUSTYPE = _descriptor.EnumDescriptor(
  name='CorpusType',
  full_name='com.mammut.interpreter.InterpreterMainOutput.CorpusType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='WithoutCorpusType', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Mammut', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Crawler', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Natural', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Issue', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=6396,
  serialized_end=6480,
)
_sym_db.RegisterEnumDescriptor(_INTERPRETERMAINOUTPUT_CORPUSTYPE)


_INTERPRETERMAININPUT = _descriptor.Descriptor(
  name='InterpreterMainInput',
  full_name='com.mammut.interpreter.InterpreterMainInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mammutId', full_name='com.mammut.interpreter.InterpreterMainInput.mammutId', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='slideId', full_name='com.mammut.interpreter.InterpreterMainInput.slideId', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='executeSamplingCorpus', full_name='com.mammut.interpreter.InterpreterMainInput.executeSamplingCorpus', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='corpusIds', full_name='com.mammut.interpreter.InterpreterMainInput.corpusIds', index=3,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadataTokenSeparator', full_name='com.mammut.interpreter.InterpreterMainInput.metadataTokenSeparator', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='regionalSettings', full_name='com.mammut.interpreter.InterpreterMainInput.regionalSettings', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='userId', full_name='com.mammut.interpreter.InterpreterMainInput.userId', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='roomId', full_name='com.mammut.interpreter.InterpreterMainInput.roomId', index=7,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mammutLoggerId', full_name='com.mammut.interpreter.InterpreterMainInput.mammutLoggerId', index=8,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=55,
  serialized_end=276,
)


_IMPORTANTMESSAGEPROTO = _descriptor.Descriptor(
  name='ImportantMessageProto',
  full_name='com.mammut.interpreter.ImportantMessageProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=278,
  serialized_end=301,
)


_UNSUPPORTEDFEATUREPROTO = _descriptor.Descriptor(
  name='UnsupportedFeatureProto',
  full_name='com.mammut.interpreter.UnsupportedFeatureProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.UnsupportedFeatureProto.errorComponent', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=303,
  serialized_end=392,
)


_COMPILERPHASEPROTO = _descriptor.Descriptor(
  name='CompilerPhaseProto',
  full_name='com.mammut.interpreter.CompilerPhaseProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extraInfo', full_name='com.mammut.interpreter.CompilerPhaseProto.extraInfo', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=394,
  serialized_end=433,
)


_INTERPRETERPHASEPROTO = _descriptor.Descriptor(
  name='InterpreterPhaseProto',
  full_name='com.mammut.interpreter.InterpreterPhaseProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='roomId', full_name='com.mammut.interpreter.InterpreterPhaseProto.roomId', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extraInfo', full_name='com.mammut.interpreter.InterpreterPhaseProto.extraInfo', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=435,
  serialized_end=493,
)


_PROPERTYDESCRIPTORNOTFOUNDPROTO = _descriptor.Descriptor(
  name='PropertyDescriptorNotFoundProto',
  full_name='com.mammut.interpreter.PropertyDescriptorNotFoundProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='propertyName', full_name='com.mammut.interpreter.PropertyDescriptorNotFoundProto.propertyName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.PropertyDescriptorNotFoundProto.errorComponent', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=495,
  serialized_end=614,
)


_EDGEDESCRIPTORNOTFOUNDPROTO = _descriptor.Descriptor(
  name='EdgeDescriptorNotFoundProto',
  full_name='com.mammut.interpreter.EdgeDescriptorNotFoundProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='edgeName', full_name='com.mammut.interpreter.EdgeDescriptorNotFoundProto.edgeName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.EdgeDescriptorNotFoundProto.errorComponent', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=616,
  serialized_end=727,
)


_PROPERTYVALUENOTFOUNDPROTO = _descriptor.Descriptor(
  name='PropertyValueNotFoundProto',
  full_name='com.mammut.interpreter.PropertyValueNotFoundProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='propertyName', full_name='com.mammut.interpreter.PropertyValueNotFoundProto.propertyName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vertexName', full_name='com.mammut.interpreter.PropertyValueNotFoundProto.vertexName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sheetId', full_name='com.mammut.interpreter.PropertyValueNotFoundProto.sheetId', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.PropertyValueNotFoundProto.errorComponent', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=730,
  serialized_end=881,
)


_EDGEVALUENOTFOUNDPROTO = _descriptor.Descriptor(
  name='EdgeValueNotFoundProto',
  full_name='com.mammut.interpreter.EdgeValueNotFoundProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='traverse', full_name='com.mammut.interpreter.EdgeValueNotFoundProto.traverse', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.EdgeValueNotFoundProto.errorComponent', index=1,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=883,
  serialized_end=989,
)


_UNKNOWNPROPERTYOREDGEPROTO = _descriptor.Descriptor(
  name='UnknownPropertyOrEdgeProto',
  full_name='com.mammut.interpreter.UnknownPropertyOrEdgeProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='elementName', full_name='com.mammut.interpreter.UnknownPropertyOrEdgeProto.elementName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vertexName', full_name='com.mammut.interpreter.UnknownPropertyOrEdgeProto.vertexName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.UnknownPropertyOrEdgeProto.errorComponent', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=992,
  serialized_end=1125,
)


_ONTOLOGYTYPENOTFOUNDPROTO = _descriptor.Descriptor(
  name='OntologyTypeNotFoundProto',
  full_name='com.mammut.interpreter.OntologyTypeNotFoundProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='typeName', full_name='com.mammut.interpreter.OntologyTypeNotFoundProto.typeName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.OntologyTypeNotFoundProto.errorComponent', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1127,
  serialized_end=1236,
)


_VERTEXUNDEFINEDPROTO = _descriptor.Descriptor(
  name='VertexUndefinedProto',
  full_name='com.mammut.interpreter.VertexUndefinedProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='srcVertex', full_name='com.mammut.interpreter.VertexUndefinedProto.srcVertex', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='srcEdge', full_name='com.mammut.interpreter.VertexUndefinedProto.srcEdge', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sheetId', full_name='com.mammut.interpreter.VertexUndefinedProto.sheetId', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.VertexUndefinedProto.errorComponent', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1239,
  serialized_end=1378,
)


_SHEETNOTFOUNDPROTO = _descriptor.Descriptor(
  name='SheetNotFoundProto',
  full_name='com.mammut.interpreter.SheetNotFoundProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sheetId', full_name='com.mammut.interpreter.SheetNotFoundProto.sheetId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.SheetNotFoundProto.errorComponent', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1380,
  serialized_end=1481,
)


_RESERVEDKEYWORDFOUNDPROTO = _descriptor.Descriptor(
  name='ReservedKeywordFoundProto',
  full_name='com.mammut.interpreter.ReservedKeywordFoundProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='reserverdWord', full_name='com.mammut.interpreter.ReservedKeywordFoundProto.reserverdWord', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.ReservedKeywordFoundProto.errorComponent', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1483,
  serialized_end=1597,
)


_UNEXPECTEDCOMPILERERRORPROTO = _descriptor.Descriptor(
  name='UnexpectedCompilerErrorProto',
  full_name='com.mammut.interpreter.UnexpectedCompilerErrorProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='causeMessage', full_name='com.mammut.interpreter.UnexpectedCompilerErrorProto.causeMessage', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.UnexpectedCompilerErrorProto.errorComponent', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1599,
  serialized_end=1715,
)


_OBJECTUNDEFINEDPROTO = _descriptor.Descriptor(
  name='ObjectUndefinedProto',
  full_name='com.mammut.interpreter.ObjectUndefinedProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectElement', full_name='com.mammut.interpreter.ObjectUndefinedProto.objectElement', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='packageElement', full_name='com.mammut.interpreter.ObjectUndefinedProto.packageElement', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.ObjectUndefinedProto.errorComponent', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1718,
  serialized_end=1851,
)


_DEFAULTVERTEXPROPERTYNOTFOUNDPROTO = _descriptor.Descriptor(
  name='DefaultVertexPropertyNotFoundProto',
  full_name='com.mammut.interpreter.DefaultVertexPropertyNotFoundProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vertexName', full_name='com.mammut.interpreter.DefaultVertexPropertyNotFoundProto.vertexName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.DefaultVertexPropertyNotFoundProto.errorComponent', index=1,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1853,
  serialized_end=1973,
)


_MULTIPLEDEFAULTVERTEXPROPERTIESPROTO = _descriptor.Descriptor(
  name='MultipleDefaultVertexPropertiesProto',
  full_name='com.mammut.interpreter.MultipleDefaultVertexPropertiesProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vertexName', full_name='com.mammut.interpreter.MultipleDefaultVertexPropertiesProto.vertexName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errorComponent', full_name='com.mammut.interpreter.MultipleDefaultVertexPropertiesProto.errorComponent', index=1,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1975,
  serialized_end=2097,
)


_INTERPRETERMAINOUTPUT = _descriptor.Descriptor(
  name='InterpreterMainOutput',
  full_name='com.mammut.interpreter.InterpreterMainOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='com.mammut.interpreter.InterpreterMainOutput.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unknownMammut', full_name='com.mammut.interpreter.InterpreterMainOutput.unknownMammut', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unknownRoom', full_name='com.mammut.interpreter.InterpreterMainOutput.unknownRoom', index=2,
      number=102, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unknownUser', full_name='com.mammut.interpreter.InterpreterMainOutput.unknownUser', index=3,
      number=103, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mammutId', full_name='com.mammut.interpreter.InterpreterMainOutput.mammutId', index=4,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='roomId', full_name='com.mammut.interpreter.InterpreterMainOutput.roomId', index=5,
      number=100, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='userId', full_name='com.mammut.interpreter.InterpreterMainOutput.userId', index=6,
      number=101, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='level', full_name='com.mammut.interpreter.InterpreterMainOutput.level', index=7,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='com.mammut.interpreter.InterpreterMainOutput.message', index=8,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='throwable', full_name='com.mammut.interpreter.InterpreterMainOutput.throwable', index=9,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='phase', full_name='com.mammut.interpreter.InterpreterMainOutput.phase', index=10,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='corpusCompilationId', full_name='com.mammut.interpreter.InterpreterMainOutput.corpusCompilationId', index=11,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='corpusType', full_name='com.mammut.interpreter.InterpreterMainOutput.corpusType', index=12,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='importantMessage', full_name='com.mammut.interpreter.InterpreterMainOutput.importantMessage', index=13,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unsupportedFeature', full_name='com.mammut.interpreter.InterpreterMainOutput.unsupportedFeature', index=14,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='compilerPhase', full_name='com.mammut.interpreter.InterpreterMainOutput.compilerPhase', index=15,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='interpreterPhase', full_name='com.mammut.interpreter.InterpreterMainOutput.interpreterPhase', index=16,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='propertyValueNotFound', full_name='com.mammut.interpreter.InterpreterMainOutput.propertyValueNotFound', index=17,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='edgeValueNotFound', full_name='com.mammut.interpreter.InterpreterMainOutput.edgeValueNotFound', index=18,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unknownPropertyOrEdge', full_name='com.mammut.interpreter.InterpreterMainOutput.unknownPropertyOrEdge', index=19,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ontologyTypeNotFound', full_name='com.mammut.interpreter.InterpreterMainOutput.ontologyTypeNotFound', index=20,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vertexUndefined', full_name='com.mammut.interpreter.InterpreterMainOutput.vertexUndefined', index=21,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sheetNotFound', full_name='com.mammut.interpreter.InterpreterMainOutput.sheetNotFound', index=22,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reservedKeywordFound', full_name='com.mammut.interpreter.InterpreterMainOutput.reservedKeywordFound', index=23,
      number=23, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='propertyDescriptorNotFound', full_name='com.mammut.interpreter.InterpreterMainOutput.propertyDescriptorNotFound', index=24,
      number=24, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='edgeDescriptorNotFound', full_name='com.mammut.interpreter.InterpreterMainOutput.edgeDescriptorNotFound', index=25,
      number=27, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unexpectedCompilerError', full_name='com.mammut.interpreter.InterpreterMainOutput.unexpectedCompilerError', index=26,
      number=25, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectUndefined', full_name='com.mammut.interpreter.InterpreterMainOutput.objectUndefined', index=27,
      number=26, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='defaultVertexPropertyNotFoundProto', full_name='com.mammut.interpreter.InterpreterMainOutput.defaultVertexPropertyNotFoundProto', index=28,
      number=28, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='multipleDefaultVertexPropertiesProto', full_name='com.mammut.interpreter.InterpreterMainOutput.multipleDefaultVertexPropertiesProto', index=29,
      number=29, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _INTERPRETERMAINOUTPUT_TYPES,
    _INTERPRETERMAINOUTPUT_LEVELS,
    _INTERPRETERMAINOUTPUT_PHASE,
    _INTERPRETERMAINOUTPUT_CORPUSTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='message_oneof', full_name='com.mammut.interpreter.InterpreterMainOutput.message_oneof',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=2100,
  serialized_end=6497,
)

_UNSUPPORTEDFEATUREPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_PROPERTYDESCRIPTORNOTFOUNDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_EDGEDESCRIPTORNOTFOUNDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_PROPERTYVALUENOTFOUNDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_EDGEVALUENOTFOUNDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_UNKNOWNPROPERTYOREDGEPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_ONTOLOGYTYPENOTFOUNDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_VERTEXUNDEFINEDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_SHEETNOTFOUNDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_RESERVEDKEYWORDFOUNDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_UNEXPECTEDCOMPILERERRORPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_OBJECTUNDEFINEDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_DEFAULTVERTEXPROPERTYNOTFOUNDPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_MULTIPLEDEFAULTVERTEXPROPERTIESPROTO.fields_by_name['errorComponent'].enum_type = _ERRORCOMPONENT
_INTERPRETERMAINOUTPUT.fields_by_name['type'].enum_type = _INTERPRETERMAINOUTPUT_TYPES
_INTERPRETERMAINOUTPUT.fields_by_name['level'].enum_type = _INTERPRETERMAINOUTPUT_LEVELS
_INTERPRETERMAINOUTPUT.fields_by_name['phase'].enum_type = _INTERPRETERMAINOUTPUT_PHASE
_INTERPRETERMAINOUTPUT.fields_by_name['corpusType'].enum_type = _INTERPRETERMAINOUTPUT_CORPUSTYPE
_INTERPRETERMAINOUTPUT.fields_by_name['importantMessage'].message_type = _IMPORTANTMESSAGEPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['unsupportedFeature'].message_type = _UNSUPPORTEDFEATUREPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['compilerPhase'].message_type = _COMPILERPHASEPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['interpreterPhase'].message_type = _INTERPRETERPHASEPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['propertyValueNotFound'].message_type = _PROPERTYVALUENOTFOUNDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['edgeValueNotFound'].message_type = _EDGEVALUENOTFOUNDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['unknownPropertyOrEdge'].message_type = _UNKNOWNPROPERTYOREDGEPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['ontologyTypeNotFound'].message_type = _ONTOLOGYTYPENOTFOUNDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['vertexUndefined'].message_type = _VERTEXUNDEFINEDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['sheetNotFound'].message_type = _SHEETNOTFOUNDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['reservedKeywordFound'].message_type = _RESERVEDKEYWORDFOUNDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['propertyDescriptorNotFound'].message_type = _PROPERTYDESCRIPTORNOTFOUNDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['edgeDescriptorNotFound'].message_type = _EDGEDESCRIPTORNOTFOUNDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['unexpectedCompilerError'].message_type = _UNEXPECTEDCOMPILERERRORPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['objectUndefined'].message_type = _OBJECTUNDEFINEDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['defaultVertexPropertyNotFoundProto'].message_type = _DEFAULTVERTEXPROPERTYNOTFOUNDPROTO
_INTERPRETERMAINOUTPUT.fields_by_name['multipleDefaultVertexPropertiesProto'].message_type = _MULTIPLEDEFAULTVERTEXPROPERTIESPROTO
_INTERPRETERMAINOUTPUT_TYPES.containing_type = _INTERPRETERMAINOUTPUT
_INTERPRETERMAINOUTPUT_LEVELS.containing_type = _INTERPRETERMAINOUTPUT
_INTERPRETERMAINOUTPUT_PHASE.containing_type = _INTERPRETERMAINOUTPUT
_INTERPRETERMAINOUTPUT_CORPUSTYPE.containing_type = _INTERPRETERMAINOUTPUT
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['importantMessage'])
_INTERPRETERMAINOUTPUT.fields_by_name['importantMessage'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['unsupportedFeature'])
_INTERPRETERMAINOUTPUT.fields_by_name['unsupportedFeature'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['compilerPhase'])
_INTERPRETERMAINOUTPUT.fields_by_name['compilerPhase'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['interpreterPhase'])
_INTERPRETERMAINOUTPUT.fields_by_name['interpreterPhase'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['propertyValueNotFound'])
_INTERPRETERMAINOUTPUT.fields_by_name['propertyValueNotFound'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['edgeValueNotFound'])
_INTERPRETERMAINOUTPUT.fields_by_name['edgeValueNotFound'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['unknownPropertyOrEdge'])
_INTERPRETERMAINOUTPUT.fields_by_name['unknownPropertyOrEdge'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['ontologyTypeNotFound'])
_INTERPRETERMAINOUTPUT.fields_by_name['ontologyTypeNotFound'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['vertexUndefined'])
_INTERPRETERMAINOUTPUT.fields_by_name['vertexUndefined'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['sheetNotFound'])
_INTERPRETERMAINOUTPUT.fields_by_name['sheetNotFound'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['reservedKeywordFound'])
_INTERPRETERMAINOUTPUT.fields_by_name['reservedKeywordFound'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['propertyDescriptorNotFound'])
_INTERPRETERMAINOUTPUT.fields_by_name['propertyDescriptorNotFound'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['edgeDescriptorNotFound'])
_INTERPRETERMAINOUTPUT.fields_by_name['edgeDescriptorNotFound'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['unexpectedCompilerError'])
_INTERPRETERMAINOUTPUT.fields_by_name['unexpectedCompilerError'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['objectUndefined'])
_INTERPRETERMAINOUTPUT.fields_by_name['objectUndefined'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['defaultVertexPropertyNotFoundProto'])
_INTERPRETERMAINOUTPUT.fields_by_name['defaultVertexPropertyNotFoundProto'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
_INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof'].fields.append(
  _INTERPRETERMAINOUTPUT.fields_by_name['multipleDefaultVertexPropertiesProto'])
_INTERPRETERMAINOUTPUT.fields_by_name['multipleDefaultVertexPropertiesProto'].containing_oneof = _INTERPRETERMAINOUTPUT.oneofs_by_name['message_oneof']
DESCRIPTOR.message_types_by_name['InterpreterMainInput'] = _INTERPRETERMAININPUT
DESCRIPTOR.message_types_by_name['ImportantMessageProto'] = _IMPORTANTMESSAGEPROTO
DESCRIPTOR.message_types_by_name['UnsupportedFeatureProto'] = _UNSUPPORTEDFEATUREPROTO
DESCRIPTOR.message_types_by_name['CompilerPhaseProto'] = _COMPILERPHASEPROTO
DESCRIPTOR.message_types_by_name['InterpreterPhaseProto'] = _INTERPRETERPHASEPROTO
DESCRIPTOR.message_types_by_name['PropertyDescriptorNotFoundProto'] = _PROPERTYDESCRIPTORNOTFOUNDPROTO
DESCRIPTOR.message_types_by_name['EdgeDescriptorNotFoundProto'] = _EDGEDESCRIPTORNOTFOUNDPROTO
DESCRIPTOR.message_types_by_name['PropertyValueNotFoundProto'] = _PROPERTYVALUENOTFOUNDPROTO
DESCRIPTOR.message_types_by_name['EdgeValueNotFoundProto'] = _EDGEVALUENOTFOUNDPROTO
DESCRIPTOR.message_types_by_name['UnknownPropertyOrEdgeProto'] = _UNKNOWNPROPERTYOREDGEPROTO
DESCRIPTOR.message_types_by_name['OntologyTypeNotFoundProto'] = _ONTOLOGYTYPENOTFOUNDPROTO
DESCRIPTOR.message_types_by_name['VertexUndefinedProto'] = _VERTEXUNDEFINEDPROTO
DESCRIPTOR.message_types_by_name['SheetNotFoundProto'] = _SHEETNOTFOUNDPROTO
DESCRIPTOR.message_types_by_name['ReservedKeywordFoundProto'] = _RESERVEDKEYWORDFOUNDPROTO
DESCRIPTOR.message_types_by_name['UnexpectedCompilerErrorProto'] = _UNEXPECTEDCOMPILERERRORPROTO
DESCRIPTOR.message_types_by_name['ObjectUndefinedProto'] = _OBJECTUNDEFINEDPROTO
DESCRIPTOR.message_types_by_name['DefaultVertexPropertyNotFoundProto'] = _DEFAULTVERTEXPROPERTYNOTFOUNDPROTO
DESCRIPTOR.message_types_by_name['MultipleDefaultVertexPropertiesProto'] = _MULTIPLEDEFAULTVERTEXPROPERTIESPROTO
DESCRIPTOR.message_types_by_name['InterpreterMainOutput'] = _INTERPRETERMAINOUTPUT
DESCRIPTOR.enum_types_by_name['ErrorComponent'] = _ERRORCOMPONENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

InterpreterMainInput = _reflection.GeneratedProtocolMessageType('InterpreterMainInput', (_message.Message,), {
  'DESCRIPTOR' : _INTERPRETERMAININPUT,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.InterpreterMainInput)
  })
_sym_db.RegisterMessage(InterpreterMainInput)

ImportantMessageProto = _reflection.GeneratedProtocolMessageType('ImportantMessageProto', (_message.Message,), {
  'DESCRIPTOR' : _IMPORTANTMESSAGEPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.ImportantMessageProto)
  })
_sym_db.RegisterMessage(ImportantMessageProto)

UnsupportedFeatureProto = _reflection.GeneratedProtocolMessageType('UnsupportedFeatureProto', (_message.Message,), {
  'DESCRIPTOR' : _UNSUPPORTEDFEATUREPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.UnsupportedFeatureProto)
  })
_sym_db.RegisterMessage(UnsupportedFeatureProto)

CompilerPhaseProto = _reflection.GeneratedProtocolMessageType('CompilerPhaseProto', (_message.Message,), {
  'DESCRIPTOR' : _COMPILERPHASEPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.CompilerPhaseProto)
  })
_sym_db.RegisterMessage(CompilerPhaseProto)

InterpreterPhaseProto = _reflection.GeneratedProtocolMessageType('InterpreterPhaseProto', (_message.Message,), {
  'DESCRIPTOR' : _INTERPRETERPHASEPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.InterpreterPhaseProto)
  })
_sym_db.RegisterMessage(InterpreterPhaseProto)

PropertyDescriptorNotFoundProto = _reflection.GeneratedProtocolMessageType('PropertyDescriptorNotFoundProto', (_message.Message,), {
  'DESCRIPTOR' : _PROPERTYDESCRIPTORNOTFOUNDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.PropertyDescriptorNotFoundProto)
  })
_sym_db.RegisterMessage(PropertyDescriptorNotFoundProto)

EdgeDescriptorNotFoundProto = _reflection.GeneratedProtocolMessageType('EdgeDescriptorNotFoundProto', (_message.Message,), {
  'DESCRIPTOR' : _EDGEDESCRIPTORNOTFOUNDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.EdgeDescriptorNotFoundProto)
  })
_sym_db.RegisterMessage(EdgeDescriptorNotFoundProto)

PropertyValueNotFoundProto = _reflection.GeneratedProtocolMessageType('PropertyValueNotFoundProto', (_message.Message,), {
  'DESCRIPTOR' : _PROPERTYVALUENOTFOUNDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.PropertyValueNotFoundProto)
  })
_sym_db.RegisterMessage(PropertyValueNotFoundProto)

EdgeValueNotFoundProto = _reflection.GeneratedProtocolMessageType('EdgeValueNotFoundProto', (_message.Message,), {
  'DESCRIPTOR' : _EDGEVALUENOTFOUNDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.EdgeValueNotFoundProto)
  })
_sym_db.RegisterMessage(EdgeValueNotFoundProto)

UnknownPropertyOrEdgeProto = _reflection.GeneratedProtocolMessageType('UnknownPropertyOrEdgeProto', (_message.Message,), {
  'DESCRIPTOR' : _UNKNOWNPROPERTYOREDGEPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.UnknownPropertyOrEdgeProto)
  })
_sym_db.RegisterMessage(UnknownPropertyOrEdgeProto)

OntologyTypeNotFoundProto = _reflection.GeneratedProtocolMessageType('OntologyTypeNotFoundProto', (_message.Message,), {
  'DESCRIPTOR' : _ONTOLOGYTYPENOTFOUNDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.OntologyTypeNotFoundProto)
  })
_sym_db.RegisterMessage(OntologyTypeNotFoundProto)

VertexUndefinedProto = _reflection.GeneratedProtocolMessageType('VertexUndefinedProto', (_message.Message,), {
  'DESCRIPTOR' : _VERTEXUNDEFINEDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.VertexUndefinedProto)
  })
_sym_db.RegisterMessage(VertexUndefinedProto)

SheetNotFoundProto = _reflection.GeneratedProtocolMessageType('SheetNotFoundProto', (_message.Message,), {
  'DESCRIPTOR' : _SHEETNOTFOUNDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.SheetNotFoundProto)
  })
_sym_db.RegisterMessage(SheetNotFoundProto)

ReservedKeywordFoundProto = _reflection.GeneratedProtocolMessageType('ReservedKeywordFoundProto', (_message.Message,), {
  'DESCRIPTOR' : _RESERVEDKEYWORDFOUNDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.ReservedKeywordFoundProto)
  })
_sym_db.RegisterMessage(ReservedKeywordFoundProto)

UnexpectedCompilerErrorProto = _reflection.GeneratedProtocolMessageType('UnexpectedCompilerErrorProto', (_message.Message,), {
  'DESCRIPTOR' : _UNEXPECTEDCOMPILERERRORPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.UnexpectedCompilerErrorProto)
  })
_sym_db.RegisterMessage(UnexpectedCompilerErrorProto)

ObjectUndefinedProto = _reflection.GeneratedProtocolMessageType('ObjectUndefinedProto', (_message.Message,), {
  'DESCRIPTOR' : _OBJECTUNDEFINEDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.ObjectUndefinedProto)
  })
_sym_db.RegisterMessage(ObjectUndefinedProto)

DefaultVertexPropertyNotFoundProto = _reflection.GeneratedProtocolMessageType('DefaultVertexPropertyNotFoundProto', (_message.Message,), {
  'DESCRIPTOR' : _DEFAULTVERTEXPROPERTYNOTFOUNDPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.DefaultVertexPropertyNotFoundProto)
  })
_sym_db.RegisterMessage(DefaultVertexPropertyNotFoundProto)

MultipleDefaultVertexPropertiesProto = _reflection.GeneratedProtocolMessageType('MultipleDefaultVertexPropertiesProto', (_message.Message,), {
  'DESCRIPTOR' : _MULTIPLEDEFAULTVERTEXPROPERTIESPROTO,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.MultipleDefaultVertexPropertiesProto)
  })
_sym_db.RegisterMessage(MultipleDefaultVertexPropertiesProto)

InterpreterMainOutput = _reflection.GeneratedProtocolMessageType('InterpreterMainOutput', (_message.Message,), {
  'DESCRIPTOR' : _INTERPRETERMAINOUTPUT,
  '__module__' : 'apps.InterpreterMain_pb2'
  # @@protoc_insertion_point(class_scope:com.mammut.interpreter.InterpreterMainOutput)
  })
_sym_db.RegisterMessage(InterpreterMainOutput)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
