<!DOCTYPE html>
<html lang="es" dir="ltr"
      prefix="content: http://purl.org/rss/1.0/modules/content/ dc: http://purl.org/dc/terms/ foaf: http://xmlns.com/foaf/0.1/ og: http://ogp.me/ns# rdfs: http://www.w3.org/2000/01/rdf-schema# sioc: http://rdfs.org/sioc/ns# sioct: http://rdfs.org/sioc/types# skos: http://www.w3.org/2004/02/skos/core# xsd: http://www.w3.org/2001/XMLSchema#"
      class="js">
<head>
    <!--[if IE]><![endif]-->
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <link rel="shortcut icon" href="http://www.rae.es/favicon.ico" type="image/vnd.microsoft.icon">
    <meta name="viewport"
          content="width=device-width, initial-scale=1, maximum-scale=2, minimum-scale=1, user-scalable=yes">
    <title>Modelos de conjugación verbal | Real Academia Española</title>
    <link type="text/css" rel="stylesheet"
          href="//www.rae.es/sites/default/files/advagg_css/css__BJ6Ou6QsBRtnFTmxaakamOIS8n4QswDP2XnnZ1sxtaM__NBuvkP6eInGIkb1aJvUHx5PX79XApuxBDkk_77W5tYk__aA-3YXU9JXAjkjXGPmiBjCdM5FxsY2fQ9UKAxyZOoIU.css"
          media="all">
    <link type="text/css" rel="stylesheet"
          href="//www.rae.es/sites/default/files/advagg_css/css__Bqg1SDmp9CAe6XEIkqmrkRsGk0iHTZHCcB95hCAlYnY__pvxzJdEyVw_5TVP2BBc6RFQHZTlmfsgpKtyMTJgxw8o__aA-3YXU9JXAjkjXGPmiBjCdM5FxsY2fQ9UKAxyZOoIU.css"
          media="screen">
    <link type="text/css" rel="stylesheet"
          href="//www.rae.es/sites/default/files/advagg_css/css__6lu8wgjhjxYRNjZx4_BTjYJj1e7_BUZPuRH4jDmHIc4__gRx-B2Yt0SGUrwzi-Os1z8hYYx81CHSRgZWaQY2qC6Y__aA-3YXU9JXAjkjXGPmiBjCdM5FxsY2fQ9UKAxyZOoIU.css"
          media="all">
    <style type="text/css" media="all">
        /* <![CDATA[ */
        #sliding-popup.sliding-popup-bottom {
            background: #444444;
        }

        #sliding-popup .popup-content #popup-text h2, #sliding-popup .popup-content #popup-text p {
            color: #ffffff !important;
        }

        /* ]]> */
    </style>
    <link type="text/css" rel="stylesheet"
          href="//www.rae.es/sites/default/files/advagg_css/css__Phj1Zkvr2KK4-16u1LgD_a0o0hiKOlH6Kc7g8d3KX7Q__d0BQMfNR7Xrf-2blxY8d7wWZqsHNQSDMOOd8kD934Zo__aA-3YXU9JXAjkjXGPmiBjCdM5FxsY2fQ9UKAxyZOoIU.css"
          media="all">
    <script id="twitter-wjs" src="http://platform.twitter.com/widgets.js"></script>
    <script async="" src="//www.google-analytics.com/analytics.js"></script>
    <script type="text/javascript" src="http://w.sharethis.com/button/p.js" async=""></script>
    <script type="text/javascript" id="async-buttons" src="http://w.sharethis.com/button/async-buttons.js"></script>
    <script type="text/javascript" src="//www.rae.es/sites/all/libraries/respondjs/respond.min.js?ospwqt"></script>
    <script type="text/javascript"
            src="//www.rae.es/sites/default/files/advagg_js/js__qy9bvnaIRHylNicNIBiRsE91osNdfHwZ1PelmiSzsX0__pOfIUCJmYu4fnoXdOAgUHdHoVbXlojVNNx2h2ojwDWk__aA-3YXU9JXAjkjXGPmiBjCdM5FxsY2fQ9UKAxyZOoIU.js"></script>
    <script type="text/javascript">
        <!--//-->
        <![CDATA[//><!--
        var switchTo5x = true;
        //--><!
        ]]>
    </script>
    <script type="text/javascript" src="//w.sharethis.com/button/buttons.js"></script>
    <script type="text/javascript"
            src="//www.rae.es/if%20%28stLight%20%21%3D%3D%20undefined%29%20%7B%20stLight.options%28%7B%22publisher%22%3A%22dr-8656fd54-d140-dab7-5894-ab893540ae03%22%2C%22version%22%3A%225x%22%7D%29%3B%20%7D?ospwqt"></script>
    <link rel="stylesheet" type="text/css"
          href="http://w.sharethis.com/button/css/buttons.e80452d5e7cc382dad89d10f50bde247.css">
    <iframe id="stSegmentFrame" name="stSegmentFrame" frameborder="0" scrolling="no" width="0px" height="0px"
            sandbox="allow-scripts allow-same-origin" style="display:none;"></iframe>
    <script type="text/javascript"
            src="//www.rae.es/sites/default/files/advagg_js/js__b9ctWTDGHg52GUYO6n8BMGrkN9dAHKWCpNJqRJ5pNrI__2G5ANt_GY1Oib69FbLbDkcX1cfCK1M3AtMAVc-Uqo_U__aA-3YXU9JXAjkjXGPmiBjCdM5FxsY2fQ9UKAxyZOoIU.js"></script>
    <script type="text/javascript">
        <!--//-->
        <![CDATA[//><!--

        jQuery(document).ready(function ($) {
            $('#om-maximenu-rae-main-menu li.om-leaf .om-maximenu-content').removeClass('om-maximenu-content-nofade');
            //$('#om-maximenu-rae-main-menu li.om-leaf').hover(omFadeIn,omFadeOut); //native jquery
            $('#om-maximenu-rae-main-menu li.om-leaf').hoverIntent({
                over: omFadeIn,
                timeout: 100,
                out: omFadeOut
            });
            function omFadeIn() {
                $('.om-maximenu-content.closed', this).fadeIn();
            }

            function omFadeOut() {
                $('.om-maximenu-content.closed', this).fadeOut();
            }
        });

        //--><!
        ]]>
    </script>
    <script type="text/javascript">
        <!--//-->
        <![CDATA[//><!--

        jQuery(function ($) {
            $.each($('#om-menu-rae-main-menu .om-leaf'), function () {

                var active = $('.menu a', this).hasClass('active');
                var active_trail = $('.menu a', this).hasClass('active-trail');

                if ((active == true) || (active_trail == true)) {
                    $('#om-menu-rae-main-menu .om-leaf').removeClass('active-trail');
                    $(this).addClass('active-trail');
                }
            });
        });
        //--><!
        ]]>
    </script>
    <script type="text/javascript"
            src="//www.rae.es/sites/default/files/advagg_js/js__4kL8ApXEg_0m8qszFWSpTQwptF5k3N9qkm57JS1E2hA__avKI53-1z_1BNf3_XApChb2-lorFoOQub5c3Arvb7Zw__aA-3YXU9JXAjkjXGPmiBjCdM5FxsY2fQ9UKAxyZOoIU.js"></script>
    <script type="text/javascript">
        <!--//-->
        <![CDATA[//><!--
        jQuery.extend(Drupal.settings, {
            "basePath": "\/",
            "pathPrefix": "",
            "ajaxPageState": {
                "theme": "rae",
                "theme_token": "xB3FbuA_Q8zE76FqjL0fZhmW2be9-c8MEwyakv2_r1I",
                "css": {
                    "modules\/system\/system.base.css": 1,
                    "modules\/system\/system.menus.css": 1,
                    "modules\/system\/system.messages.css": 1,
                    "modules\/system\/system.theme.css": 1,
                    "sites\/all\/modules\/contrib\/tipsy\/stylesheets\/tipsy.css": 1,
                    "modules\/comment\/comment.css": 1,
                    "sites\/all\/modules\/contrib\/date\/date_api\/date.css": 1,
                    "sites\/all\/modules\/contrib\/domain\/domain_nav\/domain_nav.css": 1,
                    "modules\/field\/theme\/field.css": 1,
                    "modules\/node\/node.css": 1,
                    "sites\/all\/modules\/contrib\/om_maximenu\/css\/om_maximenu.css": 1,
                    "modules\/search\/search.css": 1,
                    "modules\/user\/user.css": 1,
                    "sites\/all\/modules\/contrib\/views\/css\/views.css": 1,
                    "sites\/all\/modules\/contrib\/ctools\/css\/ctools.css": 1,
                    "sites\/all\/modules\/contrib\/lightbox2\/css\/lightbox_alt.css": 1,
                    "sites\/all\/modules\/contrib\/om_maximenu\/skin\/no_style\/no_style.css": 1,
                    "0": 1,
                    "sites\/all\/modules\/contrib\/eu-cookie-compliance\/css\/eu_cookie_compliance.css": 1,
                    "sites\/all\/modules\/contrib\/responsive_menus\/styles\/responsive_menus_simple\/css\/responsive_menus_simple.css": 1,
                    "sites\/all\/themes\/omega\/alpha\/css\/alpha-reset.css": 1,
                    "sites\/all\/themes\/omega\/alpha\/css\/alpha-mobile.css": 1,
                    "sites\/all\/themes\/omega\/omega\/css\/formalize.css": 1,
                    "sites\/all\/themes\/omega\/omega\/css\/omega-branding.css": 1,
                    "sites\/all\/themes\/omega\/omega\/css\/omega-menu.css": 1,
                    "sites\/all\/themes\/omega\/omega\/css\/omega-forms.css": 1,
                    "sites\/all\/themes\/rae\/css\/global.css": 1,
                    "sites\/all\/themes\/rae\/css\/menu-styles.css": 1,
                    "sites\/all\/themes\/rae\/css\/node-styles.css": 1,
                    "narrow::sites\/all\/themes\/rae\/css\/rae-alpha-default.css": 1,
                    "narrow::sites\/all\/themes\/rae\/css\/rae-alpha-default-narrow.css": 1,
                    "sites\/all\/themes\/omega\/alpha\/css\/grid\/alpha_default\/narrow\/alpha-default-narrow-12.css": 1,
                    "normal::sites\/all\/themes\/rae\/css\/rae-alpha-default.css": 1,
                    "normal::sites\/all\/themes\/rae\/css\/rae-alpha-default-normal.css": 1,
                    "sites\/all\/themes\/omega\/alpha\/css\/grid\/alpha_default\/normal\/alpha-default-normal-12.css": 1,
                    "wide::sites\/all\/themes\/rae\/css\/rae-alpha-default.css": 1,
                    "wide::sites\/all\/themes\/rae\/css\/rae-alpha-default-wide.css": 1,
                    "sites\/all\/themes\/omega\/alpha\/css\/grid\/alpha_default\/wide\/alpha-default-wide-12.css": 1
                },
                "js": {
                    "sites\/all\/modules\/contrib\/lightbox2\/js\/auto_image_handling.js": 1,
                    "sites\/all\/modules\/contrib\/lightbox2\/js\/lightbox.js": 1,
                    "sites\/all\/modules\/contrib\/eu-cookie-compliance\/js\/eu_cookie_compliance.js": 1,
                    "sites\/all\/libraries\/respondjs\/respond.min.js": 1,
                    "sites\/all\/modules\/contrib\/jquery_update\/replace\/jquery\/1.7\/jquery.min.js": 1,
                    "misc\/jquery.once.js": 1,
                    "misc\/drupal.js": 1,
                    "sites\/all\/modules\/contrib\/tipsy\/javascripts\/jquery.tipsy.js": 1,
                    "sites\/all\/modules\/contrib\/tipsy\/javascripts\/tipsy.js": 1,
                    "sites\/all\/modules\/contrib\/om_maximenu\/js\/om_maximenu.js": 1,
                    "public:\/\/languages\/es_A6vYPIONtvZGBP3to06MFGFNBPsvj5bWkGmifthunw8.js": 1,
                    "http:\/\/w.sharethis.com\/button\/buttons.js": 1,
                    "if (stLight !== undefined) { stLight.options({\u0022publisher\u0022:\u0022dr-8656fd54-d140-dab7-5894-ab893540ae03\u0022,\u0022version\u0022:\u00225x\u0022}); }": 1,
                    "sites\/all\/modules\/contrib\/om_maximenu\/contrib\/jquery.hoverIntent.minified.js": 1,
                    "sites\/all\/modules\/contrib\/responsive_menus\/styles\/responsive_menus_simple\/js\/responsive_menus_simple.js": 1,
                    "sites\/all\/themes\/rae\/js\/hoverIntent.js": 1,
                    "sites\/all\/themes\/rae\/js\/superfish.js": 1,
                    "sites\/all\/themes\/rae\/js\/supersubs.js": 1,
                    "sites\/all\/themes\/rae\/js\/jquery.imagesloaded.min.js": 1,
                    "sites\/all\/themes\/rae\/js\/global.js": 1,
                    "sites\/all\/themes\/omega\/omega\/js\/jquery.formalize.js": 1,
                    "sites\/all\/themes\/omega\/omega\/js\/omega-mediaqueries.js": 1
                }
            },
            "jcarousel": {"ajaxPath": "\/jcarousel\/ajax\/views"},
            "lightbox2": {
                "rtl": "0",
                "file_path": "\/(\\w\\w\/)public:\/",
                "default_image": "\/sites\/all\/modules\/contrib\/lightbox2\/images\/brokenimage.jpg",
                "border_size": 10,
                "font_color": "000",
                "box_color": "FFF",
                "top_position": "100",
                "overlay_opacity": "0.2",
                "overlay_color": "000",
                "disable_close_click": 1,
                "resize_sequence": 0,
                "resize_speed": 100,
                "fade_in_speed": 100,
                "slide_down_speed": 100,
                "use_alt_layout": 1,
                "disable_resize": 1,
                "disable_zoom": 1,
                "force_show_nav": 0,
                "show_caption": 1,
                "loop_items": 0,
                "node_link_text": "View Image Details",
                "node_link_target": 0,
                "image_count": "Image !current of !total",
                "video_count": "Video !current of !total",
                "page_count": "Page !current of !total",
                "lite_press_x_close": "pulsa \u003Ca href=\u0022#\u0022 onclick=\u0022hideLightbox(); return FALSE;\u0022\u003E\u003Ckbd\u003Ex\u003C\/kbd\u003E\u003C\/a\u003E para cerrar",
                "download_link_text": "",
                "enable_login": false,
                "enable_contact": false,
                "keys_close": "c x 27",
                "keys_previous": "p 37",
                "keys_next": "n 39",
                "keys_zoom": "z",
                "keys_play_pause": "32",
                "display_image_size": "original",
                "image_node_sizes": "(\\.thumbnail)",
                "trigger_lightbox_classes": "img.thumbnail, img.image-thumbnail",
                "trigger_lightbox_group_classes": "",
                "trigger_slideshow_classes": "",
                "trigger_lightframe_classes": "",
                "trigger_lightframe_group_classes": "",
                "custom_class_handler": "lightbox_ungrouped",
                "custom_trigger_classes": "img..views-field-rendered-entity .content a,img..lightbox",
                "disable_for_gallery_lists": 0,
                "disable_for_acidfree_gallery_lists": true,
                "enable_acidfree_videos": true,
                "slideshow_interval": 5000,
                "slideshow_automatic_start": true,
                "slideshow_automatic_exit": true,
                "show_play_pause": true,
                "pause_on_next_click": false,
                "pause_on_previous_click": true,
                "loop_slides": false,
                "iframe_width": 600,
                "iframe_height": 400,
                "iframe_border": 0,
                "enable_video": 0
            },
            "tipsy": {
                "custom_selectors": [{
                    "selector": ".tooltip",
                    "options": {
                        "fade": 0,
                        "gravity": "autoNS",
                        "trigger": "hover",
                        "delayIn": "1000",
                        "delayOut": "0",
                        "opacity": "1",
                        "offset": "0",
                        "html": 1,
                        "tooltip_content": {"source": "attribute", "selector": "title"}
                    }
                }]
            },
            "urlIsAjaxTrusted": {"\/diccionario-panhispanico-de-dudas\/apendices\/modelos-de-conjugacion-verbal": true},
            "eu_cookie_compliance": {
                "popup_enabled": 1,
                "popup_agreed_enabled": 0,
                "popup_hide_agreed": 0,
                "popup_clicking_confirmation": 1,
                "popup_html_info": "\u003Cdiv\u003E\n\t\u003Cdiv class=\u0022popup-overlay\u0022\u003E\u003C\/div\u003E\n  \u003Cdiv class =\u0022popup-content info\u0022\u003E\n\t\t\u003Cdiv id=\u0022popup-text\u0022\u003E\n\t\t\t\u003Cp\u003E\u003Cspan\u003EUtilizamos\u0026nbsp;\u003C\/span\u003E\u003Ci\u003E\u003Cspan\u003Ecookies\u003C\/span\u003E\u003C\/i\u003E\u003Cspan\u003E\u0026nbsp;propias y de terceros para mejorar nuestros servicios. Si contin\u00faa navegando,\u0026nbsp;entendemos que acepta su uso.\u003C\/span\u003E\u003C\/p\u003E\t\t\t\u003Cdiv class=\u0022close\u0022\u003E\u003Cimg src=\u0022\/sites\/all\/common\/images\/main\/aspa.png\u0022\u003E\u003C\/div\u003E\n    \u003C\/div\u003E\n  \u003C\/div\u003E\n\u003C\/div\u003E\n",
                "popup_html_agreed": "\u003Cdiv\u003E\n  \u003Cdiv class =\u0022popup-content agreed\u0022\u003E\n    \u003Cdiv id=\u0022popup-text\u0022\u003E\n      \u003Cp\u003E\u0026lt;h2\u0026gt;Thank you for accepting cookies\u0026lt;\/h2\u0026gt;\u0026lt;p\u0026gt;You can now hide this message or find out more about cookies.\u0026lt;\/p\u0026gt;\u003C\/p\u003E\n    \u003C\/div\u003E\n    \u003Cdiv id=\u0022popup-buttons\u0022\u003E\n      \u003Cbutton type=\u0022button\u0022 class=\u0022hide-popup-button\u0022\u003EOcultar\u003C\/button\u003E\n      \u003Cbutton type=\u0022button\u0022 class=\u0022find-more-button\u0022 \u003EMore info\u003C\/button\u003E\n    \u003C\/div\u003E\n  \u003C\/div\u003E\n\u003C\/div\u003E",
                "popup_height": "auto",
                "popup_width": "100%",
                "popup_delay": 1000,
                "popup_link": "\/info\/cookies",
                "popup_link_new_window": 1,
                "popup_position": null,
                "popup_language": "es",
                "domain": ""
            },
            "responsive_menus": [{
                "toggler_text": "\u2630 Men\u00fa",
                "selectors": ["#block-menu-menu-rae-main-mobile .content", "#block-menu-menu-asale-main-mobile .content", "#block-menu-menu-prorae-main-mobile .content"],
                "media_size": "768",
                "absolute": true,
                "remove_attributes": true,
                "responsive_menus_style": "responsive_menus_simple"
            }],
            "omega": {
                "layouts": {
                    "primary": "normal",
                    "order": ["narrow", "normal", "wide"],
                    "queries": {
                        "narrow": "all and (min-width: 740px) and (min-device-width: 740px), (max-device-width: 800px) and (min-width: 740px) and (orientation:landscape)",
                        "normal": "all and (min-width: 980px) and (min-device-width: 980px), all and (max-device-width: 1024px) and (min-width: 1024px) and (orientation:landscape)",
                        "wide": "all and (min-width: 1220px)"
                    }
                }
            }
        });
        //--><!
        ]]>
    </script>
    <!--[if lt IE 9]><script src="http://html5shiv.googlecode.com/svn/trunk/html5.js"></script><![endif]-->
    <script>
        //<![CDATA[
        (function (i, s, o, g, r, a, m) {
            i['GoogleAnalyticsObject'] = r;
            i[r] = i[r] || function () {
                    (i[r].q = i[r].q || []).push(arguments)
                }, i[r].l = 1 * new Date();
            a = s.createElement(o),
                m = s.getElementsByTagName(o)[0];
            a.async = 1;
            a.src = g;
            m.parentNode.insertBefore(a, m)
        })(window, document, 'script', '//www.google-analytics.com/analytics.js', 'ga');
        ga('create', 'UA-44263049-1', 'auto');
        ga('send', 'pageview');
        if (Drupal.eu_cookie_compliance != undefined && !Drupal.eu_cookie_compliance.hasAgreed()) {
            window['ga-disable-UA-44263049-1'] = true;
        }
        //]]>
    </script>
    <script type="text/javascript" charset="utf-8" async=""
            src="https://platform.twitter.com/js/timeline.49f24b3ed4b1be2d087c97aca497c0d1.js"></script>
    <style id="style-1-cropbar-clipper">/* Copyright 2014 Evernote Corporation. All rights reserved. */
    .en-markup-crop-options {
        top: 18px !important;
        left: 50% !important;
        margin-left: -100px !important;
        width: 200px !important;
        border: 2px rgba(255, 255, 255, .38) solid !important;
        border-radius: 4px !important;
    }

    .en-markup-crop-options div div:first-of-type {
        margin-left: 0px !important;
    }
    </style>
</head>
<body class="html not-front not-logged-in page-node page-node- page-node-2312 node-type-page domain-rae context-diccionario-panhispanico-de-dudas responsive-menus-load-processed omega-mediaqueries-processed lightbox-processed sliding-popup-processed responsive-layout-normal">
<div id="omega-media-query-dummy">
    <style media="all">#omega-media-query-dummy {
        position: relative;
        z-index: -1;
    }</style>
    <!--[if (lt IE 9)&(!IEMobile)]><style media="all">#omega-media-query-dummy { z-index: 1; }</style><![endif]-->
    <style media="all and (min-width: 740px) and (min-device-width: 740px), (max-device-width: 800px) and (min-width: 740px) and (orientation:landscape)">#omega-media-query-dummy {
        z-index: 0;
    }</style>
    <style media="all and (min-width: 980px) and (min-device-width: 980px), all and (max-device-width: 1024px) and (min-width: 1024px) and (orientation:landscape)">#omega-media-query-dummy {
        z-index: 1;
    }</style>
    <style media="all and (min-width: 1220px)">#omega-media-query-dummy {
        z-index: 2;
    }</style>
</div>
<div class="region region-page-top" id="region-page-top">
    <div class="region-inner region-page-top-inner">
    </div>
</div>
<div class="page clearfix" id="page">
    <div id="section-content" class="section section-content">
        <div id="zone-content-wrapper" class="zone-wrapper zone-content-wrapper clearfix">
            <div id="zone-content" class="zone zone-content clearfix container-12">
                <div id="breadcrumb" class="grid-12"><h2 class="element-invisible">Se encuentra usted aquí</h2>
                    <div class="breadcrumb"><a href="/">Inicio</a> » <a
                            href="http://www.rae.es/diccionario-panhispanico-de-dudas">Diccionario panhispánico de
                        dudas</a> » <a
                            href="http://www.rae.es/diccionario-panhispanico-de-dudas/apendices">Apéndices</a> » <a
                            href="http://www.rae.es/diccionario-panhispanico-de-dudas/apendices/modelos-de-conjugacion-verbal">Modelos
                        de conjugación verbal</a></div>
                </div>

                <div class="grid-6 region region-content" id="region-content">
                    <div class="region-inner region-content-inner">
                        <a id="main-content"></a>
                        <h1 class="title invisible" id="page-title">Modelos de conjugación verbal</h1>
                        <div class="block block-system block-main block-system-main odd block-without-title"
                             id="block-system-main">
                            <div class="block-inner clearfix">

                                <div class="content clearfix">
                                    <div about="/diccionario-panhispanico-de-dudas/apendices/modelos-de-conjugacion-verbal"
                                         typeof="foaf:Document"
                                         class="node node-page node-published node-not-promoted node-not-sticky author-telefónica odd clearfix"
                                         id="node-page-2312">
                                        <span property="dc:title" content="Modelos de conjugación verbal"
                                              class="rdf-meta element-hidden"></span>

                                        <div class="content clearfix">
                                            <div class="node-type-a">
                                                <div class="node-inner-content">
                                                    <div class="view view-node-a view-id-node_a view-display-id-block view-dom-id-c9cd0d162437151262e17c90759d13e6">


                                                        <div class="view-content">
                                                            <div class="views-row views-row-1 views-row-odd views-row-first views-row-last">

                                                                <div class="views-field views-field-content"><span><div
                                                                        class="views-field views-field-title"><div
                                                                        class="field-content"><h2 itemprop="headline">Modelos de conjugación verbal</h2></div></div><div
                                                                        class="top-buttons"><a
                                                                        href="/recursos/diccionarios/dpd">Consulta</a></div><div
                                                                        class="views-field views-field-body"><div
                                                                        class="field-content" itemprop="articleBody"><p
                                                                        class="dp32"><a name="advertencias"><span
                                                                        class="dp29">Advertencias</span></a></p><p
                                                                        class="dp30">&nbsp;&nbsp;<sup>1</sup> Se recogen en este apéndice los cuadros que sirven de modelo para la conjugación de los <a
                                                                        href="#verreg">verbos regulares</a> e <a
                                                                        href="#verirreg">irregulares</a>. En los cuadros de los tres verbos escogidos como modelo para la conjugación regular (<i>amar, temer, partir</i>), se ofrecen las formas correspondientes a todos los tiempos, tanto simples como compuestos. En el resto de los verbos, por razones de espacio, solo se enuncian las formas de los tiempos simples y las formas no personales.</p><p
                                                                        class="dp30">&nbsp;&nbsp;<sup>2</sup> Cuando en las casillas en las que se recogen los nombres de los tiempos verbales aparecen dos denominaciones separadas por una barra, la primera de ellas corresponde a la terminología académica, y la segunda, a la establecida por Andrés Bello en su <i>Gramática de la lengua castellana destinada al uso de los americanos</i> (1847), vigente en varios países de Hispanoamérica.</p><p
                                                                        class="dp30">&nbsp;&nbsp;<sup>3</sup> En lo que respecta a la terminología académica, debe tenerse en cuenta que el llamado <i>pretérito indefinido</i> en la <i>Gramática de la lengua española</i> de 1931 pasó a denominarse <i>pretérito perfecto simple</i> en el <i>Esbozo de una nueva gramática de la lengua española</i> (1973), y que esta última es la denominación utilizada en los cuadros de conjugación.</p><p
                                                                        class="dp30">&nbsp;&nbsp;<sup>4</sup> En el imperativo, solo se registran las formas propias, esto es, las correspondientes a la segunda persona del singular y del plural.</p><p
                                                                        class="dp30">&nbsp;&nbsp;<sup>5</sup> En la segunda persona del singular del presente de indicativo, y en la misma persona del imperativo, se ofrecen entre paréntesis, junto a las formas corrientes en el español de España y de la América no voseante, las formas del voseo rioplatense admitidas en la norma culta (<span
                                                                        class="dp27">→</span> <span class="dp29"><a
                                                                        href="http://lema.rae.es/dpd/?id=iOTUSehtID6mVONyGX">voseo</a></span>).</p><p
                                                                        class="dp30">&nbsp;&nbsp;<sup>6</sup> Debe tenerse en cuenta que en América, en Canarias y en parte de Andalucía, no se usa el pronombre personal <i>vosotros </i>para la segunda persona del plural. En su lugar se emplea <i>ustedes,</i> que en esas zonas sirve tanto de tratamiento de confianza como de respeto. Por lo tanto, en las áreas mencionadas, las formas verbales de la segunda persona del plural coinciden con las de la tercera persona del plural.</p><p
                                                                        class="dp35"><b><a name="verreg">I. Verbos regulares</a></b></p><p
                                                                        class="dp30">&nbsp;&nbsp;Se incluyen aquí, además de los tres verbos modelo de la conjugación regular (<i>amar, temer, partir</i>), los modelos de conjugación para cada uno de los dos grupos en que se dividen, en cuanto al acento, los verbos terminados en <i>-iar</i> y en <i>-uar</i> y los verbos que presentan en su raíz los grupos vocálicos /ai/, /au/, /ei/ y /eu/.</p><table
                                                                        border="0" cellpadding="4" cellspacing="0"
                                                                        width="576"><tbody><tr><td colspan="2"
                                                                                                   valign="top"
                                                                                                   width="33%"><p
                                                                        class="dp35"><a href="#n1">AMAR</a></p></td><td
                                                                        colspan="2" valign="top" width="33%"><p
                                                                        class="dp35"><a href="#n2">TEMER</a></p></td><td
                                                                        colspan="2" valign="top" width="33%"><p
                                                                        class="dp35"><a
                                                                        href="#n3">PARTIR</a></p></td></tr><tr><td
                                                                        valign="top" width="17%"></td><td valign="top"
                                                                                                          width="17%"></td><td
                                                                        valign="top" width="17%"></td><td valign="top"
                                                                                                          width="17%"></td><td
                                                                        valign="top" width="17%"></td><td valign="top"
                                                                                                          width="17%"></td></tr><tr><td
                                                                        valign="top" width="17%"><p class="dp37"><a
                                                                        href="#n4">ANUNCIAR</a></p></td><td valign="top"
                                                                                                            width="17%"><p
                                                                        class="dp37"><a
                                                                        href="#n6">AVERIGUAR</a></p></td><td
                                                                        valign="top" width="17%"><p class="dp37"><a
                                                                        href="#n8">BAILAR</a></p></td><td valign="top"
                                                                                                          width="17%"><p
                                                                        class="dp37"><a href="#n10">CAUSAR</a></p></td><td
                                                                        valign="top" width="17%"><p class="dp37"><a
                                                                        href="#n12">PEINAR</a></p></td><td valign="top"
                                                                                                           width="17%"><p
                                                                        class="dp37"><a href="#n14">ADEUDAR</a></p></td></tr><tr><td
                                                                        valign="top" width="17%"><p class="dp37"><a
                                                                        href="#n5">ENVIAR</a></p></td><td valign="top"
                                                                                                          width="17%"><p
                                                                        class="dp37"><a href="#n7">ACTUAR</a></p></td><td
                                                                        valign="top" width="17%"><p class="dp37"><a
                                                                        href="#n9">AISLAR</a></p></td><td valign="top"
                                                                                                          width="17%"><p
                                                                        class="dp37"><a href="#n11">AUNAR</a></p></td><td
                                                                        valign="top" width="17%"><p class="dp37"><a
                                                                        href="#n13">DESCAFEINAR</a></p></td><td
                                                                        valign="top" width="17%"><p class="dp37"><a
                                                                        href="#n15">REHUSAR</a></p></td></tr></tbody></table><hr><p
                                                                        class="dp32"><a name="verirreg"><span
                                                                        class="dp29">Verbos irregulares</span></a></p><p
                                                                        class="dp30">&nbsp;&nbsp;Se incluyen bajo este epígrafe tanto los verbos de irregularidad propia, cuyo paradigma es único (<i>ir, ser,</i> etc.), como los que sirven de modelo a otros verbos irregulares (<i>acertar, agradecer,</i> etc.). También se incluye aquí el verbo <i>leer</i> —modelo de otros verbos como <i>creer</i> o <i>proveer</i>—, que aun siendo regular desde el punto de vista morfológico, no lo es desde el punto de vista gráfico-articulatorio, ya que el sonido vocálico /i/ de algunas desinencias, cuando queda entre vocales, se transforma en el sonido consonántico /y/; así, la raíz <i>le-</i> + la desinencia <i>-ió</i> no da <i>leió</i>, sino <i>leyó</i>; <i>le-</i> + <i>-iera</i> no da <i>leiera,</i> sino <i>leyera,</i> etc.</p><table
                                                                        border="0" cellpadding="4" cellspacing="0"
                                                                        width="576"><tbody><tr><td class="dp45"><p
                                                                        class="dp30"><a href="#n16">ACERTAR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n28">DECIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n40">LUCIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n52">SALIR</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a href="#n17">ADQUIRIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n29">DISCERNIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n41">MOVER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n53">SENTIR</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a href="#n18">AGRADECER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n30">DORMIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n42">MULLIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a
                                                                        href="#n54">SER</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a href="#n19">ANDAR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n31">ENTENDER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a
                                                                        href="#n43">OÍR</a></p></td><td class="dp45"><p
                                                                        class="dp30"><a href="#n55">SONREÍR</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a
                                                                        href="#n20">ASIR</a></p></td><td class="dp45"><p
                                                                        class="dp30"><a href="#n32">ERGUIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a
                                                                        href="#n44">OLER</a></p></td><td class="dp45"><p
                                                                        class="dp30"><a
                                                                        href="#n56">TAÑER</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a href="#n21">CABER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n33">ERRAR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n45">PEDIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n57">TENER</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a
                                                                        href="#n22">CAER</a></p></td><td class="dp45"><p
                                                                        class="dp30"><a href="#n34">ESTAR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n46">PODER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n58">TRAER</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a href="#n23">CEÑIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n35">HABER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n47">PONER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n59">VALER</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a href="#n24">CONDUCIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n36">HACER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n48">PUDRIR / PODRIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n60">VENIR</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a href="#n25">CONSTRUIR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a
                                                                        href="#n37">IR</a></p></td><td class="dp45"><p
                                                                        class="dp30"><a href="#n49">QUERER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a
                                                                        href="#n61">VER</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a href="#n26">CONTAR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n38">JUGAR</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a
                                                                        href="#n50">ROER</a></p></td><td class="dp45"><p
                                                                        class="dp30"><a
                                                                        href="#n62">YACER</a></p></td></tr><tr><td
                                                                        class="dp45"><p class="dp30"><a
                                                                        href="#n27">DAR</a></p></td><td class="dp45"><p
                                                                        class="dp30"><a href="#n39">LEER</a></p></td><td
                                                                        class="dp45"><p class="dp30"><a href="#n51">SABER</a></p></td><td
                                                                        class="dp45"></td></tr></tbody></table><hr><p
                                                                        class="dp32"><a name="verregc"><span
                                                                        class="dp29">Verbos regulares</span></a></p><p
                                                                        class="dp30">&nbsp;&nbsp;Se incluyen aquí, además de los tres verbos modelo de la conjugación regular (<i>amar, temer, partir</i>), los modelos de conjugación para cada uno de los dos grupos en que se dividen, en cuanto al acento, los verbos terminados en <i>-iar</i> y en <i>-uar</i> y los verbos que presentan en su raíz los grupos vocálicos /ai/, /au/, /ei/ y /eu/.</p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n1">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="5"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>1. AMAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">Verbo modelo de la 1.ª conjugación</p></td></tr><tr><td
                                                                        bgcolor="#808080" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS SIMPLES</p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33">amo</p><p
                                                                        class="dp33">amas (amás)</p><p
                                                                        class="dp33">ama</p><p class="dp33">amamos</p><p
                                                                        class="dp33">amáis</p><p
                                                                        class="dp33">aman</p></td><td class="dp23"
                                                                                                      colspan="3"><p
                                                                        class="dp33">amaba</p><p class="dp33">amabas</p><p
                                                                        class="dp33">amaba</p><p
                                                                        class="dp33">amábamos</p><p
                                                                        class="dp33">amabais</p><p
                                                                        class="dp33">amaban</p></td><td class="dp39"><p
                                                                        class="dp33">amé</p><p class="dp33">amaste</p><p
                                                                        class="dp33">amó</p><p class="dp33">amamos</p><p
                                                                        class="dp33">amasteis</p><p
                                                                        class="dp33">amaron</p></td><td class="dp40"
                                                                                                        colspan="2"><p
                                                                        class="dp33">amaré</p><p class="dp33">amarás</p><p
                                                                        class="dp33">amará</p><p
                                                                        class="dp33">amaremos</p><p
                                                                        class="dp33">amaréis</p><p
                                                                        class="dp33">amarán</p></td><td class="dp39"><p
                                                                        class="dp33">amaría</p><p
                                                                        class="dp33">amarías</p><p
                                                                        class="dp33">amaría</p><p
                                                                        class="dp33">amaríamos</p><p class="dp33">amaríais</p><p
                                                                        class="dp33">amarían</p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS COMPUESTOS</p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>pret. perfecto</b><br><b>compuesto</b> /<br><b>antepresente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret.</b><br><b>pluscuamperfecto</b> / <b>antecopretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. anterior</b> /<br><b>antepretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro compuesto</b> /<br><b>antefuturo</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>compuesto</b> /<br><b>antepospretérito</b></p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33">he amado</p><p
                                                                        class="dp33">has amado</p><p class="dp33">ha amado</p><p
                                                                        class="dp33">hemos amado</p><p class="dp33">habéis amado</p><p
                                                                        class="dp33">han amado</p></td><td class="dp23"
                                                                                                           colspan="3"><p
                                                                        class="dp33">había amado</p><p class="dp33">habías amado</p><p
                                                                        class="dp33">había amado</p><p class="dp33">habíamos amado</p><p
                                                                        class="dp33">habíais amado</p><p class="dp33">habían amado</p></td><td
                                                                        class="dp39"><p class="dp33">hube amado</p><p
                                                                        class="dp33">hubiste amado</p><p class="dp33">hubo amado</p><p
                                                                        class="dp33">hubimos amado</p><p class="dp33">hubisteis amado</p><p
                                                                        class="dp33">hubieron amado</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">habré amado</p><p
                                                                        class="dp33">habrás amado</p><p class="dp33">habrá amado</p><p
                                                                        class="dp33">habremos amado</p><p class="dp33">habréis amado</p><p
                                                                        class="dp33">habrán amado</p></td><td
                                                                        class="dp39"><p class="dp33">habría amado</p><p
                                                                        class="dp33">habrías amado</p><p class="dp33">habría amado</p><p
                                                                        class="dp33">habríamos amado</p><p class="dp33">habríais amado</p><p
                                                                        class="dp33">habrían amado</p></td></tr><tr><td
                                                                        bgcolor="#808080" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS SIMPLES</p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33">ame</p><p class="dp33">ames</p><p
                                                                        class="dp33">ame</p><p class="dp33">amemos</p><p
                                                                        class="dp33">améis</p><p
                                                                        class="dp33">amen</p></td><td class="dp22"
                                                                                                      colspan="4"><p
                                                                        class="dp33">amara <i>o</i> amase</p><p
                                                                        class="dp33">amaras <i>o</i> amases</p><p
                                                                        class="dp33">amara <i>o</i> amase</p><p
                                                                        class="dp33">amáramos <i>o</i> amásemos</p><p
                                                                        class="dp33">amarais <i>o</i> amaseis</p><p
                                                                        class="dp33">amaran <i>o</i> amasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">amare</p><p class="dp33">amares</p><p
                                                                        class="dp33">amare</p><p
                                                                        class="dp33">amáremos</p><p
                                                                        class="dp33">amareis</p><p
                                                                        class="dp33">amaren</p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS COMPUESTOS</p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33"><b>pret. perfecto compuesto</b> /<br><b>antepresente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. pluscuamperfecto</b> /<br><b>antepretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro compuesto</b> /<br><b>antefuturo</b></p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33">haya amado</p><p class="dp33">hayas amado</p><p
                                                                        class="dp33">haya amado</p><p class="dp33">hayamos amado</p><p
                                                                        class="dp33">hayáis amado</p><p class="dp33">hayan amado</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">hubiera <i>o</i> hubiese amado</p><p
                                                                        class="dp33">hubieras <i>o</i> hubieses amado</p><p
                                                                        class="dp33">hubiera <i>o</i> hubiese amado</p><p
                                                                        class="dp33">hubiéramos <i>o</i> hubiésemos amado</p><p
                                                                        class="dp33">hubierais <i>o</i> hubieseis amado</p><p
                                                                        class="dp33">hubieran <i>o</i> hubiesen amado</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">hubiere amado</p><p
                                                                        class="dp33">hubieres amado</p><p class="dp33">hubiere amado</p><p
                                                                        class="dp33">hubiéremos amado</p><p
                                                                        class="dp33">hubiereis amado</p><p class="dp33">hubieren amado</p></td></tr><tr><td
                                                                        bgcolor="#808080" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="9" valign="middle">ama (amá), amad</td></tr><tr><td
                                                                        bgcolor="#c0c0c0" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        colspan="4" valign="middle" width="39%"><p
                                                                        class="dp33"><b>infinitivo</b></p></td><td
                                                                        colspan="2" valign="middle" width="22%"><p
                                                                        class="dp33"><b>participio</b></p></td><td
                                                                        colspan="3" valign="middle" width="40%"><p
                                                                        class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        height="8" valign="middle" width="19%"><p
                                                                        class="dp33">SIMPLE</p></td><td colspan="3"
                                                                                                        height="8"
                                                                                                        valign="middle"
                                                                                                        width="19%"><p
                                                                        class="dp33">COMPUESTO</p></td><td colspan="2"
                                                                                                           height="8"
                                                                                                           rowspan="2"
                                                                                                           valign="middle"
                                                                                                           width="22%"><p
                                                                        class="dp33">amado</p></td><td colspan="2"
                                                                                                       height="8"
                                                                                                       valign="middle"
                                                                                                       width="20%"><p
                                                                        class="dp33">SIMPLE</p></td><td height="8"
                                                                                                        valign="middle"
                                                                                                        width="20%"><p
                                                                        class="dp33">COMPUESTO</p></td></tr><tr><td
                                                                        height="8" valign="middle" width="19%"><p
                                                                        class="dp33">amar</p></td><td colspan="3"
                                                                                                      height="8"
                                                                                                      valign="middle"
                                                                                                      width="19%"><p
                                                                        class="dp33">haber amado</p></td><td colspan="2"
                                                                                                             height="8"
                                                                                                             valign="middle"
                                                                                                             width="20%"><p
                                                                        class="dp33">amando</p></td><td height="8"
                                                                                                        valign="middle"
                                                                                                        width="20%"><p
                                                                        class="dp33">habiendo amado</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n2">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="5"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>2. TEMER</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">Verbo modelo de la 2.ª conjugación</p></td></tr><tr><td
                                                                        bgcolor="#808080" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS SIMPLES</p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">temo</p><p class="dp33">temes (temés)</p><p
                                                                        class="dp33">teme</p><p class="dp33">tememos</p><p
                                                                        class="dp33">teméis</p><p class="dp33">temen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">temía</p><p class="dp33">temías</p><p
                                                                        class="dp33">temía</p><p
                                                                        class="dp33">temíamos</p><p
                                                                        class="dp33">temíais</p><p
                                                                        class="dp33">temían</p></td><td class="dp39"><p
                                                                        class="dp33">temí</p><p class="dp33">temiste</p><p
                                                                        class="dp33">temió</p><p
                                                                        class="dp33">temimos</p><p class="dp33">temisteis</p><p
                                                                        class="dp33">temieron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">temeré</p><p
                                                                        class="dp33">temerás</p><p
                                                                        class="dp33">temerá</p><p
                                                                        class="dp33">temeremos</p><p class="dp33">temeréis</p><p
                                                                        class="dp33">temerán</p></td><td class="dp39"><p
                                                                        class="dp33">temería</p><p
                                                                        class="dp33">temerías</p><p
                                                                        class="dp33">temería</p><p class="dp33">temeríamos</p><p
                                                                        class="dp33">temeríais</p><p class="dp33">temerían</p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS COMPUESTOS</p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>pret. perfecto</b><br><b>compuesto</b> /<br><b>antepresente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret.</b><br><b>pluscuamperfecto</b> / <b>antecopretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. anterior</b> /<br><b>antepretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro compuesto</b> /<br><b>antefuturo</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>compuesto</b> /<br><b>antepospretérito</b></p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33">he temido</p><p
                                                                        class="dp33">has temido</p><p class="dp33">ha temido</p><p
                                                                        class="dp33">hemos temido</p><p class="dp33">habéis temido</p><p
                                                                        class="dp33">han temido</p></td><td class="dp23"
                                                                                                            colspan="3"><p
                                                                        class="dp33">había temido</p><p class="dp33">habías temido</p><p
                                                                        class="dp33">había temido</p><p class="dp33">habíamos temido</p><p
                                                                        class="dp33">habíais temido</p><p class="dp33">habían temido</p></td><td
                                                                        class="dp39"><p class="dp33">hube temido</p><p
                                                                        class="dp33">hubiste temido</p><p class="dp33">hubo temido</p><p
                                                                        class="dp33">hubimos temido</p><p class="dp33">hubisteis temido</p><p
                                                                        class="dp33">hubieron temido</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">habré temido</p><p
                                                                        class="dp33">habrás temido</p><p class="dp33">habrá temido</p><p
                                                                        class="dp33">habremos temido</p><p class="dp33">habréis temido</p><p
                                                                        class="dp33">habrán temido</p></td><td
                                                                        class="dp39"><p class="dp33">habría temido</p><p
                                                                        class="dp33">habrías temido</p><p class="dp33">habría temido</p><p
                                                                        class="dp33">habríamos temido</p><p
                                                                        class="dp33">habríais temido</p><p class="dp33">habrían temido</p></td></tr><tr><td
                                                                        bgcolor="#808080" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS SIMPLES</p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33">tema</p><p class="dp33">temas</p><p
                                                                        class="dp33">tema</p><p class="dp33">temamos</p><p
                                                                        class="dp33">temáis</p><p class="dp33">teman</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">temiera <i>o</i> temiese</p><p
                                                                        class="dp33">temieras <i>o</i> temieses</p><p
                                                                        class="dp33">temiera <i>o</i> temiese</p><p
                                                                        class="dp33">temiéramos <i>o</i> temiésemos</p><p
                                                                        class="dp33">temierais <i>o</i> temieseis</p><p
                                                                        class="dp33">temieran <i>o</i> temiesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">temiere</p><p
                                                                        class="dp33">temieres</p><p
                                                                        class="dp33">temiere</p><p class="dp33">temiéremos</p><p
                                                                        class="dp33">temiereis</p><p class="dp33">temieren</p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS COMPUESTOS</p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33"><b>pret. perfecto compuesto</b> /<br><b>antepresente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. pluscuamperfecto</b> /<br><b>antepretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro compuesto</b> /<br><b>antefuturo</b></p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33">haya temido</p><p class="dp33">hayas temido</p><p
                                                                        class="dp33">haya temido</p><p class="dp33">hayamos temido</p><p
                                                                        class="dp33">hayáis temido</p><p class="dp33">hayan temido</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">hubiera <i>o</i> hubiese temido</p><p
                                                                        class="dp33">hubieras <i>o</i> hubieses temido</p><p
                                                                        class="dp33">hubiera <i>o</i> hubiese temido</p><p
                                                                        class="dp33">hubiéramos <i>o</i> hubiésemos temido</p><p
                                                                        class="dp33">hubierais <i>o</i> hubieseis temido</p><p
                                                                        class="dp33">hubieran <i>o</i> hubiesen temido</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">hubiere temido</p><p
                                                                        class="dp33">hubieres temido</p><p class="dp33">hubiere temido</p><p
                                                                        class="dp33">hubiéremos temido</p><p
                                                                        class="dp33">hubiereis temido</p><p
                                                                        class="dp33">hubieren temido</p></td></tr><tr><td
                                                                        bgcolor="#808080" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="9" valign="middle">teme (temé), temed</td></tr><tr><td
                                                                        bgcolor="#c0c0c0" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        colspan="4" valign="middle" width="39%"><p
                                                                        class="dp33"><b>infinitivo</b></p></td><td
                                                                        colspan="2" valign="middle" width="22%"><p
                                                                        class="dp33"><b>participio</b></p></td><td
                                                                        colspan="3" valign="middle" width="40%"><p
                                                                        class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        height="8" valign="middle" width="19%"><p
                                                                        class="dp33">SIMPLE</p></td><td colspan="3"
                                                                                                        height="8"
                                                                                                        valign="middle"
                                                                                                        width="19%"><p
                                                                        class="dp33">COMPUESTO</p></td><td colspan="2"
                                                                                                           height="8"
                                                                                                           rowspan="2"
                                                                                                           valign="middle"
                                                                                                           width="22%"><p
                                                                        class="dp33">temido</p></td><td colspan="2"
                                                                                                        height="8"
                                                                                                        valign="middle"
                                                                                                        width="20%"><p
                                                                        class="dp33">SIMPLE</p></td><td height="8"
                                                                                                        valign="middle"
                                                                                                        width="20%"><p
                                                                        class="dp33">COMPUESTO</p></td></tr><tr><td
                                                                        height="8" valign="middle" width="19%"><p
                                                                        class="dp33">temer</p></td><td colspan="3"
                                                                                                       height="8"
                                                                                                       valign="middle"
                                                                                                       width="19%"><p
                                                                        class="dp33">haber temido</p></td><td
                                                                        colspan="2" height="8" valign="middle"
                                                                        width="20%"><p class="dp33">temiendo</p></td><td
                                                                        height="8" valign="middle" width="20%"><p
                                                                        class="dp33">habiendo temido</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n3">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="5"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>3. PARTIR </b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">Verbo modelo de la 3.ª conjugación</p></td></tr><tr><td
                                                                        bgcolor="#808080" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS SIMPLES</p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">parto</p><p class="dp33">partes (partís)</p><p
                                                                        class="dp33">parte</p><p
                                                                        class="dp33">partimos</p><p
                                                                        class="dp33">partís</p><p
                                                                        class="dp33">parten</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">partía</p><p
                                                                        class="dp33">partías</p><p
                                                                        class="dp33">partía</p><p
                                                                        class="dp33">partíamos</p><p class="dp33">partíais</p><p
                                                                        class="dp33">partían</p></td><td class="dp39"><p
                                                                        class="dp33">partí</p><p
                                                                        class="dp33">partiste</p><p
                                                                        class="dp33">partió</p><p
                                                                        class="dp33">partimos</p><p class="dp33">partisteis</p><p
                                                                        class="dp33">partieron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">partiré</p><p
                                                                        class="dp33">partirás</p><p
                                                                        class="dp33">partirá</p><p class="dp33">partiremos</p><p
                                                                        class="dp33">partiréis</p><p class="dp33">partirán</p></td><td
                                                                        class="dp39"><p class="dp33">partiría</p><p
                                                                        class="dp33">partirías</p><p class="dp33">partiría</p><p
                                                                        class="dp33">partiríamos</p><p class="dp33">partiríais</p><p
                                                                        class="dp33">partirían</p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS COMPUESTOS</p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>pret. perfecto</b><br><b>compuesto</b> /<br><b>antepresente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret.</b><br><b>pluscuamperfecto</b> / <b>antecopretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. anterior</b> /<br><b>antepretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro compuesto</b> /<br><b>antefuturo</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>compuesto</b> /<br><b>antepospretérito</b></p></td></tr><tr><td
                                                                        class="dp40" colspan="2"><p class="dp33">he partido</p><p
                                                                        class="dp33">has partido</p><p class="dp33">ha partido</p><p
                                                                        class="dp33">hemos partido</p><p class="dp33">habéis partido</p><p
                                                                        class="dp33">han partido</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">había partido</p><p
                                                                        class="dp33">habías partido</p><p class="dp33">había partido</p><p
                                                                        class="dp33">habíamos partido</p><p
                                                                        class="dp33">habíais partido</p><p class="dp33">habían partido</p></td><td
                                                                        class="dp39"><p class="dp33">hube partido</p><p
                                                                        class="dp33">hubiste partido</p><p class="dp33">hubo partido</p><p
                                                                        class="dp33">hubimos partido</p><p class="dp33">hubisteis partido</p><p
                                                                        class="dp33">hubieron partido</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">habré partido</p><p
                                                                        class="dp33">habrás partido</p><p class="dp33">habrá partido</p><p
                                                                        class="dp33">habremos partido</p><p
                                                                        class="dp33">habréis partido</p><p class="dp33">habrán partido</p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33">habría partido</p><p class="dp33">habrías partido</p><p
                                                                        class="dp33">habría partido</p><p class="dp33">habríamos partido</p><p
                                                                        class="dp33">habríais partido</p><p
                                                                        class="dp33">habrían partido</p></td></tr><tr><td
                                                                        bgcolor="#808080" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS SIMPLES</p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33">parta</p><p class="dp33">partas</p><p
                                                                        class="dp33">parta</p><p
                                                                        class="dp33">partamos</p><p
                                                                        class="dp33">partáis</p><p
                                                                        class="dp33">partan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">partiera <i>o</i> partiese</p><p
                                                                        class="dp33">partieras <i>o</i> partieses</p><p
                                                                        class="dp33">partiera <i>o</i> partiese</p><p
                                                                        class="dp33">partiéramos <i>o</i> partiésemos</p><p
                                                                        class="dp33">partierais <i>o</i> partieseis</p><p
                                                                        class="dp33">partieran <i>o</i> partiesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">partiere</p><p
                                                                        class="dp33">partieres</p><p class="dp33">partiere</p><p
                                                                        class="dp33">partiéremos</p><p class="dp33">partiereis</p><p
                                                                        class="dp33">partieren</p></td></tr><tr><td
                                                                        colspan="9" valign="middle"><p class="dp33">TIEMPOS COMPUESTOS</p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33"><b>pret. perfecto compuesto</b> /<br><b>antepresente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. pluscuamperfecto</b> /<br><b>antepretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro compuesto</b> /<br><b>antefuturo</b></p></td></tr><tr><td
                                                                        colspan="3" valign="middle" width="30%"><p
                                                                        class="dp33">haya partido</p><p class="dp33">hayas partido</p><p
                                                                        class="dp33">haya partido</p><p class="dp33">hayamos partido</p><p
                                                                        class="dp33">hayáis partido</p><p class="dp33">hayan partido</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">hubiera <i>o</i> hubiese partido</p><p
                                                                        class="dp33">hubieras <i>o</i> hubieses partido</p><p
                                                                        class="dp33">hubiera <i>o</i> hubiese partido</p><p
                                                                        class="dp33">hubiéramos <i>o</i> hubiésemos partido</p><p
                                                                        class="dp33">hubierais <i>o</i> hubieseis partido</p><p
                                                                        class="dp33">hubieran <i>o</i> hubiesen partido</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">hubiere partido</p><p
                                                                        class="dp33">hubieres partido</p><p
                                                                        class="dp33">hubiere partido</p><p class="dp33">hubiéremos partido</p><p
                                                                        class="dp33">hubiereis partido</p><p
                                                                        class="dp33">hubieren partido</p></td></tr><tr><td
                                                                        bgcolor="#808080" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="9" valign="middle">parte (partí), partid</td></tr><tr><td
                                                                        bgcolor="#c0c0c0" colspan="9" valign="middle"><p
                                                                        class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        colspan="4" valign="middle" width="39%"><p
                                                                        class="dp33"><b>infinitivo</b></p></td><td
                                                                        colspan="2" valign="middle" width="22%"><p
                                                                        class="dp33"><b>participio</b></p></td><td
                                                                        colspan="3" valign="middle" width="40%"><p
                                                                        class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        height="8" valign="middle" width="19%"><p
                                                                        class="dp33">SIMPLE</p></td><td colspan="3"
                                                                                                        height="8"
                                                                                                        valign="middle"
                                                                                                        width="19%"><p
                                                                        class="dp33">COMPUESTO</p></td><td colspan="2"
                                                                                                           height="8"
                                                                                                           rowspan="2"
                                                                                                           valign="middle"
                                                                                                           width="22%"><p
                                                                        class="dp33">partido</p></td><td colspan="2"
                                                                                                         height="8"
                                                                                                         valign="middle"
                                                                                                         width="20%"><p
                                                                        class="dp33">SIMPLE</p></td><td height="8"
                                                                                                        valign="middle"
                                                                                                        width="20%"><p
                                                                        class="dp33">COMPUESTO</p></td></tr><tr><td
                                                                        height="8" valign="middle" width="19%"><p
                                                                        class="dp33">partir</p></td><td colspan="3"
                                                                                                        height="8"
                                                                                                        valign="middle"
                                                                                                        width="19%"><p
                                                                        class="dp33">haber partido</p></td><td
                                                                        colspan="2" height="8" valign="middle"
                                                                        width="20%"><p class="dp33">partiendo</p></td><td
                                                                        height="8" valign="middle" width="20%"><p
                                                                        class="dp33">habiendo partido</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n4">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>4. ANUNCIAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>i</i> que precede a la desinencia es átona<br>en todas las formas de este verbo.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">anuncio</p><p
                                                                        class="dp33">anuncias (anunciás)</p><p
                                                                        class="dp33">anuncia</p><p class="dp33">anunciamos</p><p
                                                                        class="dp33">anunciáis</p><p class="dp33">anuncian</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">anunciaba</p><p
                                                                        class="dp33">anunciabas</p><p class="dp33">anunciaba</p><p
                                                                        class="dp33">anunciábamos</p><p class="dp33">anunciabais</p><p
                                                                        class="dp33">anunciaban</p></td><td
                                                                        class="dp39"><p class="dp33">anuncié</p><p
                                                                        class="dp33">anunciaste</p><p class="dp33">anunció</p><p
                                                                        class="dp33">anunciamos</p><p class="dp33">anunciasteis</p><p
                                                                        class="dp33">anunciaron</p></td><td class="dp40"
                                                                                                            colspan="2"><p
                                                                        class="dp33">anunciaré</p><p class="dp33">anunciarás</p><p
                                                                        class="dp33">anunciará</p><p class="dp33">anunciaremos</p><p
                                                                        class="dp33">anunciaréis</p><p class="dp33">anunciarán</p></td><td
                                                                        class="dp39"><p class="dp33">anunciaría</p><p
                                                                        class="dp33">anunciarías</p><p class="dp33">anunciaría</p><p
                                                                        class="dp33">anunciaríamos</p><p class="dp33">anunciaríais</p><p
                                                                        class="dp33">anunciarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">anuncie</p><p
                                                                        class="dp33">anuncies</p><p
                                                                        class="dp33">anuncie</p><p class="dp33">anunciemos</p><p
                                                                        class="dp33">anunciéis</p><p class="dp33">anuncien</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">anunciara <i>o</i> anunciase</p><p
                                                                        class="dp33">anunciaras <i>o</i> anunciases</p><p
                                                                        class="dp33">anunciara <i>o</i> anunciase</p><p
                                                                        class="dp33">anunciáramos <i>o</i> anunciásemos</p><p
                                                                        class="dp33">anunciarais <i>o</i> anunciaseis</p><p
                                                                        class="dp33">anunciaran <i>o</i> anunciasen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">anunciare</p><p
                                                                        class="dp33">anunciares</p><p class="dp33">anunciare</p><p
                                                                        class="dp33">anunciáremos</p><p class="dp33">anunciareis</p><p
                                                                        class="dp33">anunciaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">anuncia (anunciá), anunciad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        colspan="2" valign="middle" width="22%"><p
                                                                        class="dp33"><b>participio</b></p></td><td
                                                                        colspan="3" valign="middle" width="40%"><p
                                                                        class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">anunciar</p></td><td
                                                                        colspan="2" valign="middle" width="22%"><p
                                                                        class="dp33">anunciado</p></td><td colspan="3"
                                                                                                           valign="middle"
                                                                                                           width="40%"><p
                                                                        class="dp33">anunciando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n5">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>5. ENVIAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>i</i> que precede a la desinencia es tónica en las formas<br>de este verbo que llevan el acento prosódico en la raíz.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">envío</p><p
                                                                        class="dp33">envías (enviás)</p><p class="dp33">envía</p><p
                                                                        class="dp33">enviamos</p><p
                                                                        class="dp33">enviáis</p><p
                                                                        class="dp33">envían</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">enviaba</p><p
                                                                        class="dp33">enviabas</p><p
                                                                        class="dp33">enviaba</p><p class="dp33">enviábamos</p><p
                                                                        class="dp33">enviabais</p><p class="dp33">enviaban</p></td><td
                                                                        class="dp39"><p class="dp33">envié</p><p
                                                                        class="dp33">enviaste</p><p
                                                                        class="dp33">envió</p><p
                                                                        class="dp33">enviamos</p><p class="dp33">enviasteis</p><p
                                                                        class="dp33">enviaron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">enviaré</p><p
                                                                        class="dp33">enviarás</p><p
                                                                        class="dp33">enviará</p><p class="dp33">enviaremos</p><p
                                                                        class="dp33">enviaréis</p><p class="dp33">enviarán</p></td><td
                                                                        class="dp39"><p class="dp33">enviaría</p><p
                                                                        class="dp33">enviarías</p><p class="dp33">enviaría</p><p
                                                                        class="dp33">enviaríamos</p><p class="dp33">enviaríais</p><p
                                                                        class="dp33">enviarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">envíe</p><p class="dp33">envíes</p><p
                                                                        class="dp33">envíe</p><p
                                                                        class="dp33">enviemos</p><p
                                                                        class="dp33">enviéis</p><p
                                                                        class="dp33">envíen</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">enviara <i>o</i> enviase</p><p
                                                                        class="dp33">enviaras <i>o</i> enviases</p><p
                                                                        class="dp33">enviara <i>o</i> enviase</p><p
                                                                        class="dp33">enviáramos <i>o</i> enviásemos</p><p
                                                                        class="dp33">enviarais <i>o</i> enviaseis</p><p
                                                                        class="dp33">enviaran <i>o</i> enviasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">enviare</p><p
                                                                        class="dp33">enviares</p><p
                                                                        class="dp33">enviare</p><p class="dp33">enviáremos</p><p
                                                                        class="dp33">enviareis</p><p class="dp33">enviaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="8" valign="middle">envía (enviá), enviad</td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        colspan="2" valign="middle" width="22%"><p
                                                                        class="dp33"><b>participio</b></p></td><td
                                                                        colspan="3" valign="middle" width="40%"><p
                                                                        class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">enviar</p></td><td colspan="2"
                                                                                                        valign="middle"
                                                                                                        width="22%"><p
                                                                        class="dp33">enviado</p></td><td colspan="3"
                                                                                                         valign="middle"
                                                                                                         width="40%"><p
                                                                        class="dp33">enviando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n6">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>6. AVERIGUAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>u</i> que precede a la desinencia es átona<br>en todas las formas de este verbo.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">averiguo</p><p
                                                                        class="dp33">averiguas (averiguás)</p><p
                                                                        class="dp33">averigua</p><p class="dp33">averiguamos</p><p
                                                                        class="dp33">averiguáis</p><p class="dp33">averiguan</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">averiguaba</p><p
                                                                        class="dp33">averiguabas</p><p class="dp33">averiguaba</p><p
                                                                        class="dp33">averiguábamos</p><p class="dp33">averiguabais</p><p
                                                                        class="dp33">averiguaban</p></td><td
                                                                        class="dp39"><p class="dp33">averigüé</p><p
                                                                        class="dp33">averiguaste</p><p class="dp33">averiguó</p><p
                                                                        class="dp33">averiguamos</p><p class="dp33">averiguasteis</p><p
                                                                        class="dp33">averiguaron</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">averiguaré</p><p
                                                                        class="dp33">averiguarás</p><p class="dp33">averiguará</p><p
                                                                        class="dp33">averiguaremos</p><p class="dp33">averiguaréis</p><p
                                                                        class="dp33">averiguarán</p></td><td
                                                                        class="dp39"><p class="dp33">averiguaría</p><p
                                                                        class="dp33">averiguarías</p><p class="dp33">averiguaría</p><p
                                                                        class="dp33">averiguaríamos</p><p class="dp33">averiguaríais</p><p
                                                                        class="dp33">averiguarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">averigüe</p><p
                                                                        class="dp33">averigües</p><p class="dp33">averigüe</p><p
                                                                        class="dp33">averigüemos</p><p class="dp33">averigüéis</p><p
                                                                        class="dp33">averigüen</p></td><td class="dp22"
                                                                                                           colspan="4"><p
                                                                        class="dp33">averiguara <i>o</i> averiguase</p><p
                                                                        class="dp33">averiguaras <i>o</i> averiguases</p><p
                                                                        class="dp33">averiguara <i>o</i> averiguase</p><p
                                                                        class="dp33">averiguáramos <i>o</i> averiguásemos</p><p
                                                                        class="dp33">averiguarais <i>o</i> averiguaseis</p><p
                                                                        class="dp33">averiguaran <i>o</i> averiguasen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">averiguare</p><p
                                                                        class="dp33">averiguares</p><p class="dp33">averiguare</p><p
                                                                        class="dp33">averiguáremos</p><p class="dp33">averiguareis</p><p
                                                                        class="dp33">averiguaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">averigua (averiguá), averiguad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">averiguar</p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33">averiguado</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">averiguando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n7">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>7. ACTUAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>u</i> que precede a la desinencia es tónica en las formas<br>de este verbo que llevan el acento prosódico en la raíz.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">actúo</p><p
                                                                        class="dp33">actúas (actuás)</p><p class="dp33">actúa</p><p
                                                                        class="dp33">actuamos</p><p
                                                                        class="dp33">actuáis</p><p
                                                                        class="dp33">actúan</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">actuaba</p><p
                                                                        class="dp33">actuabas</p><p
                                                                        class="dp33">actuaba</p><p class="dp33">actuábamos</p><p
                                                                        class="dp33">actuabais</p><p class="dp33">actuaban</p></td><td
                                                                        class="dp39"><p class="dp33">actué</p><p
                                                                        class="dp33">actuaste</p><p
                                                                        class="dp33">actuó</p><p
                                                                        class="dp33">actuamos</p><p class="dp33">actuasteis</p><p
                                                                        class="dp33">actuaron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">actuaré</p><p
                                                                        class="dp33">actuarás</p><p
                                                                        class="dp33">actuará</p><p class="dp33">actuaremos</p><p
                                                                        class="dp33">actuaréis</p><p class="dp33">actuarán</p></td><td
                                                                        class="dp39"><p class="dp33">actuaría</p><p
                                                                        class="dp33">actuarías</p><p class="dp33">actuaría</p><p
                                                                        class="dp33">actuaríamos</p><p class="dp33">actuaríais</p><p
                                                                        class="dp33">actuarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">actúe</p><p class="dp33">actúes</p><p
                                                                        class="dp33">actúe</p><p
                                                                        class="dp33">actuemos</p><p
                                                                        class="dp33">actuéis</p><p
                                                                        class="dp33">actúen</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">actuara <i>o</i> actuase</p><p
                                                                        class="dp33">actuaras <i>o</i> actuases</p><p
                                                                        class="dp33">actuara <i>o</i> actuase</p><p
                                                                        class="dp33">actuáramos <i>o</i> actuásemos</p><p
                                                                        class="dp33">actuarais <i>o</i> actuaseis</p><p
                                                                        class="dp33">actuaran <i>o</i> actuasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">actuare</p><p
                                                                        class="dp33">actuares</p><p
                                                                        class="dp33">actuare</p><p class="dp33">actuáremos</p><p
                                                                        class="dp33">actuareis</p><p class="dp33">actuaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">actúa (actuá), actuad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">actuar</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">actuado</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">actuando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n8">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>8. BAILAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>i</i> del grupo /ai/ es átona<br>en todas las formas de este verbo.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">bailo</p><p
                                                                        class="dp33">bailas (bailás)</p><p class="dp33">baila</p><p
                                                                        class="dp33">bailamos</p><p
                                                                        class="dp33">bailáis</p><p
                                                                        class="dp33">bailan</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">bailaba</p><p
                                                                        class="dp33">bailabas</p><p
                                                                        class="dp33">bailaba</p><p class="dp33">bailábamos</p><p
                                                                        class="dp33">bailabais</p><p class="dp33">bailaban</p></td><td
                                                                        class="dp39"><p class="dp33">bailé</p><p
                                                                        class="dp33">bailaste</p><p
                                                                        class="dp33">bailó</p><p
                                                                        class="dp33">bailamos</p><p class="dp33">bailasteis</p><p
                                                                        class="dp33">bailaron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">bailaré</p><p
                                                                        class="dp33">bailarás</p><p
                                                                        class="dp33">bailará</p><p class="dp33">bailaremos</p><p
                                                                        class="dp33">bailaréis</p><p class="dp33">bailarán</p></td><td
                                                                        class="dp39"><p class="dp33">bailaría</p><p
                                                                        class="dp33">bailarías</p><p class="dp33">bailaría</p><p
                                                                        class="dp33">bailaríamos</p><p class="dp33">bailaríais</p><p
                                                                        class="dp33">bailarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">baile</p><p class="dp33">bailes</p><p
                                                                        class="dp33">baile</p><p
                                                                        class="dp33">bailemos</p><p
                                                                        class="dp33">bailéis</p><p
                                                                        class="dp33">bailen</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">bailara <i>o</i> bailase</p><p
                                                                        class="dp33">bailaras <i>o</i> bailases</p><p
                                                                        class="dp33">bailara <i>o</i> bailase</p><p
                                                                        class="dp33">bailáramos <i>o</i> bailásemos</p><p
                                                                        class="dp33">bailarais <i>o</i> bailaseis</p><p
                                                                        class="dp33">bailaran <i>o</i> bailasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">bailare</p><p
                                                                        class="dp33">bailares</p><p
                                                                        class="dp33">bailare</p><p class="dp33">bailáremos</p><p
                                                                        class="dp33">bailareis</p><p class="dp33">bailaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">baila (bailá), bailad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">bailar</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">bailado</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">bailando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n9">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>9. AISLAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>i</i> del grupo /ai/ es tónica en las formas de este<br>verbo que llevan el acento prosódico en la raíz.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">aíslo</p><p
                                                                        class="dp33">aíslas (aislás)</p><p class="dp33">aísla</p><p
                                                                        class="dp33">aislamos</p><p
                                                                        class="dp33">aisláis</p><p
                                                                        class="dp33">aíslan</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">aislaba</p><p
                                                                        class="dp33">aislabas</p><p
                                                                        class="dp33">aislaba</p><p class="dp33">aislábamos</p><p
                                                                        class="dp33">aislabais</p><p class="dp33">aislaban</p></td><td
                                                                        class="dp39"><p class="dp33">aislé</p><p
                                                                        class="dp33">aislaste</p><p
                                                                        class="dp33">aisló</p><p
                                                                        class="dp33">aislamos</p><p class="dp33">aislasteis</p><p
                                                                        class="dp33">aislaron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">aislaré</p><p
                                                                        class="dp33">aislarás</p><p
                                                                        class="dp33">aislará</p><p class="dp33">aislaremos</p><p
                                                                        class="dp33">aislaréis</p><p class="dp33">aislarán</p></td><td
                                                                        class="dp39"><p class="dp33">aislaría</p><p
                                                                        class="dp33">aislarías</p><p class="dp33">aislaría</p><p
                                                                        class="dp33">aislaríamos</p><p class="dp33">aislaríais</p><p
                                                                        class="dp33">aislarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">aísle</p><p class="dp33">aísles</p><p
                                                                        class="dp33">aísle</p><p
                                                                        class="dp33">aislemos</p><p
                                                                        class="dp33">aisléis</p><p
                                                                        class="dp33">aíslen</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">aislara <i>o</i> aislase</p><p
                                                                        class="dp33">aislaras <i>o</i> aislases</p><p
                                                                        class="dp33">aislara <i>o</i> aislase</p><p
                                                                        class="dp33">aisláramos <i>o</i> aislásemos</p><p
                                                                        class="dp33">aislarais <i>o</i> aislaseis</p><p
                                                                        class="dp33">aislaran <i>o</i> aislasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">aislare</p><p
                                                                        class="dp33">aislares</p><p
                                                                        class="dp33">aislare</p><p class="dp33">aisláremos</p><p
                                                                        class="dp33">aislareis</p><p class="dp33">aislaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">aísla (aislá), aislad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">aislar</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">aislado</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">aislando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n10">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>10. CAUSAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>u</i> del grupo /au/ es átona en<br>todas las formas de este verbo.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">causo</p><p
                                                                        class="dp33">causas (causás)</p><p class="dp33">causa</p><p
                                                                        class="dp33">causamos</p><p
                                                                        class="dp33">causáis</p><p
                                                                        class="dp33">causan</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">causaba</p><p
                                                                        class="dp33">causabas</p><p
                                                                        class="dp33">causaba</p><p class="dp33">causábamos</p><p
                                                                        class="dp33">causabais</p><p class="dp33">causaban</p></td><td
                                                                        class="dp39"><p class="dp33">causé</p><p
                                                                        class="dp33">causaste</p><p
                                                                        class="dp33">causó</p><p
                                                                        class="dp33">causamos</p><p class="dp33">causasteis</p><p
                                                                        class="dp33">causaron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">causaré</p><p
                                                                        class="dp33">causarás</p><p
                                                                        class="dp33">causará</p><p class="dp33">causaremos</p><p
                                                                        class="dp33">causaréis</p><p class="dp33">causarán</p></td><td
                                                                        class="dp39"><p class="dp33">causaría</p><p
                                                                        class="dp33">causarías</p><p class="dp33">causaría</p><p
                                                                        class="dp33">causaríamos</p><p class="dp33">causaríais</p><p
                                                                        class="dp33">causarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">cause</p><p class="dp33">causes</p><p
                                                                        class="dp33">cause</p><p
                                                                        class="dp33">causemos</p><p
                                                                        class="dp33">causéis</p><p
                                                                        class="dp33">causen</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">causara <i>o</i> causase</p><p
                                                                        class="dp33">causaras <i>o</i> causases</p><p
                                                                        class="dp33">causara <i>o</i> causase</p><p
                                                                        class="dp33">causáramos <i>o</i> causásemos</p><p
                                                                        class="dp33">causarais <i>o</i> causaseis</p><p
                                                                        class="dp33">causaran <i>o</i> causasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">causare</p><p
                                                                        class="dp33">causares</p><p
                                                                        class="dp33">causare</p><p class="dp33">causáremos</p><p
                                                                        class="dp33">causareis</p><p class="dp33">causaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">causa (causá), causad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">causar</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">causado</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">causando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n11">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>11. AUNAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>u</i> del grupo /au/ es tónica en las formas de este<br>verbo que llevan el acento prosódico en la raíz.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">aúno</p><p
                                                                        class="dp33">aúnas (aunás)</p><p class="dp33">aúna</p><p
                                                                        class="dp33">aunamos</p><p
                                                                        class="dp33">aunáis</p><p class="dp33">aúnan</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">aunaba</p><p
                                                                        class="dp33">aunabas</p><p
                                                                        class="dp33">aunaba</p><p
                                                                        class="dp33">aunábamos</p><p class="dp33">aunabais</p><p
                                                                        class="dp33">aunaban</p></td><td class="dp39"><p
                                                                        class="dp33">auné</p><p class="dp33">aunaste</p><p
                                                                        class="dp33">aunó</p><p class="dp33">aunamos</p><p
                                                                        class="dp33">aunasteis</p><p class="dp33">aunaron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">aunaré</p><p
                                                                        class="dp33">aunarás</p><p
                                                                        class="dp33">aunará</p><p
                                                                        class="dp33">aunaremos</p><p class="dp33">aunaréis</p><p
                                                                        class="dp33">aunarán</p></td><td class="dp39"><p
                                                                        class="dp33">aunaría</p><p
                                                                        class="dp33">aunarías</p><p
                                                                        class="dp33">aunaría</p><p class="dp33">aunaríamos</p><p
                                                                        class="dp33">aunaríais</p><p class="dp33">aunarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">aúne</p><p class="dp33">aúnes</p><p
                                                                        class="dp33">aúne</p><p class="dp33">aunemos</p><p
                                                                        class="dp33">aunéis</p><p class="dp33">aúnen</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">aunara <i>o</i> aunase</p><p
                                                                        class="dp33">aunaras <i>o</i> aunases</p><p
                                                                        class="dp33">aunara <i>o</i> aunase</p><p
                                                                        class="dp33">aunáramos <i>o</i> aunásemos</p><p
                                                                        class="dp33">aunarais <i>o</i> aunaseis</p><p
                                                                        class="dp33">aunaran <i>o</i> aunasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">aunare</p><p
                                                                        class="dp33">aunares</p><p
                                                                        class="dp33">aunare</p><p
                                                                        class="dp33">aunáremos</p><p class="dp33">aunareis</p><p
                                                                        class="dp33">aunaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">aúna (auná), aunad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">aunar</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">aunado</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">aunando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n12">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>12. PEINAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>i</i> del grupo /ei/ es átona<br>en todas las formas de este verbo.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">peino</p><p
                                                                        class="dp33">peinas (peinás)</p><p class="dp33">peina</p><p
                                                                        class="dp33">peinamos</p><p
                                                                        class="dp33">peináis</p><p
                                                                        class="dp33">peinan</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">peinaba</p><p
                                                                        class="dp33">peinabas</p><p
                                                                        class="dp33">peinaba</p><p class="dp33">peinábamos</p><p
                                                                        class="dp33">peinabais</p><p class="dp33">peinaban</p></td><td
                                                                        class="dp39"><p class="dp33">peiné</p><p
                                                                        class="dp33">peinaste</p><p
                                                                        class="dp33">peinó</p><p
                                                                        class="dp33">peinamos</p><p class="dp33">peinasteis</p><p
                                                                        class="dp33">peinaron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">peinaré</p><p
                                                                        class="dp33">peinarás</p><p
                                                                        class="dp33">peinará</p><p class="dp33">peinaremos</p><p
                                                                        class="dp33">peinaréis</p><p class="dp33">peinarán</p></td><td
                                                                        class="dp39"><p class="dp33">peinaría</p><p
                                                                        class="dp33">peinarías</p><p class="dp33">peinaría</p><p
                                                                        class="dp33">peinaríamos</p><p class="dp33">peinaríais</p><p
                                                                        class="dp33">peinarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">peine</p><p class="dp33">peines</p><p
                                                                        class="dp33">peine</p><p
                                                                        class="dp33">peinemos</p><p
                                                                        class="dp33">peinéis</p><p
                                                                        class="dp33">peinen</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">peinara <i>o</i> peinase</p><p
                                                                        class="dp33">peinaras <i>o</i> peinases</p><p
                                                                        class="dp33">peinara <i>o</i> peinase</p><p
                                                                        class="dp33">peináramos <i>o</i> peinásemos</p><p
                                                                        class="dp33">peinarais <i>o</i> peinaseis</p><p
                                                                        class="dp33">peinaran <i>o</i> peinasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">peinare</p><p
                                                                        class="dp33">peinares</p><p
                                                                        class="dp33">peinare</p><p class="dp33">peináremos</p><p
                                                                        class="dp33">peinareis</p><p class="dp33">peinaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">peina (peiná), peinad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">peinar</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">peinado</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">peinando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n13">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>13. DESCAFEINAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>i</i> del grupo /ei/ es tónica en las formas de este<br>verbo que llevan el acento prosódico en la raíz.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">descafeíno</p><p
                                                                        class="dp33">descafeínas (descafeinás)</p><p
                                                                        class="dp33">descafeína</p><p class="dp33">descafeinamos</p><p
                                                                        class="dp33">descafeináis</p><p class="dp33">descafeínan</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">descafeinaba</p><p
                                                                        class="dp33">descafeinabas</p><p class="dp33">descafeinaba</p><p
                                                                        class="dp33">descafeinábamos</p><p class="dp33">descafeinabais</p><p
                                                                        class="dp33">descafeinaban</p></td><td
                                                                        class="dp39"><p class="dp33">descafeiné</p><p
                                                                        class="dp33">descafeinaste</p><p class="dp33">descafeinó</p><p
                                                                        class="dp33">descafeinamos</p><p class="dp33">descafeinasteis</p><p
                                                                        class="dp33">descafeinaron</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">descafeinaré</p><p
                                                                        class="dp33">descafeinarás</p><p class="dp33">descafeinará</p><p
                                                                        class="dp33">descafeinaremos</p><p class="dp33">descafeinaréis</p><p
                                                                        class="dp33">descafeinarán</p></td><td
                                                                        class="dp39"><p class="dp33">descafeinaría</p><p
                                                                        class="dp33">descafeinarías</p><p class="dp33">descafeinaría</p><p
                                                                        class="dp33">descafeinaríamos</p><p
                                                                        class="dp33">descafeinaríais</p><p class="dp33">descafeinarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">descafeíne</p><p
                                                                        class="dp33">descafeínes</p><p class="dp33">descafeíne</p><p
                                                                        class="dp33">descafeinemos</p><p class="dp33">descafeinéis</p><p
                                                                        class="dp33">descafeínen</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">descafeinara <i>o</i> descafeinase</p><p
                                                                        class="dp33">descafeinaras <i>o</i> descafeinases</p><p
                                                                        class="dp33">descafeinara <i>o</i> descafeinase</p><p
                                                                        class="dp33">descafeináramos <i>o</i> descafeinásemos</p><p
                                                                        class="dp33">descafeinarais <i>o</i> descafeinaseis</p><p
                                                                        class="dp33">descafeinaran <i>o</i> descafeinasen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">descafeinare</p><p
                                                                        class="dp33">descafeinares</p><p class="dp33">descafeinare</p><p
                                                                        class="dp33">descafeináremos</p><p class="dp33">descafeinareis</p><p
                                                                        class="dp33">descafeinaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">descafeína (descafeiná), descafeinad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">descafeinar</p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33">descafeinado</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">descafeinando</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n14">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>14. ADEUDAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>u</i> del grupo /eu/ es átona en<br>todas las formas de este verbo.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">adeudo</p><p
                                                                        class="dp33">adeudas (adeudás)</p><p
                                                                        class="dp33">adeuda</p><p
                                                                        class="dp33">adeudamos</p><p class="dp33">adeudáis</p><p
                                                                        class="dp33">adeudan</p></td><td class="dp23"
                                                                                                         colspan="3"><p
                                                                        class="dp33">adeudaba</p><p class="dp33">adeudabas</p><p
                                                                        class="dp33">adeudaba</p><p class="dp33">adeudábamos</p><p
                                                                        class="dp33">adeudabais</p><p class="dp33">adeudaban</p></td><td
                                                                        class="dp39"><p class="dp33">adeudé</p><p
                                                                        class="dp33">adeudaste</p><p
                                                                        class="dp33">adeudó</p><p
                                                                        class="dp33">adeudamos</p><p class="dp33">adeudasteis</p><p
                                                                        class="dp33">adeudaron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">adeudaré</p><p class="dp33">adeudarás</p><p
                                                                        class="dp33">adeudará</p><p class="dp33">adeudaremos</p><p
                                                                        class="dp33">adeudaréis</p><p class="dp33">adeudarán</p></td><td
                                                                        class="dp39"><p class="dp33">adeudaría</p><p
                                                                        class="dp33">adeudarías</p><p class="dp33">adeudaría</p><p
                                                                        class="dp33">adeudaríamos</p><p class="dp33">adeudaríais</p><p
                                                                        class="dp33">adeudarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">adeude</p><p
                                                                        class="dp33">adeudes</p><p
                                                                        class="dp33">adeude</p><p
                                                                        class="dp33">adeudemos</p><p class="dp33">adeudéis</p><p
                                                                        class="dp33">adeuden</p></td><td class="dp22"
                                                                                                         colspan="4"><p
                                                                        class="dp33">adeudara <i>o</i> adeudase</p><p
                                                                        class="dp33">adeudaras <i>o</i> adeudases</p><p
                                                                        class="dp33">adeudara <i>o</i> adeudase</p><p
                                                                        class="dp33">adeudáramos <i>o</i> adeudásemos</p><p
                                                                        class="dp33">adeudarais <i>o</i> adeudaseis</p><p
                                                                        class="dp33">adeudaran <i>o</i> adeudasen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">adeudare</p><p
                                                                        class="dp33">adeudares</p><p class="dp33">adeudare</p><p
                                                                        class="dp33">adeudáremos</p><p class="dp33">adeudareis</p><p
                                                                        class="dp33">adeudaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">adeuda (adeudá), adeudad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">adeudar</p></td><td class="dp26"
                                                                                                         coslpan="3"><p
                                                                        class="dp33">adeudado</p></td><td class="dp42"
                                                                                                          colspan="2"><p
                                                                        class="dp33">adeudando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n15">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="4"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>15. REHUSAR</b></td><td
                                                                        colspan="4" valign="middle" width="60%"><p
                                                                        class="dp33">La <i>u</i> del grupo /eu/ es tónica en las formas de este<br>verbo que llevan el acento prosódico en la raíz.</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">rehúso</p><p
                                                                        class="dp33">rehúsas (rehusás)</p><p
                                                                        class="dp33">rehúsa</p><p
                                                                        class="dp33">rehusamos</p><p class="dp33">rehusáis</p><p
                                                                        class="dp33">rehúsan</p></td><td class="dp23"
                                                                                                         colspan="3"><p
                                                                        class="dp33">rehusaba</p><p class="dp33">rehusabas</p><p
                                                                        class="dp33">rehusaba</p><p class="dp33">rehusábamos</p><p
                                                                        class="dp33">rehusabais</p><p class="dp33">rehusaban</p></td><td
                                                                        class="dp39"><p class="dp33">rehusé</p><p
                                                                        class="dp33">rehusaste</p><p
                                                                        class="dp33">rehusó</p><p
                                                                        class="dp33">rehusamos</p><p class="dp33">rehusasteis</p><p
                                                                        class="dp33">rehusaron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">rehusaré</p><p class="dp33">rehusarás</p><p
                                                                        class="dp33">rehusará</p><p class="dp33">rehusaremos</p><p
                                                                        class="dp33">rehusaréis</p><p class="dp33">rehusarán</p></td><td
                                                                        class="dp39"><p class="dp33">rehusaría</p><p
                                                                        class="dp33">rehusarías</p><p class="dp33">rehusaría</p><p
                                                                        class="dp33">rehusaríamos</p><p class="dp33">rehusaríais</p><p
                                                                        class="dp33">rehusarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">rehúse</p><p
                                                                        class="dp33">rehúses</p><p
                                                                        class="dp33">rehúse</p><p
                                                                        class="dp33">rehusemos</p><p class="dp33">rehuséis</p><p
                                                                        class="dp33">rehúsen</p></td><td class="dp22"
                                                                                                         colspan="4"><p
                                                                        class="dp33">rehusara <i>o</i> rehusase</p><p
                                                                        class="dp33">rehusaras <i>o</i> rehusases</p><p
                                                                        class="dp33">rehusara <i>o</i> rehusase</p><p
                                                                        class="dp33">rehusáramos <i>o</i> rehusásemos</p><p
                                                                        class="dp33">rehusarais <i>o</i> rehusaseis</p><p
                                                                        class="dp33">rehusaran <i>o</i> rehusasen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">rehusare</p><p
                                                                        class="dp33">rehusares</p><p class="dp33">rehusare</p><p
                                                                        class="dp33">rehusáremos</p><p class="dp33">rehusareis</p><p
                                                                        class="dp33">rehusaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">rehúsa (rehusá), rehusad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">rehusar</p></td><td class="dp26"
                                                                                                         coslpan="3"><p
                                                                        class="dp33">rehusado</p></td><td class="dp42"
                                                                                                          colspan="2"><p
                                                                        class="dp33">rehusando</p></td></tr></tbody></table><p></p><p
                                                                        class="dp35"><b><a name="verirregc">II. Verbos irregulares</a></b></p><p
                                                                        class="dp30">&nbsp;&nbsp;Se incluyen bajo este epígrafe tanto los verbos de irregularidad propia, cuyo paradigma es único (<i>ir, ser,</i> etc.), como los que sirven de modelo a otros verbos irregulares (<i>acertar, agradecer,</i> etc.). También se incluye aquí el verbo <i>leer</i> —modelo de otros verbos como <i>creer</i> o <i>proveer</i>—, que aun siendo regular desde el punto de vista morfológico, no lo es desde el punto de vista gráfico-articulatorio, ya que el sonido vocálico /i/ de algunas desinencias, cuando queda entre vocales, se transforma en el sonido consonántico /y/; así, la raíz <i>le-</i> + la desinencia <i>-ió</i> no da <i>leió</i>, sino <i>leyó</i>; <i>le-</i> + <i>-iera</i> no da <i>leiera,</i> sino <i>leyera,</i> etc.</p><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n16">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>16. ACERTAR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">acierto</p><p
                                                                        class="dp33">aciertas (acertás)</p><p
                                                                        class="dp33">acierta</p><p class="dp33">acertamos</p><p
                                                                        class="dp33">acertáis</p><p class="dp33">aciertan</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">acertaba</p><p
                                                                        class="dp33">acertabas</p><p class="dp33">acertaba</p><p
                                                                        class="dp33">acertábamos</p><p class="dp33">acertabais</p><p
                                                                        class="dp33">acertaban</p></td><td class="dp39"><p
                                                                        class="dp33">acerté</p><p
                                                                        class="dp33">acertaste</p><p
                                                                        class="dp33">acertó</p><p
                                                                        class="dp33">acertamos</p><p class="dp33">acertasteis</p><p
                                                                        class="dp33">acertaron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">acertaré</p><p class="dp33">acertarás</p><p
                                                                        class="dp33">acertará</p><p class="dp33">acertaremos</p><p
                                                                        class="dp33">acertaréis</p><p class="dp33">acertarán</p></td><td
                                                                        class="dp39"><p class="dp33">acertaría</p><p
                                                                        class="dp33">acertarías</p><p class="dp33">acertaría</p><p
                                                                        class="dp33">acertaríamos</p><p class="dp33">acertaríais</p><p
                                                                        class="dp33">acertarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">acierte</p><p
                                                                        class="dp33">aciertes</p><p
                                                                        class="dp33">acierte</p><p class="dp33">acertemos</p><p
                                                                        class="dp33">acertéis</p><p class="dp33">acierten</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">acertara <i>o</i> acertase</p><p
                                                                        class="dp33">acertaras <i>o</i> acertases</p><p
                                                                        class="dp33">acertara <i>o</i> acertase</p><p
                                                                        class="dp33">acertáramos <i>o</i> acertásemos</p><p
                                                                        class="dp33">acertarais <i>o</i> acertaseis</p><p
                                                                        class="dp33">acertaran <i>o</i> acertasen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">acertare</p><p
                                                                        class="dp33">acertares</p><p class="dp33">acertare</p><p
                                                                        class="dp33">acertáremos</p><p class="dp33">acertareis</p><p
                                                                        class="dp33">acertaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">acierta (acertá), acertad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">acertar</p></td><td class="dp26"
                                                                                                         coslpan="3"><p
                                                                        class="dp33">acertado</p></td><td class="dp42"
                                                                                                          colspan="2"><p
                                                                        class="dp33">acertando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n17">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>17. ADQUIRIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">adquiero</p><p
                                                                        class="dp33">adquieres (adquirís)</p><p
                                                                        class="dp33">adquiere</p><p class="dp33">adquirimos</p><p
                                                                        class="dp33">adquirís</p><p class="dp33">adquieren</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">adquiría</p><p
                                                                        class="dp33">adquirías</p><p class="dp33">adquiría</p><p
                                                                        class="dp33">adquiríamos</p><p class="dp33">adquiríais</p><p
                                                                        class="dp33">adquirían</p></td><td class="dp39"><p
                                                                        class="dp33">adquirí</p><p class="dp33">adquiriste</p><p
                                                                        class="dp33">adquirió</p><p class="dp33">adquirimos</p><p
                                                                        class="dp33">adquiristeis</p><p class="dp33">adquirieron</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">adquiriré</p><p
                                                                        class="dp33">adquirirás</p><p class="dp33">adquirirá</p><p
                                                                        class="dp33">adquiriremos</p><p class="dp33">adquiriréis</p><p
                                                                        class="dp33">adquirirán</p></td><td
                                                                        class="dp39"><p class="dp33">adquiriría</p><p
                                                                        class="dp33">adquirirías</p><p class="dp33">adquiriría</p><p
                                                                        class="dp33">adquiriríamos</p><p class="dp33">adquiriríais</p><p
                                                                        class="dp33">adquirirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">adquiera</p><p
                                                                        class="dp33">adquieras</p><p class="dp33">adquiera</p><p
                                                                        class="dp33">adquiramos</p><p class="dp33">adquiráis</p><p
                                                                        class="dp33">adquieran</p></td><td class="dp22"
                                                                                                           colspan="4"><p
                                                                        class="dp33">adquiriera <i>o</i> adquiriese</p><p
                                                                        class="dp33">adquirieras <i>o</i> adquirieses</p><p
                                                                        class="dp33">adquiriera <i>o</i> adquiriese</p><p
                                                                        class="dp33">adquiriéramos <i>o</i> adquiriésemos</p><p
                                                                        class="dp33">adquirierais <i>o</i> adquirieseis</p><p
                                                                        class="dp33">adquirieran <i>o</i> adquiriesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">adquiriere</p><p
                                                                        class="dp33">adquirieres</p><p class="dp33">adquiriere</p><p
                                                                        class="dp33">adquiriéremos</p><p class="dp33">adquiriereis</p><p
                                                                        class="dp33">adquirieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="8" valign="middle">adquiere (adquirí), adquirid</td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">adquirir</p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33">adquirido</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">adquiriendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n18">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>18. AGRADECER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">agradezco</p><p
                                                                        class="dp33">agradeces (agradecés)</p><p
                                                                        class="dp33">agradece</p><p class="dp33">agradecemos</p><p
                                                                        class="dp33">agradecéis</p><p class="dp33">agradecen</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">agradecía</p><p
                                                                        class="dp33">agradecías</p><p class="dp33">agradecía</p><p
                                                                        class="dp33">agradecíamos</p><p class="dp33">agradecíais</p><p
                                                                        class="dp33">agradecían</p></td><td
                                                                        class="dp39"><p class="dp33">agradecí</p><p
                                                                        class="dp33">agradeciste</p><p class="dp33">agradeció</p><p
                                                                        class="dp33">agradecimos</p><p class="dp33">agradecisteis</p><p
                                                                        class="dp33">agradecieron</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">agradeceré</p><p
                                                                        class="dp33">agradecerás</p><p class="dp33">agradecerá</p><p
                                                                        class="dp33">agradeceremos</p><p class="dp33">agradeceréis</p><p
                                                                        class="dp33">agradecerán</p></td><td
                                                                        class="dp39"><p class="dp33">agradecería</p><p
                                                                        class="dp33">agradecerías</p><p class="dp33">agradecería</p><p
                                                                        class="dp33">agradeceríamos</p><p class="dp33">agradeceríais</p><p
                                                                        class="dp33">agradecerían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">agradezca</p><p
                                                                        class="dp33">agradezcas</p><p class="dp33">agradezca</p><p
                                                                        class="dp33">agradezcamos</p><p class="dp33">agradezcáis</p><p
                                                                        class="dp33">agradezcan</p></td><td class="dp22"
                                                                                                            colspan="4"><p
                                                                        class="dp33">agradeciera <i>o</i> agradeciese</p><p
                                                                        class="dp33">agradecieras <i>o</i> agradecieses</p><p
                                                                        class="dp33">agradeciera <i>o</i> agradeciese</p><p
                                                                        class="dp33">agradeciéramos <i>o</i> agradeciésemos</p><p
                                                                        class="dp33">agradecierais <i>o</i> agradecieseis</p><p
                                                                        class="dp33">agradecieran <i>o</i> agradeciesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">agradeciere</p><p
                                                                        class="dp33">agradecieres</p><p class="dp33">agradeciere</p><p
                                                                        class="dp33">agradeciéremos</p><p class="dp33">agradeciereis</p><p
                                                                        class="dp33">agradecieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="8" valign="middle">agradece (agradecé), agradeced</td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">agradecer</p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33">agradecido</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">agradeciendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n19">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>19. ANDAR </b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">ando</p><p
                                                                        class="dp33">andas (andás)</p><p class="dp33">anda</p><p
                                                                        class="dp33">andamos</p><p
                                                                        class="dp33">andáis</p><p class="dp33">andan</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">andaba</p><p
                                                                        class="dp33">andabas</p><p
                                                                        class="dp33">andaba</p><p
                                                                        class="dp33">andábamos</p><p class="dp33">andabais</p><p
                                                                        class="dp33">andaban</p></td><td class="dp39"><p
                                                                        class="dp33">anduve</p><p
                                                                        class="dp33">anduviste</p><p
                                                                        class="dp33">anduvo</p><p
                                                                        class="dp33">anduvimos</p><p class="dp33">anduvisteis</p><p
                                                                        class="dp33">anduvieron</p></td><td class="dp40"
                                                                                                            colspan="2"><p
                                                                        class="dp33">andaré</p><p
                                                                        class="dp33">andarás</p><p
                                                                        class="dp33">andará</p><p
                                                                        class="dp33">andaremos</p><p class="dp33">andaréis</p><p
                                                                        class="dp33">andarán</p></td><td class="dp39"><p
                                                                        class="dp33">andaría</p><p
                                                                        class="dp33">andarías</p><p
                                                                        class="dp33">andaría</p><p class="dp33">andaríamos</p><p
                                                                        class="dp33">andaríais</p><p class="dp33">andarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">ande</p><p class="dp33">andes</p><p
                                                                        class="dp33">ande</p><p class="dp33">andemos</p><p
                                                                        class="dp33">andéis</p><p class="dp33">anden</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">anduviera <i>o</i> anduviese</p><p
                                                                        class="dp33">anduvieras <i>o</i> anduvieses</p><p
                                                                        class="dp33">anduviera <i>o</i> anduviese</p><p
                                                                        class="dp33">anduviéramos <i>o</i> anduviésemos</p><p
                                                                        class="dp33">anduvierais <i>o</i> anduvieseis</p><p
                                                                        class="dp33">anduvieran <i>o</i> anduviesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">anduviere</p><p
                                                                        class="dp33">anduvieres</p><p class="dp33">anduviere</p><p
                                                                        class="dp33">anduviéremos</p><p class="dp33">anduviereis</p><p
                                                                        class="dp33">anduvieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">anda (andá), andad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">andar</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">andado</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">andando</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n20">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>20. ASIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">asgo</p><p
                                                                        class="dp33">ases (asís)</p><p
                                                                        class="dp33">ase</p><p class="dp33">asimos</p><p
                                                                        class="dp33">asís</p><p
                                                                        class="dp33">asen</p></td><td class="dp23"
                                                                                                      colspan="3"><p
                                                                        class="dp33">asía</p><p class="dp33">asías</p><p
                                                                        class="dp33">asía</p><p class="dp33">asíamos</p><p
                                                                        class="dp33">asíais</p><p class="dp33">asían</p></td><td
                                                                        class="dp39"><p class="dp33">así</p><p
                                                                        class="dp33">asiste</p><p
                                                                        class="dp33">asió</p><p
                                                                        class="dp33">asimos</p><p
                                                                        class="dp33">asisteis</p><p
                                                                        class="dp33">asieron</p></td><td class="dp40"
                                                                                                         colspan="2"><p
                                                                        class="dp33">asiré</p><p class="dp33">asirás</p><p
                                                                        class="dp33">asirá</p><p
                                                                        class="dp33">asiremos</p><p
                                                                        class="dp33">asiréis</p><p
                                                                        class="dp33">asirán</p></td><td class="dp39"><p
                                                                        class="dp33">asiría</p><p
                                                                        class="dp33">asirías</p><p
                                                                        class="dp33">asiría</p><p
                                                                        class="dp33">asiríamos</p><p class="dp33">asiríais</p><p
                                                                        class="dp33">asirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">asga</p><p class="dp33">asgas</p><p
                                                                        class="dp33">asga</p><p class="dp33">asgamos</p><p
                                                                        class="dp33">asgáis</p><p class="dp33">asgan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">asiera <i>o</i> asiese</p><p
                                                                        class="dp33">asieras <i>o</i> asieses</p><p
                                                                        class="dp33">asiera <i>o</i> asiese</p><p
                                                                        class="dp33">asiéramos <i>o</i> asiésemos</p><p
                                                                        class="dp33">asierais <i>o</i> asieseis</p><p
                                                                        class="dp33">asieran <i>o</i> asiesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">asiere</p><p
                                                                        class="dp33">asieres</p><p
                                                                        class="dp33">asiere</p><p
                                                                        class="dp33">asiéremos</p><p class="dp33">asiereis</p><p
                                                                        class="dp33">asieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">ase (así), asid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">asir</p></td><td class="dp26"
                                                                                                      coslpan="3"><p
                                                                        class="dp33">asido</p></td><td class="dp42"
                                                                                                       colspan="2"><p
                                                                        class="dp33">asiendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n21">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>21. CABER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">quepo</p><p
                                                                        class="dp33">cabes (cabés)</p><p class="dp33">cabe</p><p
                                                                        class="dp33">cabemos</p><p
                                                                        class="dp33">cabéis</p><p class="dp33">caben</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">cabía</p><p class="dp33">cabías</p><p
                                                                        class="dp33">cabía</p><p
                                                                        class="dp33">cabíamos</p><p
                                                                        class="dp33">cabíais</p><p
                                                                        class="dp33">cabían</p></td><td class="dp39"><p
                                                                        class="dp33">cupe</p><p class="dp33">cupiste</p><p
                                                                        class="dp33">cupo</p><p class="dp33">cupimos</p><p
                                                                        class="dp33">cupisteis</p><p class="dp33">cupieron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">cabré</p><p class="dp33">cabrás</p><p
                                                                        class="dp33">cabrá</p><p
                                                                        class="dp33">cabremos</p><p
                                                                        class="dp33">cabréis</p><p
                                                                        class="dp33">cabrán</p></td><td class="dp39"><p
                                                                        class="dp33">cabría</p><p
                                                                        class="dp33">cabrías</p><p
                                                                        class="dp33">cabría</p><p
                                                                        class="dp33">cabríamos</p><p class="dp33">cabríais</p><p
                                                                        class="dp33">cabrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">quepa</p><p class="dp33">quepas</p><p
                                                                        class="dp33">quepa</p><p
                                                                        class="dp33">quepamos</p><p
                                                                        class="dp33">quepáis</p><p
                                                                        class="dp33">quepan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">cupiera <i>o</i> cupiese</p><p
                                                                        class="dp33">cupieras <i>o</i> cupieses</p><p
                                                                        class="dp33">cupiera <i>o</i> cupiese</p><p
                                                                        class="dp33">cupiéramos <i>o</i> cupiésemos</p><p
                                                                        class="dp33">cupierais <i>o</i> cupieseis</p><p
                                                                        class="dp33">cupieran <i>o</i> cupiesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">cupiere</p><p
                                                                        class="dp33">cupieres</p><p
                                                                        class="dp33">cupiere</p><p class="dp33">cupiéremos</p><p
                                                                        class="dp33">cupiereis</p><p class="dp33">cupieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33"><i>No se usa</i></p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">caber</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">cabido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">cabiendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n22">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>22. CAER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">caigo</p><p
                                                                        class="dp33">caes (caés)</p><p
                                                                        class="dp33">cae</p><p class="dp33">caemos</p><p
                                                                        class="dp33">caéis</p><p
                                                                        class="dp33">caen</p></td><td class="dp23"
                                                                                                      colspan="3"><p
                                                                        class="dp33">caía</p><p class="dp33">caías</p><p
                                                                        class="dp33">caía</p><p class="dp33">caíamos</p><p
                                                                        class="dp33">caíais</p><p class="dp33">caían</p></td><td
                                                                        class="dp39"><p class="dp33">caí</p><p
                                                                        class="dp33">caíste</p><p
                                                                        class="dp33">cayó</p><p
                                                                        class="dp33">caímos</p><p
                                                                        class="dp33">caísteis</p><p
                                                                        class="dp33">cayeron</p></td><td class="dp40"
                                                                                                         colspan="2"><p
                                                                        class="dp33">caeré</p><p class="dp33">caerás</p><p
                                                                        class="dp33">caerá</p><p
                                                                        class="dp33">caeremos</p><p
                                                                        class="dp33">caeréis</p><p
                                                                        class="dp33">caerán</p></td><td class="dp39"><p
                                                                        class="dp33">caería</p><p
                                                                        class="dp33">caerías</p><p
                                                                        class="dp33">caería</p><p
                                                                        class="dp33">caeríamos</p><p class="dp33">caeríais</p><p
                                                                        class="dp33">caerían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">caiga</p><p class="dp33">caigas</p><p
                                                                        class="dp33">caiga</p><p
                                                                        class="dp33">caigamos</p><p
                                                                        class="dp33">caigáis</p><p
                                                                        class="dp33">caigan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">cayera <i>o</i> cayese</p><p
                                                                        class="dp33">cayeras <i>o</i> cayeses</p><p
                                                                        class="dp33">cayera <i>o</i> cayese</p><p
                                                                        class="dp33">cayéramos <i>o</i> cayésemos</p><p
                                                                        class="dp33">cayerais <i>o</i> cayeseis</p><p
                                                                        class="dp33">cayeran <i>o</i> cayesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">cayere</p><p
                                                                        class="dp33">cayeres</p><p
                                                                        class="dp33">cayere</p><p
                                                                        class="dp33">cayéremos</p><p class="dp33">cayereis</p><p
                                                                        class="dp33">cayeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">cae (caé), caed</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">caer</p></td><td class="dp26"
                                                                                                      coslpan="3"><p
                                                                        class="dp33">caído</p></td><td class="dp42"
                                                                                                       colspan="2"><p
                                                                        class="dp33">cayendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n23">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>23. CEÑIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">ciño</p><p
                                                                        class="dp33">ciñes (ceñís)</p><p class="dp33">ciñe</p><p
                                                                        class="dp33">ceñimos</p><p
                                                                        class="dp33">ceñís</p><p class="dp33">ciñen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">ceñía</p><p class="dp33">ceñías</p><p
                                                                        class="dp33">ceñía</p><p
                                                                        class="dp33">ceñíamos</p><p
                                                                        class="dp33">ceñíais</p><p
                                                                        class="dp33">ceñían</p></td><td class="dp39"><p
                                                                        class="dp33">ceñí</p><p class="dp33">ceñiste</p><p
                                                                        class="dp33">ciñó</p><p class="dp33">ceñimos</p><p
                                                                        class="dp33">ceñisteis</p><p class="dp33">ciñeron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">ceñiré</p><p
                                                                        class="dp33">ceñirás</p><p
                                                                        class="dp33">ceñirá</p><p
                                                                        class="dp33">ceñiremos</p><p class="dp33">ceñiréis</p><p
                                                                        class="dp33">ceñirán</p></td><td class="dp39"><p
                                                                        class="dp33">ceñiría</p><p
                                                                        class="dp33">ceñirías</p><p
                                                                        class="dp33">ceñiría</p><p class="dp33">ceñiríamos</p><p
                                                                        class="dp33">ceñiríais</p><p class="dp33">ceñirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">ciña</p><p class="dp33">ciñas</p><p
                                                                        class="dp33">ciña</p><p class="dp33">ciñamos</p><p
                                                                        class="dp33">ciñáis</p><p class="dp33">ciñan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">ciñera <i>o</i> ciñese</p><p
                                                                        class="dp33">ciñeras <i>o</i> ciñeses</p><p
                                                                        class="dp33">ciñera <i>o</i> ciñese</p><p
                                                                        class="dp33">ciñéramos <i>o</i> ciñésemos</p><p
                                                                        class="dp33">ciñerais <i>o</i> ciñeseis</p><p
                                                                        class="dp33">ciñeran <i>o</i> ciñesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">ciñere</p><p
                                                                        class="dp33">ciñeres</p><p
                                                                        class="dp33">ciñere</p><p
                                                                        class="dp33">ciñéremos</p><p class="dp33">ciñereis</p><p
                                                                        class="dp33">ciñeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="8" valign="middle">ciñe (ceñí) ceñid</td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">ceñir</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">ceñido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">ciñendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n24">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>24. CONDUCIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">conduzco</p><p
                                                                        class="dp33">conduces (conducís)</p><p
                                                                        class="dp33">conduce</p><p class="dp33">conducimos</p><p
                                                                        class="dp33">conducís</p><p class="dp33">conducen</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">conducía</p><p
                                                                        class="dp33">conducías</p><p class="dp33">conducía</p><p
                                                                        class="dp33">conducíamos</p><p class="dp33">conducíais</p><p
                                                                        class="dp33">conducían</p></td><td class="dp39"><p
                                                                        class="dp33">conduje</p><p class="dp33">condujiste</p><p
                                                                        class="dp33">condujo</p><p class="dp33">condujimos</p><p
                                                                        class="dp33">condujisteis</p><p class="dp33">condujeron</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">conduciré</p><p
                                                                        class="dp33">conducirás</p><p class="dp33">conducirá</p><p
                                                                        class="dp33">conduciremos</p><p class="dp33">conduciréis</p><p
                                                                        class="dp33">conducirán</p></td><td
                                                                        class="dp39"><p class="dp33">conduciría</p><p
                                                                        class="dp33">conducirías</p><p class="dp33">conduciría</p><p
                                                                        class="dp33">conduciríamos</p><p class="dp33">conduciríais</p><p
                                                                        class="dp33">conducirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">conduzca</p><p
                                                                        class="dp33">conduzcas</p><p class="dp33">conduzca</p><p
                                                                        class="dp33">conduzcamos</p><p class="dp33">conduzcáis</p><p
                                                                        class="dp33">conduzcan</p></td><td class="dp22"
                                                                                                           colspan="4"><p
                                                                        class="dp33">condujera <i>o</i> condujese</p><p
                                                                        class="dp33">condujeras <i>o</i> condujeses</p><p
                                                                        class="dp33">condujera <i>o</i> condujese</p><p
                                                                        class="dp33">condujéramos <i>o</i> condujésemos</p><p
                                                                        class="dp33">condujerais <i>o</i> condujeseis</p><p
                                                                        class="dp33">condujeran <i>o</i> condujesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">condujere</p><p
                                                                        class="dp33">condujeres</p><p class="dp33">condujere</p><p
                                                                        class="dp33">condujéremos</p><p class="dp33">condujereis</p><p
                                                                        class="dp33">condujeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="8" valign="middle">conduce (conducí), conducid</td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">conducir</p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33">conducido</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">conduciendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n25">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>25. CONSTRUIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">construyo</p><p
                                                                        class="dp33">construyes (construís)</p><p
                                                                        class="dp33">construye</p><p class="dp33">construimos</p><p
                                                                        class="dp33">construís</p><p class="dp33">construyen</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">construía</p><p
                                                                        class="dp33">construías</p><p class="dp33">construía</p><p
                                                                        class="dp33">construíamos</p><p class="dp33">construíais</p><p
                                                                        class="dp33">construían</p></td><td
                                                                        class="dp39"><p class="dp33">construí</p><p
                                                                        class="dp33">construiste</p><p class="dp33">construyó</p><p
                                                                        class="dp33">construimos</p><p class="dp33">construisteis</p><p
                                                                        class="dp33">construyeron</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">construiré</p><p
                                                                        class="dp33">construirás</p><p class="dp33">construirá</p><p
                                                                        class="dp33">construiremos</p><p class="dp33">construiréis</p><p
                                                                        class="dp33">construirán</p></td><td
                                                                        class="dp39"><p class="dp33">construiría</p><p
                                                                        class="dp33">construirías</p><p class="dp33">construiría</p><p
                                                                        class="dp33">construiríamos</p><p class="dp33">construiríais</p><p
                                                                        class="dp33">construirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">construya</p><p
                                                                        class="dp33">construyas</p><p class="dp33">construya</p><p
                                                                        class="dp33">construyamos</p><p class="dp33">construyáis</p><p
                                                                        class="dp33">construyan</p></td><td class="dp22"
                                                                                                            colspan="4"><p
                                                                        class="dp33">construyera <i>o</i> construyese</p><p
                                                                        class="dp33">construyeras <i>o</i> construyeses</p><p
                                                                        class="dp33">construyera <i>o</i> construyese</p><p
                                                                        class="dp33">construyéramos <i>o</i> construyésemos</p><p
                                                                        class="dp33">construyerais <i>o</i> construyeseis</p><p
                                                                        class="dp33">construyeran <i>o</i> construyesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">construyere</p><p
                                                                        class="dp33">construyeres</p><p class="dp33">construyere</p><p
                                                                        class="dp33">construyéremos</p><p class="dp33">construyereis</p><p
                                                                        class="dp33">construyeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">construye (construí), construid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">construir</p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33">construido</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">construyendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n26">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>26. CONTAR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">cuento</p><p
                                                                        class="dp33">cuentas (contás)</p><p
                                                                        class="dp33">cuenta</p><p
                                                                        class="dp33">contamos</p><p
                                                                        class="dp33">contáis</p><p
                                                                        class="dp33">cuentan</p></td><td class="dp23"
                                                                                                         colspan="3"><p
                                                                        class="dp33">contaba</p><p
                                                                        class="dp33">contabas</p><p
                                                                        class="dp33">contaba</p><p class="dp33">contábamos</p><p
                                                                        class="dp33">contabais</p><p class="dp33">contaban</p></td><td
                                                                        class="dp39"><p class="dp33">conté</p><p
                                                                        class="dp33">contaste</p><p
                                                                        class="dp33">contó</p><p
                                                                        class="dp33">contamos</p><p class="dp33">contasteis</p><p
                                                                        class="dp33">contaron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">contaré</p><p
                                                                        class="dp33">contarás</p><p
                                                                        class="dp33">contará</p><p class="dp33">contaremos</p><p
                                                                        class="dp33">contaréis</p><p class="dp33">contarán</p></td><td
                                                                        class="dp39"><p class="dp33">contaría</p><p
                                                                        class="dp33">contarías</p><p class="dp33">contaría</p><p
                                                                        class="dp33">contaríamos</p><p class="dp33">contaríais</p><p
                                                                        class="dp33">contarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">cuente</p><p
                                                                        class="dp33">cuentes</p><p
                                                                        class="dp33">cuente</p><p
                                                                        class="dp33">contemos</p><p
                                                                        class="dp33">contéis</p><p
                                                                        class="dp33">cuenten</p></td><td class="dp22"
                                                                                                         colspan="4"><p
                                                                        class="dp33">contara <i>o</i> contase</p><p
                                                                        class="dp33">contaras <i>o</i> contases</p><p
                                                                        class="dp33">contara <i>o</i> contase</p><p
                                                                        class="dp33">contáramos <i>o</i> contásemos</p><p
                                                                        class="dp33">contarais <i>o</i> contaseis</p><p
                                                                        class="dp33">contaran <i>o</i> contasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">contare</p><p
                                                                        class="dp33">contares</p><p
                                                                        class="dp33">contare</p><p class="dp33">contáremos</p><p
                                                                        class="dp33">contareis</p><p class="dp33">contaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">cuenta (contá), contad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">contar</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">contado</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">contando</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n27">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>27. DAR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">doy</p><p
                                                                        class="dp33">das (das)</p><p class="dp33">da</p><p
                                                                        class="dp33">damos</p><p class="dp33">dais</p><p
                                                                        class="dp33">dan</p></td><td class="dp23"
                                                                                                     colspan="3"><p
                                                                        class="dp33">daba</p><p class="dp33">dabas</p><p
                                                                        class="dp33">daba</p><p class="dp33">dábamos</p><p
                                                                        class="dp33">dabais</p><p class="dp33">daban</p></td><td
                                                                        class="dp39"><p class="dp33">di</p><p
                                                                        class="dp33">diste</p><p class="dp33">dio</p><p
                                                                        class="dp33">dimos</p><p
                                                                        class="dp33">disteis</p><p
                                                                        class="dp33">dieron</p></td><td class="dp40"
                                                                                                        colspan="2"><p
                                                                        class="dp33">daré</p><p class="dp33">darás</p><p
                                                                        class="dp33">dará</p><p class="dp33">daremos</p><p
                                                                        class="dp33">daréis</p><p class="dp33">darán</p></td><td
                                                                        class="dp39"><p class="dp33">daría</p><p
                                                                        class="dp33">darías</p><p class="dp33">daría</p><p
                                                                        class="dp33">daríamos</p><p
                                                                        class="dp33">daríais</p><p
                                                                        class="dp33">darían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">dé</p><p class="dp33">des</p><p
                                                                        class="dp33">dé</p><p class="dp33">demos</p><p
                                                                        class="dp33">deis</p><p
                                                                        class="dp33">den</p></td><td class="dp22"
                                                                                                     colspan="4"><p
                                                                        class="dp33">diera <i>o</i> diese</p><p
                                                                        class="dp33">dieras <i>o</i> dieses</p><p
                                                                        class="dp33">diera <i>o</i> diese</p><p
                                                                        class="dp33">diéramos <i>o</i> diésemos</p><p
                                                                        class="dp33">dierais <i>o</i> dieseis</p><p
                                                                        class="dp33">dieran <i>o</i> diesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">diere</p><p class="dp33">dieres</p><p
                                                                        class="dp33">diere</p><p
                                                                        class="dp33">diéremos</p><p
                                                                        class="dp33">diereis</p><p
                                                                        class="dp33">dieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">da (da), dad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">dar</p></td><td
                                                                        class="dp26" coslpan="3"><p
                                                                        class="dp33">dado</p></td><td class="dp42"
                                                                                                      colspan="2"><p
                                                                        class="dp33">dando</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n28">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>28. DECIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">digo</p><p
                                                                        class="dp33">dices (decís)</p><p class="dp33">dice</p><p
                                                                        class="dp33">decimos</p><p
                                                                        class="dp33">decís</p><p class="dp33">dicen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">decía</p><p class="dp33">decías</p><p
                                                                        class="dp33">decía</p><p
                                                                        class="dp33">decíamos</p><p
                                                                        class="dp33">decíais</p><p
                                                                        class="dp33">decían</p></td><td class="dp39"><p
                                                                        class="dp33">dije</p><p class="dp33">dijiste</p><p
                                                                        class="dp33">dijo</p><p class="dp33">dijimos</p><p
                                                                        class="dp33">dijisteis</p><p class="dp33">dijeron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">diré</p><p class="dp33">dirás</p><p
                                                                        class="dp33">dirá</p><p class="dp33">diremos</p><p
                                                                        class="dp33">diréis</p><p class="dp33">dirán</p></td><td
                                                                        class="dp39"><p class="dp33">diría</p><p
                                                                        class="dp33">dirías</p><p class="dp33">diría</p><p
                                                                        class="dp33">diríamos</p><p
                                                                        class="dp33">diríais</p><p
                                                                        class="dp33">dirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">diga</p><p class="dp33">digas</p><p
                                                                        class="dp33">diga</p><p class="dp33">digamos</p><p
                                                                        class="dp33">digáis</p><p class="dp33">digan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">dijera <i>o</i> dijese</p><p
                                                                        class="dp33">dijeras <i>o</i> dijeses</p><p
                                                                        class="dp33">dijera <i>o</i> dijese</p><p
                                                                        class="dp33">dijéramos <i>o</i> dijésemos</p><p
                                                                        class="dp33">dijerais <i>o</i> dijeseis</p><p
                                                                        class="dp33">dijeran <i>o</i> dijesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">dijere</p><p
                                                                        class="dp33">dijeres</p><p
                                                                        class="dp33">dijere</p><p
                                                                        class="dp33">dijéremos</p><p class="dp33">dijereis</p><p
                                                                        class="dp33">dijeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">di (decí), decid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">decir</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">dicho</p></td><td class="dp42"
                                                                                                       colspan="2"><p
                                                                        class="dp33">diciendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n29">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>29. DISCERNIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">discierno</p><p
                                                                        class="dp33">disciernes (discernís)</p><p
                                                                        class="dp33">discierne</p><p class="dp33">discernimos</p><p
                                                                        class="dp33">discernís</p><p class="dp33">disciernen</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">discernía</p><p
                                                                        class="dp33">discernías</p><p class="dp33">discernía</p><p
                                                                        class="dp33">discerníamos</p><p class="dp33">discerníais</p><p
                                                                        class="dp33">discernían</p></td><td
                                                                        class="dp39"><p class="dp33">discerní</p><p
                                                                        class="dp33">discerniste</p><p class="dp33">discernió</p><p
                                                                        class="dp33">discernimos</p><p class="dp33">discernisteis</p><p
                                                                        class="dp33">discernieron</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">discerniré</p><p
                                                                        class="dp33">discernirás</p><p class="dp33">discernirá</p><p
                                                                        class="dp33">discerniremos</p><p class="dp33">discerniréis</p><p
                                                                        class="dp33">discernirán</p></td><td
                                                                        class="dp39"><p class="dp33">discerniría</p><p
                                                                        class="dp33">discernirías</p><p class="dp33">discerniría</p><p
                                                                        class="dp33">discerniríamos</p><p class="dp33">discerniríais</p><p
                                                                        class="dp33">discernirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">discierna</p><p
                                                                        class="dp33">disciernas</p><p class="dp33">discierna</p><p
                                                                        class="dp33">discernamos</p><p class="dp33">discernáis</p><p
                                                                        class="dp33">disciernan</p></td><td class="dp22"
                                                                                                            colspan="4"><p
                                                                        class="dp33">discerniera <i>o</i> discerniese</p><p
                                                                        class="dp33">discernieras <i>o</i> discernieses</p><p
                                                                        class="dp33">discerniera <i>o</i> discerniese</p><p
                                                                        class="dp33">discerniéramos <i>o</i> discerniésemos</p><p
                                                                        class="dp33">discernierais <i>o</i> discernieseis</p><p
                                                                        class="dp33">discernieran <i>o</i> discerniesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">discerniere</p><p
                                                                        class="dp33">discernieres</p><p class="dp33">discerniere</p><p
                                                                        class="dp33">discerniéremos</p><p class="dp33">discerniereis</p><p
                                                                        class="dp33">discernieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">discierne (discerní), discernid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">discernir</p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33">discernido</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">discerniendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n30">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>30. DORMIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">duermo</p><p
                                                                        class="dp33">duermes (dormís)</p><p
                                                                        class="dp33">duerme</p><p
                                                                        class="dp33">dormimos</p><p
                                                                        class="dp33">dormís</p><p
                                                                        class="dp33">duermen</p></td><td class="dp23"
                                                                                                         colspan="3"><p
                                                                        class="dp33">dormía</p><p
                                                                        class="dp33">dormías</p><p
                                                                        class="dp33">dormía</p><p
                                                                        class="dp33">dormíamos</p><p class="dp33">dormíais</p><p
                                                                        class="dp33">dormían</p></td><td class="dp39"><p
                                                                        class="dp33">dormí</p><p
                                                                        class="dp33">dormiste</p><p
                                                                        class="dp33">durmió</p><p
                                                                        class="dp33">dormimos</p><p class="dp33">dormisteis</p><p
                                                                        class="dp33">durmieron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">dormiré</p><p
                                                                        class="dp33">dormirás</p><p
                                                                        class="dp33">dormirá</p><p class="dp33">dormiremos</p><p
                                                                        class="dp33">dormiréis</p><p class="dp33">dormirán</p></td><td
                                                                        class="dp39"><p class="dp33">dormiría</p><p
                                                                        class="dp33">dormirías</p><p class="dp33">dormiría</p><p
                                                                        class="dp33">dormiríamos</p><p class="dp33">dormiríais</p><p
                                                                        class="dp33">dormirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">duerma</p><p
                                                                        class="dp33">duermas</p><p
                                                                        class="dp33">duerma</p><p
                                                                        class="dp33">durmamos</p><p
                                                                        class="dp33">durmáis</p><p
                                                                        class="dp33">duerman</p></td><td class="dp22"
                                                                                                         colspan="4"><p
                                                                        class="dp33">durmiera <i>o</i> durmiese</p><p
                                                                        class="dp33">durmieras <i>o</i> durmieses</p><p
                                                                        class="dp33">durmiera <i>o</i> durmiese</p><p
                                                                        class="dp33">durmiéramos <i>o</i> durmiésemos</p><p
                                                                        class="dp33">durmierais <i>o</i> durmieseis</p><p
                                                                        class="dp33">durmieran <i>o</i> durmiesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">durmiere</p><p
                                                                        class="dp33">durmieres</p><p class="dp33">durmiere</p><p
                                                                        class="dp33">durmiéremos</p><p class="dp33">durmiereis</p><p
                                                                        class="dp33">durmieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">duerme (dormí), dormid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">dormir</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">dormido</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">durmiendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n31">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>31. ENTENDER </b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">entiendo</p><p
                                                                        class="dp33">entiendes (entendés)</p><p
                                                                        class="dp33">entiende</p><p class="dp33">entendemos</p><p
                                                                        class="dp33">entendéis</p><p class="dp33">entienden</p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33">entendía</p><p
                                                                        class="dp33">entendías</p><p class="dp33">entendía</p><p
                                                                        class="dp33">entendíamos</p><p class="dp33">entendíais</p><p
                                                                        class="dp33">entendían</p></td><td class="dp39"><p
                                                                        class="dp33">entendí</p><p class="dp33">entendiste</p><p
                                                                        class="dp33">entendió</p><p class="dp33">entendimos</p><p
                                                                        class="dp33">entendisteis</p><p class="dp33">entendieron</p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33">entenderé</p><p
                                                                        class="dp33">entenderás</p><p class="dp33">entenderá</p><p
                                                                        class="dp33">entenderemos</p><p class="dp33">entenderéis</p><p
                                                                        class="dp33">entenderán</p></td><td
                                                                        class="dp39"><p class="dp33">entendería</p><p
                                                                        class="dp33">entenderías</p><p class="dp33">entendería</p><p
                                                                        class="dp33">entenderíamos</p><p class="dp33">entenderíais</p><p
                                                                        class="dp33">entenderían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">entienda</p><p
                                                                        class="dp33">entiendas</p><p class="dp33">entienda</p><p
                                                                        class="dp33">entendamos</p><p class="dp33">entendáis</p><p
                                                                        class="dp33">entiendan</p></td><td class="dp22"
                                                                                                           colspan="4"><p
                                                                        class="dp33">entendiera <i>o</i> entendiese</p><p
                                                                        class="dp33">entendieras <i>o</i> entendieses</p><p
                                                                        class="dp33">entendiera <i>o</i> entendiese</p><p
                                                                        class="dp33">entendiéramos <i>o</i> entendiésemos</p><p
                                                                        class="dp33">entendierais <i>o</i> entendieseis</p><p
                                                                        class="dp33">entendieran <i>o</i> entendiesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">entendiere</p><p
                                                                        class="dp33">entendieres</p><p class="dp33">entendiere</p><p
                                                                        class="dp33">entendiéremos</p><p class="dp33">entendiereis</p><p
                                                                        class="dp33">entendieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="8" valign="middle">entiende (entendé) entended</td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">entender</p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33">entendido</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">entendiendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n32">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>32. ERGUIR*</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">yergo</p><p
                                                                        class="dp33">yergues (erguís)</p><p
                                                                        class="dp33">yergue</p><p
                                                                        class="dp33">erguimos</p><p
                                                                        class="dp33">erguís</p><p
                                                                        class="dp33">yerguen</p></td><td class="dp23"
                                                                                                         colspan="3"><p
                                                                        class="dp33">erguía</p><p
                                                                        class="dp33">erguías</p><p
                                                                        class="dp33">erguía</p><p
                                                                        class="dp33">erguíamos</p><p class="dp33">erguíais</p><p
                                                                        class="dp33">erguían</p></td><td class="dp39"><p
                                                                        class="dp33">erguí</p><p
                                                                        class="dp33">erguiste</p><p
                                                                        class="dp33">irguió</p><p
                                                                        class="dp33">erguimos</p><p class="dp33">erguisteis</p><p
                                                                        class="dp33">irguieron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">erguiré</p><p
                                                                        class="dp33">erguirás</p><p
                                                                        class="dp33">erguirá</p><p class="dp33">erguiremos</p><p
                                                                        class="dp33">erguiréis</p><p class="dp33">erguirán</p></td><td
                                                                        class="dp39"><p class="dp33">erguiría</p><p
                                                                        class="dp33">erguirías</p><p class="dp33">erguiría</p><p
                                                                        class="dp33">erguiríamos</p><p class="dp33">erguiríais</p><p
                                                                        class="dp33">erguirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">yerga</p><p class="dp33">yergas</p><p
                                                                        class="dp33">yerga</p><p
                                                                        class="dp33">irgamos</p><p
                                                                        class="dp33">irgáis</p><p
                                                                        class="dp33">yergan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">irguiera <i>o</i> irguiese</p><p
                                                                        class="dp33">irguieras <i>o</i> irguieses</p><p
                                                                        class="dp33">irguiera <i>o</i> irguiese</p><p
                                                                        class="dp33">irguiéramos <i>o</i> irguiésemos</p><p
                                                                        class="dp33">irguierais <i>o</i> irguieseis</p><p
                                                                        class="dp33">irguieran <i>o</i> irguiesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">irguiere</p><p
                                                                        class="dp33">irguieres</p><p class="dp33">irguiere</p><p
                                                                        class="dp33">irguiéremos</p><p class="dp33">irguiereis</p><p
                                                                        class="dp33">irguieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">yergue (erguí), erguid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">erguir</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">erguido</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">irguiendo</p></td></tr></tbody></table><p
                                                                        class="dp30">*&nbsp; También son válidas, aunque muy raras, las formas <i>irgo, irgues, irgue, irguen,</i> en el presente de indicativo; <i>irga, irgas, irga, yergamos, yergáis, irgan,</i> en el presente de subjuntivo; e <i>irgue,</i> en el imperativo.</p><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n33">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>33. ERRAR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">yerro</p><p
                                                                        class="dp33">yerras (errás)</p><p class="dp33">yerra</p><p
                                                                        class="dp33">erramos</p><p
                                                                        class="dp33">erráis</p><p
                                                                        class="dp33">yerran</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">erraba</p><p
                                                                        class="dp33">errabas</p><p
                                                                        class="dp33">erraba</p><p
                                                                        class="dp33">errábamos</p><p class="dp33">errabais</p><p
                                                                        class="dp33">erraban</p></td><td class="dp39"><p
                                                                        class="dp33">erré</p><p class="dp33">erraste</p><p
                                                                        class="dp33">erró</p><p class="dp33">erramos</p><p
                                                                        class="dp33">errasteis</p><p class="dp33">erraron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">erraré</p><p
                                                                        class="dp33">errarás</p><p
                                                                        class="dp33">errará</p><p
                                                                        class="dp33">erraremos</p><p class="dp33">erraréis</p><p
                                                                        class="dp33">errarán</p></td><td class="dp39"><p
                                                                        class="dp33">erraría</p><p
                                                                        class="dp33">errarías</p><p
                                                                        class="dp33">erraría</p><p class="dp33">erraríamos</p><p
                                                                        class="dp33">erraríais</p><p class="dp33">errarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">yerre</p><p class="dp33">yerres</p><p
                                                                        class="dp33">yerre</p><p
                                                                        class="dp33">erremos</p><p
                                                                        class="dp33">erréis</p><p
                                                                        class="dp33">yerren</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">errara <i>o</i> errase</p><p
                                                                        class="dp33">erraras <i>o</i> errases</p><p
                                                                        class="dp33">errara <i>o</i> errase</p><p
                                                                        class="dp33">erráramos <i>o</i> errásemos</p><p
                                                                        class="dp33">errarais <i>o</i> erraseis</p><p
                                                                        class="dp33">erraran <i>o</i> errasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">errare</p><p
                                                                        class="dp33">errares</p><p
                                                                        class="dp33">errare</p><p
                                                                        class="dp33">erráremos</p><p class="dp33">errareis</p><p
                                                                        class="dp33">erraren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">yerra (errá), errad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">errar</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">errado</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">errando</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n34">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>34. ESTAR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">estoy</p><p
                                                                        class="dp33">estás (estás)</p><p class="dp33">está</p><p
                                                                        class="dp33">estamos</p><p
                                                                        class="dp33">estáis</p><p class="dp33">están</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">estaba</p><p
                                                                        class="dp33">estabas</p><p
                                                                        class="dp33">estaba</p><p
                                                                        class="dp33">estábamos</p><p class="dp33">estabais</p><p
                                                                        class="dp33">estaban</p></td><td class="dp39"><p
                                                                        class="dp33">estuve</p><p
                                                                        class="dp33">estuviste</p><p
                                                                        class="dp33">estuvo</p><p
                                                                        class="dp33">estuvimos</p><p class="dp33">estuvisteis</p><p
                                                                        class="dp33">estuvieron</p></td><td class="dp40"
                                                                                                            colspan="2"><p
                                                                        class="dp33">estaré</p><p
                                                                        class="dp33">estarás</p><p
                                                                        class="dp33">estará</p><p
                                                                        class="dp33">estaremos</p><p class="dp33">estaréis</p><p
                                                                        class="dp33">estarán</p></td><td class="dp39"><p
                                                                        class="dp33">estaría</p><p
                                                                        class="dp33">estarías</p><p
                                                                        class="dp33">estaría</p><p class="dp33">estaríamos</p><p
                                                                        class="dp33">estaríais</p><p class="dp33">estarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">esté</p><p class="dp33">estés</p><p
                                                                        class="dp33">esté</p><p class="dp33">estemos</p><p
                                                                        class="dp33">estéis</p><p class="dp33">estén</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">estuviera <i>o</i> estuviese</p><p
                                                                        class="dp33">estuvieras <i>o</i> estuvieses</p><p
                                                                        class="dp33">estuviera <i>o</i> estuviese</p><p
                                                                        class="dp33">estuviéramos <i>o</i> estuviésemos</p><p
                                                                        class="dp33">estuvierais <i>o</i> estuvieseis</p><p
                                                                        class="dp33">estuvieran <i>o</i> estuviesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">estuviere</p><p
                                                                        class="dp33">estuvieres</p><p class="dp33">estuviere</p><p
                                                                        class="dp33">estuviéremos</p><p class="dp33">estuviereis</p><p
                                                                        class="dp33">estuvieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        align="center" colspan="8" valign="middle">está (está)*, estad</td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">estar</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">estado</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">estando</p></td></tr></tbody></table><p
                                                                        class="dp30">*&nbsp; El imperativo de la segunda persona del singular solo se usa en forma pronominal (<i>estate</i>): <i>Estate tranquilo.</i></p><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n35">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>35. HABER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">he</p><p
                                                                        class="dp33">has (has)</p><p
                                                                        class="dp33">ha (<i>como</i></p><p class="dp33"><i>impersonal:</i> hay)</p><p
                                                                        class="dp33">hemos</p><p class="dp33">habéis</p><p
                                                                        class="dp33">han</p></td><td class="dp23"
                                                                                                     colspan="3"><p
                                                                        class="dp33">había</p><p class="dp33">habías</p><p
                                                                        class="dp33">había</p><p
                                                                        class="dp33">habíamos</p><p
                                                                        class="dp33">habíais</p><p
                                                                        class="dp33">habían</p></td><td class="dp39"><p
                                                                        class="dp33">hube</p><p class="dp33">hubiste</p><p
                                                                        class="dp33">hubo</p><p class="dp33">hubimos</p><p
                                                                        class="dp33">hubisteis</p><p class="dp33">hubieron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">habré</p><p class="dp33">habrás</p><p
                                                                        class="dp33">habrá</p><p
                                                                        class="dp33">habremos</p><p
                                                                        class="dp33">habréis</p><p
                                                                        class="dp33">habrán</p></td><td class="dp39"><p
                                                                        class="dp33">habría</p><p
                                                                        class="dp33">habrías</p><p
                                                                        class="dp33">habría</p><p
                                                                        class="dp33">habríamos</p><p class="dp33">habríais</p><p
                                                                        class="dp33">habrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">haya</p><p class="dp33">hayas</p><p
                                                                        class="dp33">haya</p><p class="dp33">hayamos</p><p
                                                                        class="dp33">hayáis</p><p class="dp33">hayan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">hubiera <i>o</i> hubiese</p><p
                                                                        class="dp33">hubieras <i>o</i> hubieses</p><p
                                                                        class="dp33">hubiera <i>o</i> hubiese</p><p
                                                                        class="dp33">hubiéramos <i>o</i> hubiésemos</p><p
                                                                        class="dp33">hubierais <i>o</i> hubieseis</p><p
                                                                        class="dp33">hubieran <i>o</i> hubiesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">hubiere</p><p
                                                                        class="dp33">hubieres</p><p
                                                                        class="dp33">hubiere</p><p class="dp33">hubiéremos</p><p
                                                                        class="dp33">hubiereis</p><p class="dp33">hubieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33"><i>Las formas heredadas del latín son</i> habe <i>y</i> habed, <i>pero carecen totalmente de uso en la actualidad.</i></p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">haber</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">habido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">habiendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n36">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>36. HACER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">hago</p><p
                                                                        class="dp33">haces (hacés)</p><p class="dp33">hace</p><p
                                                                        class="dp33">hacemos</p><p
                                                                        class="dp33">hacéis</p><p class="dp33">hacen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">hacía</p><p class="dp33">hacías</p><p
                                                                        class="dp33">hacía</p><p
                                                                        class="dp33">hacíamos</p><p
                                                                        class="dp33">hacíais</p><p
                                                                        class="dp33">hacían</p></td><td class="dp39"><p
                                                                        class="dp33">hice</p><p class="dp33">hiciste</p><p
                                                                        class="dp33">hizo</p><p class="dp33">hicimos</p><p
                                                                        class="dp33">hicisteis</p><p class="dp33">hicieron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">haré</p><p class="dp33">harás</p><p
                                                                        class="dp33">hará</p><p class="dp33">haremos</p><p
                                                                        class="dp33">haréis</p><p class="dp33">harán</p></td><td
                                                                        class="dp39"><p class="dp33">haría</p><p
                                                                        class="dp33">harías</p><p class="dp33">haría</p><p
                                                                        class="dp33">haríamos</p><p
                                                                        class="dp33">haríais</p><p
                                                                        class="dp33">harían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">haga</p><p class="dp33">hagas</p><p
                                                                        class="dp33">haga</p><p class="dp33">hagamos</p><p
                                                                        class="dp33">hagáis</p><p class="dp33">hagan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">hiciera <i>o</i> hiciese</p><p
                                                                        class="dp33">hicieras <i>o</i> hicieses</p><p
                                                                        class="dp33">hiciera <i>o</i> hiciese</p><p
                                                                        class="dp33">hiciéramos <i>o</i> hiciésemos</p><p
                                                                        class="dp33">hicierais <i>o</i> hicieseis</p><p
                                                                        class="dp33">hicieran <i>o</i> hiciesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">hiciere</p><p
                                                                        class="dp33">hicieres</p><p
                                                                        class="dp33">hiciere</p><p class="dp33">hiciéremos</p><p
                                                                        class="dp33">hiciereis</p><p class="dp33">hicieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">haz (hacé), haced</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">hacer</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">hecho</p></td><td class="dp42"
                                                                                                       colspan="2"><p
                                                                        class="dp33">haciendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n37">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>37. IR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">voy</p><p
                                                                        class="dp33">vas (vas)</p><p class="dp33">va</p><p
                                                                        class="dp33">vamos</p><p class="dp33">vais</p><p
                                                                        class="dp33">van</p></td><td class="dp23"
                                                                                                     colspan="3"><p
                                                                        class="dp33">iba</p><p class="dp33">ibas</p><p
                                                                        class="dp33">iba</p><p class="dp33">íbamos</p><p
                                                                        class="dp33">ibais</p><p
                                                                        class="dp33">iban</p></td><td class="dp39"><p
                                                                        class="dp33">fui</p><p class="dp33">fuiste</p><p
                                                                        class="dp33">fue</p><p class="dp33">fuimos</p><p
                                                                        class="dp33">fuisteis</p><p
                                                                        class="dp33">fueron</p></td><td class="dp40"
                                                                                                        colspan="2"><p
                                                                        class="dp33">iré</p><p class="dp33">irás</p><p
                                                                        class="dp33">irá</p><p class="dp33">iremos</p><p
                                                                        class="dp33">iréis</p><p
                                                                        class="dp33">irán</p></td><td class="dp39"><p
                                                                        class="dp33">iría</p><p class="dp33">irías</p><p
                                                                        class="dp33">iría</p><p class="dp33">iríamos</p><p
                                                                        class="dp33">iríais</p><p class="dp33">irían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">vaya</p><p class="dp33">vayas</p><p
                                                                        class="dp33">vaya</p><p class="dp33">vayamos</p><p
                                                                        class="dp33">vayáis</p><p class="dp33">vayan</p></td><td
                                                                        class="dp22" colspan="4"><p
                                                                        class="dp33">fuera <i>o</i> fuese</p><p
                                                                        class="dp33">fueras <i>o</i> fueses</p><p
                                                                        class="dp33">fuera <i>o</i> fuese</p><p
                                                                        class="dp33">fuéramos <i>o</i> fuésemos</p><p
                                                                        class="dp33">fuerais <i>o</i> fueseis</p><p
                                                                        class="dp33">fueran <i>o</i> fuesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">fuere</p><p class="dp33">fueres</p><p
                                                                        class="dp33">fuere</p><p
                                                                        class="dp33">fuéremos</p><p
                                                                        class="dp33">fuereis</p><p
                                                                        class="dp33">fueren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">ve (<i>no tiene forma propia de voseo; en su lugar se usa</i> andá, <i>imperativo de</i> andar), id</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">ir</p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33">ido</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">yendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n38">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>38. JUGAR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">juego</p><p
                                                                        class="dp33">juegas (jugás)</p><p class="dp33">juega</p><p
                                                                        class="dp33">jugamos</p><p
                                                                        class="dp33">jugáis</p><p
                                                                        class="dp33">juegan</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">jugaba</p><p
                                                                        class="dp33">jugabas</p><p
                                                                        class="dp33">jugaba</p><p
                                                                        class="dp33">jugábamos</p><p class="dp33">jugabais</p><p
                                                                        class="dp33">jugaban</p></td><td class="dp39"><p
                                                                        class="dp33">jugué</p><p
                                                                        class="dp33">jugaste</p><p class="dp33">jugó</p><p
                                                                        class="dp33">jugamos</p><p class="dp33">jugasteis</p><p
                                                                        class="dp33">jugaron</p></td><td class="dp40"
                                                                                                         colspan="2"><p
                                                                        class="dp33">jugaré</p><p
                                                                        class="dp33">jugarás</p><p
                                                                        class="dp33">jugará</p><p
                                                                        class="dp33">jugaremos</p><p class="dp33">jugaréis</p><p
                                                                        class="dp33">jugarán</p></td><td class="dp39"><p
                                                                        class="dp33">jugaría</p><p
                                                                        class="dp33">jugarías</p><p
                                                                        class="dp33">jugaría</p><p class="dp33">jugaríamos</p><p
                                                                        class="dp33">jugaríais</p><p class="dp33">jugarían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">juegue</p><p
                                                                        class="dp33">juegues</p><p
                                                                        class="dp33">juegue</p><p
                                                                        class="dp33">juguemos</p><p
                                                                        class="dp33">juguéis</p><p
                                                                        class="dp33">jueguen</p></td><td class="dp22"
                                                                                                         colspan="4"><p
                                                                        class="dp33">jugara <i>o</i> jugase</p><p
                                                                        class="dp33">jugaras <i>o</i> jugases</p><p
                                                                        class="dp33">jugara <i>o</i> jugase</p><p
                                                                        class="dp33">jugáramos <i>o</i> jugásemos</p><p
                                                                        class="dp33">jugarais <i>o</i> jugaseis</p><p
                                                                        class="dp33">jugaran <i>o</i> jugasen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">jugare</p><p
                                                                        class="dp33">jugares</p><p
                                                                        class="dp33">jugare</p><p
                                                                        class="dp33">jugáremos</p><p class="dp33">jugareis</p><p
                                                                        class="dp33">jugaren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">juega (jugá), jugad</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">jugar</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">jugado</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">jugando</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n39">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>39. LEER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">leo</p><p
                                                                        class="dp33">lees (leés)</p><p
                                                                        class="dp33">lee</p><p class="dp33">leemos</p><p
                                                                        class="dp33">leéis</p><p
                                                                        class="dp33">leen</p></td><td class="dp23"
                                                                                                      colspan="3"><p
                                                                        class="dp33">leía</p><p class="dp33">leías</p><p
                                                                        class="dp33">leía</p><p class="dp33">leíamos</p><p
                                                                        class="dp33">leíais</p><p class="dp33">leían</p></td><td
                                                                        class="dp39"><p class="dp33">leí</p><p
                                                                        class="dp33">leíste</p><p
                                                                        class="dp33">leyó</p><p
                                                                        class="dp33">leímos</p><p
                                                                        class="dp33">leísteis</p><p
                                                                        class="dp33">leyeron</p></td><td class="dp40"
                                                                                                         colspan="2"><p
                                                                        class="dp33">leeré</p><p class="dp33">leerás</p><p
                                                                        class="dp33">leerá</p><p
                                                                        class="dp33">leeremos</p><p
                                                                        class="dp33">leeréis</p><p
                                                                        class="dp33">leerán</p></td><td class="dp39"><p
                                                                        class="dp33">leería</p><p
                                                                        class="dp33">leerías</p><p
                                                                        class="dp33">leería</p><p
                                                                        class="dp33">leeríamos</p><p class="dp33">leeríais</p><p
                                                                        class="dp33">leerían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">lea</p><p
                                                                        class="dp33">leas</p><p class="dp33">lea</p><p
                                                                        class="dp33">leamos</p><p class="dp33">leáis</p><p
                                                                        class="dp33">lean</p></td><td class="dp22"
                                                                                                      colspan="4"><p
                                                                        class="dp33">leyera <i>o</i> leyese</p><p
                                                                        class="dp33">leyeras <i>o</i> leyeses</p><p
                                                                        class="dp33">leyera <i>o</i> leyese</p><p
                                                                        class="dp33">leyéramos <i>o</i> leyésemos</p><p
                                                                        class="dp33">leyerais <i>o</i> leyeseis</p><p
                                                                        class="dp33">leyeran <i>o</i> leyesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">leyere</p><p
                                                                        class="dp33">leyeres</p><p
                                                                        class="dp33">leyere</p><p
                                                                        class="dp33">leyéremos</p><p class="dp33">leyereis</p><p
                                                                        class="dp33">leyeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">lee (leé), leed</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">leer</p></td><td class="dp26"
                                                                                                      coslpan="3"><p
                                                                        class="dp33">leído</p></td><td class="dp42"
                                                                                                       colspan="2"><p
                                                                        class="dp33">leyendo</p></td></tr></tbody></table><p></p><table
                                                                        class="dp55"><tbody><tr><td><p class="dp36"><a
                                                                        name="n40">&nbsp;</a><a href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>40. LUCIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">luzco</p><p
                                                                        class="dp33">luces (lucís)</p><p class="dp33">luce</p><p
                                                                        class="dp33">lucimos</p><p
                                                                        class="dp33">lucís</p><p class="dp33">lucen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">lucía</p><p class="dp33">lucías</p><p
                                                                        class="dp33">lucía</p><p
                                                                        class="dp33">lucíamos</p><p
                                                                        class="dp33">lucíais</p><p
                                                                        class="dp33">lucían</p></td><td class="dp39"><p
                                                                        class="dp33">lucí</p><p class="dp33">luciste</p><p
                                                                        class="dp33">lució</p><p
                                                                        class="dp33">lucimos</p><p class="dp33">lucisteis</p><p
                                                                        class="dp33">lucieron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">luciré</p><p
                                                                        class="dp33">lucirás</p><p
                                                                        class="dp33">lucirá</p><p
                                                                        class="dp33">luciremos</p><p class="dp33">luciréis</p><p
                                                                        class="dp33">lucirán</p></td><td class="dp39"><p
                                                                        class="dp33">luciría</p><p
                                                                        class="dp33">lucirías</p><p
                                                                        class="dp33">luciría</p><p class="dp33">luciríamos</p><p
                                                                        class="dp33">luciríais</p><p class="dp33">lucirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">luzca</p><p class="dp33">luzcas</p><p
                                                                        class="dp33">luzca</p><p
                                                                        class="dp33">luzcamos</p><p
                                                                        class="dp33">luzcáis</p><p
                                                                        class="dp33">luzcan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">luciera <i>o</i> luciese</p><p
                                                                        class="dp33">lucieras <i>o</i> lucieses</p><p
                                                                        class="dp33">luciera <i>o</i> luciese</p><p
                                                                        class="dp33">luciéramos <i>o</i> luciésemos</p><p
                                                                        class="dp33">lucierais <i>o</i> lucieseis</p><p
                                                                        class="dp33">lucieran <i>o</i> luciesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">luciere</p><p
                                                                        class="dp33">lucieres</p><p
                                                                        class="dp33">luciere</p><p class="dp33">luciéremos</p><p
                                                                        class="dp33">luciereis</p><p class="dp33">lucieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">luce (lucí), lucid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">lucir</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">lucido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">luciendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n41">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>41. MOVER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">muevo</p><p
                                                                        class="dp33">mueves (movés)</p><p class="dp33">mueve</p><p
                                                                        class="dp33">movemos</p><p
                                                                        class="dp33">movéis</p><p
                                                                        class="dp33">mueven</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">movía</p><p class="dp33">movías</p><p
                                                                        class="dp33">movía</p><p
                                                                        class="dp33">movíamos</p><p
                                                                        class="dp33">movíais</p><p
                                                                        class="dp33">movían</p></td><td class="dp39"><p
                                                                        class="dp33">moví</p><p class="dp33">moviste</p><p
                                                                        class="dp33">movió</p><p
                                                                        class="dp33">movimos</p><p class="dp33">movisteis</p><p
                                                                        class="dp33">movieron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">moveré</p><p
                                                                        class="dp33">moverás</p><p
                                                                        class="dp33">moverá</p><p
                                                                        class="dp33">moveremos</p><p class="dp33">moveréis</p><p
                                                                        class="dp33">moverán</p></td><td class="dp39"><p
                                                                        class="dp33">movería</p><p
                                                                        class="dp33">moverías</p><p
                                                                        class="dp33">movería</p><p class="dp33">moveríamos</p><p
                                                                        class="dp33">moveríais</p><p class="dp33">moverían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">mueva</p><p class="dp33">muevas</p><p
                                                                        class="dp33">mueva</p><p
                                                                        class="dp33">movamos</p><p
                                                                        class="dp33">mováis</p><p
                                                                        class="dp33">muevan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">moviera <i>o</i> moviese</p><p
                                                                        class="dp33">movieras <i>o</i> movieses</p><p
                                                                        class="dp33">moviera <i>o</i> moviese</p><p
                                                                        class="dp33">moviéramos <i>o</i> moviésemos</p><p
                                                                        class="dp33">movierais <i>o</i> movieseis</p><p
                                                                        class="dp33">movieran <i>o</i> moviesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">moviere</p><p
                                                                        class="dp33">movieres</p><p
                                                                        class="dp33">moviere</p><p class="dp33">moviéremos</p><p
                                                                        class="dp33">moviereis</p><p class="dp33">movieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">mueve (mové), moved</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">mover</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">movido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">moviendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n42">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="8"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>42. MULLIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">mullo</p><p
                                                                        class="dp33">mulles (mullís)</p><p class="dp33">mulle</p><p
                                                                        class="dp33">mullimos</p><p
                                                                        class="dp33">mullís</p><p
                                                                        class="dp33">mullen</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">mullía</p><p
                                                                        class="dp33">mullías</p><p
                                                                        class="dp33">mullía</p><p
                                                                        class="dp33">mullíamos</p><p class="dp33">mullíais</p><p
                                                                        class="dp33">mullían</p></td><td class="dp39"><p
                                                                        class="dp33">mullí</p><p
                                                                        class="dp33">mulliste</p><p
                                                                        class="dp33">mulló</p><p
                                                                        class="dp33">mullimos</p><p class="dp33">mullisteis</p><p
                                                                        class="dp33">mulleron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">mulliré</p><p
                                                                        class="dp33">mullirás</p><p
                                                                        class="dp33">mullirá</p><p class="dp33">mulliremos</p><p
                                                                        class="dp33">mulliréis</p><p class="dp33">mullirán</p></td><td
                                                                        class="dp39"><p class="dp33">mulliría</p><p
                                                                        class="dp33">mullirías</p><p class="dp33">mulliría</p><p
                                                                        class="dp33">mulliríamos</p><p class="dp33">mulliríais</p><p
                                                                        class="dp33">mullirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">mulla</p><p class="dp33">mullas</p><p
                                                                        class="dp33">mulla</p><p
                                                                        class="dp33">mullamos</p><p
                                                                        class="dp33">mulláis</p><p
                                                                        class="dp33">mullan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">mullera <i>o</i> mullese</p><p
                                                                        class="dp33">mulleras <i>o</i> mulleses</p><p
                                                                        class="dp33">mullera <i>o</i> mullese</p><p
                                                                        class="dp33">mulléramos <i>o</i> mullésemos</p><p
                                                                        class="dp33">mullerais <i>o</i> mulleseis</p><p
                                                                        class="dp33">mulleran <i>o</i> mullesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">mullere</p><p
                                                                        class="dp33">mulleres</p><p
                                                                        class="dp33">mullere</p><p class="dp33">mulléremos</p><p
                                                                        class="dp33">mullereis</p><p class="dp33">mulleren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">mulle (mullí), mullid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">mullir</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">mullido</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">mullendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n43">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="8"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>43. OÍR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">oigo</p><p
                                                                        class="dp33">oyes (oís)</p><p
                                                                        class="dp33">oye</p><p class="dp33">oímos</p><p
                                                                        class="dp33">oís</p><p
                                                                        class="dp33">oyen</p></td><td class="dp23"
                                                                                                      colspan="3"><p
                                                                        class="dp33">oía</p><p class="dp33">oías</p><p
                                                                        class="dp33">oía</p><p class="dp33">oíamos</p><p
                                                                        class="dp33">oíais</p><p
                                                                        class="dp33">oían</p></td><td class="dp39"><p
                                                                        class="dp33">oí</p><p class="dp33">oíste</p><p
                                                                        class="dp33">oyó</p><p class="dp33">oímos</p><p
                                                                        class="dp33">oísteis</p><p
                                                                        class="dp33">oyeron</p></td><td class="dp40"
                                                                                                        colspan="2"><p
                                                                        class="dp33">oiré</p><p class="dp33">oirás</p><p
                                                                        class="dp33">oirá</p><p class="dp33">oiremos</p><p
                                                                        class="dp33">oiréis</p><p class="dp33">oirán</p></td><td
                                                                        class="dp39"><p class="dp33">oiría</p><p
                                                                        class="dp33">oirías</p><p class="dp33">oiría</p><p
                                                                        class="dp33">oiríamos</p><p
                                                                        class="dp33">oiríais</p><p
                                                                        class="dp33">oirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">oiga</p><p class="dp33">oigas</p><p
                                                                        class="dp33">oiga</p><p class="dp33">oigamos</p><p
                                                                        class="dp33">oigáis</p><p class="dp33">oigan</p></td><td
                                                                        class="dp22" colspan="4"><p
                                                                        class="dp33">oyera <i>u</i> oyese</p><p
                                                                        class="dp33">oyeras <i>u</i> oyeses</p><p
                                                                        class="dp33">oyera <i>u</i> oyese</p><p
                                                                        class="dp33">oyéramos <i>u</i> oyésemos</p><p
                                                                        class="dp33">oyerais <i>u</i> oyeseis</p><p
                                                                        class="dp33">oyeran <i>u</i> oyesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">oyere</p><p class="dp33">oyeres</p><p
                                                                        class="dp33">oyere</p><p
                                                                        class="dp33">oyéremos</p><p
                                                                        class="dp33">oyereis</p><p
                                                                        class="dp33">oyeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">oye (oí), oíd</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">oír</p></td><td
                                                                        class="dp26" coslpan="3"><p
                                                                        class="dp33">oído</p></td><td class="dp42"
                                                                                                      colspan="2"><p
                                                                        class="dp33">oyendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n44">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="8"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>44. OLER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">huelo</p><p
                                                                        class="dp33">hueles (olés)</p><p class="dp33">huele</p><p
                                                                        class="dp33">olemos</p><p class="dp33">oléis</p><p
                                                                        class="dp33">huelen</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">olía</p><p class="dp33">olías</p><p
                                                                        class="dp33">olía</p><p class="dp33">olíamos</p><p
                                                                        class="dp33">olíais</p><p class="dp33">olían</p></td><td
                                                                        class="dp39"><p class="dp33">olí</p><p
                                                                        class="dp33">oliste</p><p
                                                                        class="dp33">olió</p><p
                                                                        class="dp33">olimos</p><p
                                                                        class="dp33">olisteis</p><p
                                                                        class="dp33">olieron</p></td><td class="dp40"
                                                                                                         colspan="2"><p
                                                                        class="dp33">oleré</p><p class="dp33">olerás</p><p
                                                                        class="dp33">olerá</p><p
                                                                        class="dp33">oleremos</p><p
                                                                        class="dp33">oleréis</p><p
                                                                        class="dp33">olerán</p></td><td class="dp39"><p
                                                                        class="dp33">olería</p><p
                                                                        class="dp33">olerías</p><p
                                                                        class="dp33">olería</p><p
                                                                        class="dp33">oleríamos</p><p class="dp33">oleríais</p><p
                                                                        class="dp33">olerían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">huela</p><p class="dp33">huelas</p><p
                                                                        class="dp33">huela</p><p class="dp33">olamos</p><p
                                                                        class="dp33">oláis</p><p class="dp33">huelan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">oliera <i>u</i> oliese</p><p
                                                                        class="dp33">olieras <i>u</i> olieses</p><p
                                                                        class="dp33">oliera <i>u</i> oliese</p><p
                                                                        class="dp33">oliéramos <i>u</i> oliésemos</p><p
                                                                        class="dp33">olierais <i>u</i> olieseis</p><p
                                                                        class="dp33">olieran <i>u</i> oliesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">oliere</p><p
                                                                        class="dp33">olieres</p><p
                                                                        class="dp33">oliere</p><p
                                                                        class="dp33">oliéremos</p><p class="dp33">oliereis</p><p
                                                                        class="dp33">olieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">huele (olé), oled</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">oler</p></td><td class="dp26"
                                                                                                      coslpan="3"><p
                                                                        class="dp33">olido</p></td><td class="dp42"
                                                                                                       colspan="2"><p
                                                                        class="dp33">oliendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n45">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="8"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>45. PEDIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">pido</p><p
                                                                        class="dp33">pides (pedís)</p><p class="dp33">pide</p><p
                                                                        class="dp33">pedimos</p><p
                                                                        class="dp33">pedís</p><p class="dp33">piden</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">pedía</p><p class="dp33">pedías</p><p
                                                                        class="dp33">pedía</p><p
                                                                        class="dp33">pedíamos</p><p
                                                                        class="dp33">pedíais</p><p
                                                                        class="dp33">pedían</p></td><td class="dp39"><p
                                                                        class="dp33">pedí</p><p class="dp33">pediste</p><p
                                                                        class="dp33">pidió</p><p
                                                                        class="dp33">pedimos</p><p class="dp33">pedisteis</p><p
                                                                        class="dp33">pidieron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">pediré</p><p
                                                                        class="dp33">pedirás</p><p
                                                                        class="dp33">pedirá</p><p
                                                                        class="dp33">pediremos</p><p class="dp33">pediréis</p><p
                                                                        class="dp33">pedirán</p></td><td class="dp39"><p
                                                                        class="dp33">pediría</p><p
                                                                        class="dp33">pedirías</p><p
                                                                        class="dp33">pediría</p><p class="dp33">pediríamos</p><p
                                                                        class="dp33">pediríais</p><p class="dp33">pedirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">pida</p><p class="dp33">pidas</p><p
                                                                        class="dp33">pida</p><p class="dp33">pidamos</p><p
                                                                        class="dp33">pidáis</p><p class="dp33">pidan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">pidiera <i>o</i> pidiese</p><p
                                                                        class="dp33">pidieras <i>o</i> pidieses</p><p
                                                                        class="dp33">pidiera <i>o</i> pidiese</p><p
                                                                        class="dp33">pidiéramos <i>o</i> pidiésemos</p><p
                                                                        class="dp33">pidierais <i>o</i> pidieseis</p><p
                                                                        class="dp33">pidieran <i>o</i> pidiesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">pidiere</p><p
                                                                        class="dp33">pidieres</p><p
                                                                        class="dp33">pidiere</p><p class="dp33">pidiéremos</p><p
                                                                        class="dp33">pidiereis</p><p class="dp33">pidieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">pide (pedí), pedid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">pedir</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">pedido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">pidiendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n46">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td bgcolor="#c0c0c0"
                                                                                                    colspan="8"
                                                                                                    valign="middle"
                                                                                                    width="40%"><b>46. PODER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">puedo</p><p
                                                                        class="dp33">puedes (podés)</p><p class="dp33">puede</p><p
                                                                        class="dp33">podemos</p><p
                                                                        class="dp33">podéis</p><p
                                                                        class="dp33">pueden</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">podía</p><p class="dp33">podías</p><p
                                                                        class="dp33">podía</p><p
                                                                        class="dp33">podíamos</p><p
                                                                        class="dp33">podíais</p><p
                                                                        class="dp33">podían</p></td><td class="dp39"><p
                                                                        class="dp33">pude</p><p class="dp33">pudiste</p><p
                                                                        class="dp33">pudo</p><p class="dp33">pudimos</p><p
                                                                        class="dp33">pudisteis</p><p class="dp33">pudieron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">podré</p><p class="dp33">podrás</p><p
                                                                        class="dp33">podrá</p><p
                                                                        class="dp33">podremos</p><p
                                                                        class="dp33">podréis</p><p
                                                                        class="dp33">podrán</p></td><td class="dp39"><p
                                                                        class="dp33">podría</p><p
                                                                        class="dp33">podrías</p><p
                                                                        class="dp33">podría</p><p
                                                                        class="dp33">podríamos</p><p class="dp33">podríais</p><p
                                                                        class="dp33">podrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">pueda</p><p class="dp33">puedas</p><p
                                                                        class="dp33">pueda</p><p
                                                                        class="dp33">podamos</p><p
                                                                        class="dp33">podáis</p><p
                                                                        class="dp33">puedan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">pudiera <i>o</i> pudiese</p><p
                                                                        class="dp33">pudieras <i>o</i> pudieses</p><p
                                                                        class="dp33">pudiera <i>o</i> pudiese</p><p
                                                                        class="dp33">pudiéramos <i>o</i> pudiésemos</p><p
                                                                        class="dp33">pudierais <i>o</i> pudieseis</p><p
                                                                        class="dp33">pudieran <i>o</i> pudiesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">pudiere</p><p
                                                                        class="dp33">pudieres</p><p
                                                                        class="dp33">pudiere</p><p class="dp33">pudiéremos</p><p
                                                                        class="dp33">pudiereis</p><p class="dp33">pudieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">puede (podé), poded</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">poder</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">podido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">pudiendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n47">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>47. PONER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">pongo</p><p
                                                                        class="dp33">pones (ponés)</p><p class="dp33">pone</p><p
                                                                        class="dp33">ponemos</p><p
                                                                        class="dp33">ponéis</p><p class="dp33">ponen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">ponía</p><p class="dp33">ponías</p><p
                                                                        class="dp33">ponía</p><p
                                                                        class="dp33">poníamos</p><p
                                                                        class="dp33">poníais</p><p
                                                                        class="dp33">ponían</p></td><td class="dp39"><p
                                                                        class="dp33">puse</p><p class="dp33">pusiste</p><p
                                                                        class="dp33">puso</p><p class="dp33">pusimos</p><p
                                                                        class="dp33">pusisteis</p><p class="dp33">pusieron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">pondré</p><p
                                                                        class="dp33">pondrás</p><p
                                                                        class="dp33">pondrá</p><p
                                                                        class="dp33">pondremos</p><p class="dp33">pondréis</p><p
                                                                        class="dp33">pondrán</p></td><td class="dp39"><p
                                                                        class="dp33">pondría</p><p
                                                                        class="dp33">pondrías</p><p
                                                                        class="dp33">pondría</p><p class="dp33">pondríamos</p><p
                                                                        class="dp33">pondríais</p><p class="dp33">pondrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">ponga</p><p class="dp33">pongas</p><p
                                                                        class="dp33">ponga</p><p
                                                                        class="dp33">pongamos</p><p
                                                                        class="dp33">pongáis</p><p
                                                                        class="dp33">pongan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">pusiera <i>o</i> pusiese</p><p
                                                                        class="dp33">pusieras <i>o</i> pusieses</p><p
                                                                        class="dp33">pusiera <i>o</i> pusiese</p><p
                                                                        class="dp33">pusiéramos <i>o</i> pusiésemos</p><p
                                                                        class="dp33">pusierais <i>o</i> pusieseis</p><p
                                                                        class="dp33">pusieran <i>o</i> pusiesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">pusiere</p><p
                                                                        class="dp33">pusieres</p><p
                                                                        class="dp33">pusiere</p><p class="dp33">pusiéremos</p><p
                                                                        class="dp33">pusiereis</p><p class="dp33">pusieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">pon (poné), poned</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">poner</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">puesto</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">poniendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n48">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>48. PUDRIR / PODRIR*</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">pudro</p><p
                                                                        class="dp33">pudres (pudrís)</p><p class="dp33">pudre</p><p
                                                                        class="dp33">pudrimos <i>o</i></p><p
                                                                        class="dp33">podrimos</p><p class="dp33">pudrís <i>o</i> podrís</p><p
                                                                        class="dp33">pudren</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">pudría <i>o</i> podría</p><p
                                                                        class="dp33">pudrías <i>o</i> podrías</p><p
                                                                        class="dp33">pudría <i>o</i> podría</p><p
                                                                        class="dp33">pudríamos <i>o</i></p><p
                                                                        class="dp33">podríamos</p><p class="dp33">pudríais <i>o</i> podríais</p><p
                                                                        class="dp33">pudrían <i>o</i> podrían</p></td><td
                                                                        class="dp39"><p class="dp33">pudrí <i>o</i> podrí</p><p
                                                                        class="dp33">pudriste <i>o</i> podriste</p><p
                                                                        class="dp33">pudrió <i>o</i> podrió</p><p
                                                                        class="dp33">pudrimos <i>o</i> podrimos</p><p
                                                                        class="dp33">pudristeis <i>o</i> podristeis</p><p
                                                                        class="dp33">pudrieron <i>o</i></p><p
                                                                        class="dp33">podrieron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">pudriré <i>o</i> podriré</p><p
                                                                        class="dp33">pudrirás <i>o</i> podrirás</p><p
                                                                        class="dp33">pudrirá <i>o</i> podrirá</p><p
                                                                        class="dp33">pudriremos <i>o</i></p><p
                                                                        class="dp33">podriremos</p><p class="dp33">pudriréis <i>o</i> podriréis</p><p
                                                                        class="dp33">pudrirán <i>o</i> podrirán</p></td><td
                                                                        class="dp39"><p class="dp33">pudriría <i>o</i> podriría</p><p
                                                                        class="dp33">pudrirías <i>o</i> podrirías</p><p
                                                                        class="dp33">pudriría <i>o</i> podriría</p><p
                                                                        class="dp33">pudriríamos <i>o</i></p><p
                                                                        class="dp33">podriríamos</p><p class="dp33">pudriríais <i>o</i> podriríais</p><p
                                                                        class="dp33">pudrirían <i>o</i> podrirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">pudra</p><p class="dp33">pudras</p><p
                                                                        class="dp33">pudra</p><p
                                                                        class="dp33">pudramos</p><p
                                                                        class="dp33">pudráis</p><p
                                                                        class="dp33">pudran</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">pudriera <i>o</i> pudriese</p><p
                                                                        class="dp33">pudrieras <i>o</i> pudrieses</p><p
                                                                        class="dp33">pudriera <i>o</i> pudriese</p><p
                                                                        class="dp33">pudriéramos <i>o</i> pudriésemos</p><p
                                                                        class="dp33">pudrierais <i>o</i> pudrieseis</p><p
                                                                        class="dp33">pudrieran <i>o</i> pudriesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">pudriere</p><p
                                                                        class="dp33">pudrieres</p><p class="dp33">pudriere</p><p
                                                                        class="dp33">pudriéremos</p><p class="dp33">pudriereis</p><p
                                                                        class="dp33">pudrieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">pudre (pudrí <i>o</i> podrí), pudrid <i>o</i> podrid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">pudrir <i>o</i> podrir</p></td><td
                                                                        class="dp26" coslpan="3"><p
                                                                        class="dp33">podrido</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">pudriendo</p></td></tr></tbody></table><p
                                                                        class="dp30">*&nbsp;En la norma culta americana, en el infinitivo y en algunas formas conjugadas de este verbo se presenta la alternancia <i>-u- / -o-</i> en la raíz; en esos mismos casos, la norma culta española solo admite hoy las formas con <i>-u-,</i> que también suelen ser las preferidas en América.</p><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n49">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>49. QUERER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">quiero</p><p
                                                                        class="dp33">quieres (querés)</p><p
                                                                        class="dp33">quiere</p><p
                                                                        class="dp33">queremos</p><p
                                                                        class="dp33">queréis</p><p
                                                                        class="dp33">quieren</p></td><td class="dp23"
                                                                                                         colspan="3"><p
                                                                        class="dp33">quería</p><p
                                                                        class="dp33">querías</p><p
                                                                        class="dp33">quería</p><p
                                                                        class="dp33">queríamos</p><p class="dp33">queríais</p><p
                                                                        class="dp33">querían</p></td><td class="dp39"><p
                                                                        class="dp33">quise</p><p
                                                                        class="dp33">quisiste</p><p
                                                                        class="dp33">quiso</p><p
                                                                        class="dp33">quisimos</p><p class="dp33">quisisteis</p><p
                                                                        class="dp33">quisieron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">querré</p><p
                                                                        class="dp33">querrás</p><p
                                                                        class="dp33">querrá</p><p
                                                                        class="dp33">querremos</p><p class="dp33">querréis</p><p
                                                                        class="dp33">querrán</p></td><td class="dp39"><p
                                                                        class="dp33">querría</p><p
                                                                        class="dp33">querrías</p><p
                                                                        class="dp33">querría</p><p class="dp33">querríamos</p><p
                                                                        class="dp33">querríais</p><p class="dp33">querrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">quiera</p><p
                                                                        class="dp33">quieras</p><p
                                                                        class="dp33">quiera</p><p
                                                                        class="dp33">queramos</p><p
                                                                        class="dp33">queráis</p><p
                                                                        class="dp33">quieran</p></td><td class="dp22"
                                                                                                         colspan="4"><p
                                                                        class="dp33">quisiera <i>o</i> quisiese</p><p
                                                                        class="dp33">quisieras <i>o</i> quisieses</p><p
                                                                        class="dp33">quisiera <i>o</i> quisiese</p><p
                                                                        class="dp33">quisiéramos <i>o</i> quisiésemos</p><p
                                                                        class="dp33">quisierais <i>o</i> quisieseis</p><p
                                                                        class="dp33">quisieran <i>o</i> quisiesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">quisiere</p><p
                                                                        class="dp33">quisieres</p><p class="dp33">quisiere</p><p
                                                                        class="dp33">quisiéremos</p><p class="dp33">quisiereis</p><p
                                                                        class="dp33">quisieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">quiere (queré), quered</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">querer</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">querido</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">queriendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n50">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>50. ROER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">roo <i>o</i> roigo <i>o</i> royo</p><p
                                                                        class="dp33">roes (roés)</p><p
                                                                        class="dp33">roe</p><p class="dp33">roemos</p><p
                                                                        class="dp33">roéis</p><p
                                                                        class="dp33">roen</p></td><td class="dp23"
                                                                                                      colspan="3"><p
                                                                        class="dp33">roía</p><p class="dp33">roías</p><p
                                                                        class="dp33">roía</p><p class="dp33">roíamos</p><p
                                                                        class="dp33">roíais</p><p class="dp33">roían</p></td><td
                                                                        class="dp39"><p class="dp33">roí</p><p
                                                                        class="dp33">roíste</p><p
                                                                        class="dp33">royó</p><p
                                                                        class="dp33">roímos</p><p
                                                                        class="dp33">roísteis</p><p
                                                                        class="dp33">royeron</p></td><td class="dp40"
                                                                                                         colspan="2"><p
                                                                        class="dp33">roeré</p><p class="dp33">roerás</p><p
                                                                        class="dp33">roerá</p><p
                                                                        class="dp33">roeremos</p><p
                                                                        class="dp33">roeréis</p><p
                                                                        class="dp33">roerán</p></td><td class="dp39"><p
                                                                        class="dp33">roería</p><p
                                                                        class="dp33">roerías</p><p
                                                                        class="dp33">roería</p><p
                                                                        class="dp33">roeríamos</p><p class="dp33">roeríais</p><p
                                                                        class="dp33">roerían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">roa <i>o</i> roiga <i>o</i> roya</p><p
                                                                        class="dp33">roas <i>o</i> roigas <i>o</i> royas</p><p
                                                                        class="dp33">roa <i>o</i> roiga <i>o</i> roya</p><p
                                                                        class="dp33">roamos <i>o</i> roigamos <i>o</i> royamos</p><p
                                                                        class="dp33">roáis <i>o</i> roigáis <i>o</i> royáis</p><p
                                                                        class="dp33">roan <i>o</i> roigan <i>o</i> royan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">royera <i>o</i> royese</p><p
                                                                        class="dp33">royeras <i>o</i> royeses</p><p
                                                                        class="dp33">royera <i>o</i> royese</p><p
                                                                        class="dp33">royéramos <i>o</i> royésemos</p><p
                                                                        class="dp33">royerais <i>o</i> royeseis</p><p
                                                                        class="dp33">royeran <i>o</i> royesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">royere</p><p
                                                                        class="dp33">royeres</p><p
                                                                        class="dp33">royere</p><p
                                                                        class="dp33">royéremos</p><p class="dp33">royereis</p><p
                                                                        class="dp33">royeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">roe (roé), roed</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">roer</p></td><td class="dp26"
                                                                                                      coslpan="3"><p
                                                                        class="dp33">roído</p></td><td class="dp42"
                                                                                                       colspan="2"><p
                                                                        class="dp33">royendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n51">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>51. SABER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">sé</p><p
                                                                        class="dp33">sabes (sabés)</p><p class="dp33">sabe</p><p
                                                                        class="dp33">sabemos</p><p
                                                                        class="dp33">sabéis</p><p class="dp33">saben</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">sabía</p><p class="dp33">sabías</p><p
                                                                        class="dp33">sabía</p><p
                                                                        class="dp33">sabíamos</p><p
                                                                        class="dp33">sabíais</p><p
                                                                        class="dp33">sabían</p></td><td class="dp39"><p
                                                                        class="dp33">supe</p><p class="dp33">supiste</p><p
                                                                        class="dp33">supo</p><p class="dp33">supimos</p><p
                                                                        class="dp33">supisteis</p><p class="dp33">supieron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">sabré</p><p class="dp33">sabrás</p><p
                                                                        class="dp33">sabrá</p><p
                                                                        class="dp33">sabremos</p><p
                                                                        class="dp33">sabréis</p><p
                                                                        class="dp33">sabrán</p></td><td class="dp39"><p
                                                                        class="dp33">sabría</p><p
                                                                        class="dp33">sabrías</p><p
                                                                        class="dp33">sabría</p><p
                                                                        class="dp33">sabríamos</p><p class="dp33">sabríais</p><p
                                                                        class="dp33">sabrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">sepa</p><p class="dp33">sepas</p><p
                                                                        class="dp33">sepa</p><p class="dp33">sepamos</p><p
                                                                        class="dp33">sepáis</p><p class="dp33">sepan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">supiera <i>o</i> supiese</p><p
                                                                        class="dp33">supieras <i>o</i> supieses</p><p
                                                                        class="dp33">supiera <i>o</i> supiese</p><p
                                                                        class="dp33">supiéramos <i>o</i> supiésemos</p><p
                                                                        class="dp33">supierais <i>o</i> supieseis</p><p
                                                                        class="dp33">supieran <i>o</i> supiesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">supiere</p><p
                                                                        class="dp33">supieres</p><p
                                                                        class="dp33">supiere</p><p class="dp33">supiéremos</p><p
                                                                        class="dp33">supiereis</p><p class="dp33">supieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">sabe (sabé), sabed</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">saber</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">sabido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">sabiendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n52">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>52. SALIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">salgo</p><p
                                                                        class="dp33">sales (salís)</p><p class="dp33">sale</p><p
                                                                        class="dp33">salimos</p><p
                                                                        class="dp33">salís</p><p class="dp33">salen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">salía</p><p class="dp33">salías</p><p
                                                                        class="dp33">salía</p><p
                                                                        class="dp33">salíamos</p><p
                                                                        class="dp33">salíais</p><p
                                                                        class="dp33">salían</p></td><td class="dp39"><p
                                                                        class="dp33">salí</p><p class="dp33">saliste</p><p
                                                                        class="dp33">salió</p><p
                                                                        class="dp33">salimos</p><p class="dp33">salisteis</p><p
                                                                        class="dp33">salieron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">saldré</p><p
                                                                        class="dp33">saldrás</p><p
                                                                        class="dp33">saldrá</p><p
                                                                        class="dp33">saldremos</p><p class="dp33">saldréis</p><p
                                                                        class="dp33">saldrán</p></td><td class="dp39"><p
                                                                        class="dp33">saldría</p><p
                                                                        class="dp33">saldrías</p><p
                                                                        class="dp33">saldría</p><p class="dp33">saldríamos</p><p
                                                                        class="dp33">saldríais</p><p class="dp33">saldrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">salga</p><p class="dp33">salgas</p><p
                                                                        class="dp33">salga</p><p
                                                                        class="dp33">salgamos</p><p
                                                                        class="dp33">salgáis</p><p
                                                                        class="dp33">salgan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">saliera <i>o</i> saliese</p><p
                                                                        class="dp33">salieras <i>o</i> salieses</p><p
                                                                        class="dp33">saliera <i>o</i> saliese</p><p
                                                                        class="dp33">saliéramos <i>o</i> saliésemos</p><p
                                                                        class="dp33">salierais <i>o</i> salieseis</p><p
                                                                        class="dp33">salieran <i>o</i> saliesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">saliere</p><p
                                                                        class="dp33">salieres</p><p
                                                                        class="dp33">saliere</p><p class="dp33">saliéremos</p><p
                                                                        class="dp33">saliereis</p><p class="dp33">salieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">sal (salí), salid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">salir</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">salido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">saliendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n53">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>53. SENTIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">siento</p><p
                                                                        class="dp33">sientes (sentís)</p><p
                                                                        class="dp33">siente</p><p
                                                                        class="dp33">sentimos</p><p
                                                                        class="dp33">sentís</p><p
                                                                        class="dp33">sienten</p></td><td class="dp23"
                                                                                                         colspan="3"><p
                                                                        class="dp33">sentía</p><p
                                                                        class="dp33">sentías</p><p
                                                                        class="dp33">sentía</p><p
                                                                        class="dp33">sentíamos</p><p class="dp33">sentíais</p><p
                                                                        class="dp33">sentían</p></td><td class="dp39"><p
                                                                        class="dp33">sentí</p><p
                                                                        class="dp33">sentiste</p><p
                                                                        class="dp33">sintió</p><p
                                                                        class="dp33">sentimos</p><p class="dp33">sentisteis</p><p
                                                                        class="dp33">sintieron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">sentiré</p><p
                                                                        class="dp33">sentirás</p><p
                                                                        class="dp33">sentirá</p><p class="dp33">sentiremos</p><p
                                                                        class="dp33">sentiréis</p><p class="dp33">sentirán</p></td><td
                                                                        class="dp39"><p class="dp33">sentiría</p><p
                                                                        class="dp33">sentirías</p><p class="dp33">sentiría</p><p
                                                                        class="dp33">sentiríamos</p><p class="dp33">sentiríais</p><p
                                                                        class="dp33">sentirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">sienta</p><p
                                                                        class="dp33">sientas</p><p
                                                                        class="dp33">sienta</p><p
                                                                        class="dp33">sintamos</p><p
                                                                        class="dp33">sintáis</p><p
                                                                        class="dp33">sientan</p></td><td class="dp22"
                                                                                                         colspan="4"><p
                                                                        class="dp33">sintiera <i>o</i> sintiese</p><p
                                                                        class="dp33">sintieras <i>o</i> sintieses</p><p
                                                                        class="dp33">sintiera <i>o</i> sintiese</p><p
                                                                        class="dp33">sintiéramos <i>o</i> sintiésemos</p><p
                                                                        class="dp33">sintierais <i>o</i> sintieseis</p><p
                                                                        class="dp33">sintieran <i>o</i> sintiesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">sintiere</p><p
                                                                        class="dp33">sintieres</p><p class="dp33">sintiere</p><p
                                                                        class="dp33">sintiéremos</p><p class="dp33">sintiereis</p><p
                                                                        class="dp33">sintieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">siente (sentí), sentid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">sentir</p></td><td class="dp26"
                                                                                                        coslpan="3"><p
                                                                        class="dp33">sentido</p></td><td class="dp42"
                                                                                                         colspan="2"><p
                                                                        class="dp33">sintiendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n54">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>54. SER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">soy</p><p
                                                                        class="dp33">eres (sos)</p><p
                                                                        class="dp33">es</p><p class="dp33">somos</p><p
                                                                        class="dp33">sois</p><p
                                                                        class="dp33">son</p></td><td class="dp23"
                                                                                                     colspan="3"><p
                                                                        class="dp33">era</p><p class="dp33">eras</p><p
                                                                        class="dp33">era</p><p class="dp33">éramos</p><p
                                                                        class="dp33">erais</p><p
                                                                        class="dp33">eran</p></td><td class="dp39"><p
                                                                        class="dp33">fui</p><p class="dp33">fuiste</p><p
                                                                        class="dp33">fue</p><p class="dp33">fuimos</p><p
                                                                        class="dp33">fuisteis</p><p
                                                                        class="dp33">fueron</p></td><td class="dp40"
                                                                                                        colspan="2"><p
                                                                        class="dp33">seré</p><p class="dp33">serás</p><p
                                                                        class="dp33">será</p><p class="dp33">seremos</p><p
                                                                        class="dp33">seréis</p><p class="dp33">serán</p></td><td
                                                                        class="dp39"><p class="dp33">sería</p><p
                                                                        class="dp33">serías</p><p class="dp33">sería</p><p
                                                                        class="dp33">seríamos</p><p
                                                                        class="dp33">seríais</p><p
                                                                        class="dp33">serían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">sea</p><p
                                                                        class="dp33">seas</p><p class="dp33">sea</p><p
                                                                        class="dp33">seamos</p><p class="dp33">seáis</p><p
                                                                        class="dp33">sean</p></td><td class="dp22"
                                                                                                      colspan="4"><p
                                                                        class="dp33">fuera <i>o</i> fuese</p><p
                                                                        class="dp33">fueras <i>o</i> fueses</p><p
                                                                        class="dp33">fuera <i>o</i> fuese</p><p
                                                                        class="dp33">fuéramos <i>o</i> fuésemos</p><p
                                                                        class="dp33">fuerais <i>o</i> fueseis</p><p
                                                                        class="dp33">fueran <i>o</i> fuesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">fuere</p><p class="dp33">fueres</p><p
                                                                        class="dp33">fuere</p><p
                                                                        class="dp33">fuéremos</p><p
                                                                        class="dp33">fuereis</p><p
                                                                        class="dp33">fueren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">sé (sé), sed</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">ser</p></td><td
                                                                        class="dp26" coslpan="3"><p
                                                                        class="dp33">sido</p></td><td class="dp42"
                                                                                                      colspan="2"><p
                                                                        class="dp33">siendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n55">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>55. SONREIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">sonrío</p><p
                                                                        class="dp33">sonríes (sonreís)</p><p
                                                                        class="dp33">sonríe</p><p
                                                                        class="dp33">sonreímos</p><p class="dp33">sonreís</p><p
                                                                        class="dp33">sonríen</p></td><td class="dp23"
                                                                                                         colspan="3"><p
                                                                        class="dp33">sonreía</p><p
                                                                        class="dp33">sonreías</p><p
                                                                        class="dp33">sonreía</p><p class="dp33">sonreíamos</p><p
                                                                        class="dp33">sonreíais</p><p class="dp33">sonreían</p></td><td
                                                                        class="dp39"><p class="dp33">sonreí</p><p
                                                                        class="dp33">sonreíste</p><p
                                                                        class="dp33">sonrió</p><p
                                                                        class="dp33">sonreímos</p><p class="dp33">sonreísteis</p><p
                                                                        class="dp33">sonrieron</p></td><td class="dp40"
                                                                                                           colspan="2"><p
                                                                        class="dp33">sonreiré</p><p class="dp33">sonreirás</p><p
                                                                        class="dp33">sonreirá</p><p class="dp33">sonreiremos</p><p
                                                                        class="dp33">sonreiréis</p><p class="dp33">sonreirán</p></td><td
                                                                        class="dp39"><p class="dp33">sonreiría</p><p
                                                                        class="dp33">sonreirías</p><p class="dp33">sonreiría</p><p
                                                                        class="dp33">sonreiríamos</p><p class="dp33">sonreiríais</p><p
                                                                        class="dp33">sonreirían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">sonría</p><p
                                                                        class="dp33">sonrías</p><p
                                                                        class="dp33">sonría</p><p
                                                                        class="dp33">sonriamos</p><p class="dp33">sonriáis</p><p
                                                                        class="dp33">sonrían</p></td><td class="dp22"
                                                                                                         colspan="4"><p
                                                                        class="dp33">sonriera <i>o</i> sonriese</p><p
                                                                        class="dp33">sonrieras <i>o</i> sonrieses</p><p
                                                                        class="dp33">sonriera <i>o</i> sonriese</p><p
                                                                        class="dp33">sonriéramos <i>o</i> sonriésemos</p><p
                                                                        class="dp33">sonrierais <i>o</i> sonrieseis</p><p
                                                                        class="dp33">sonrieran <i>o</i> sonriesen</p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33">sonriere</p><p
                                                                        class="dp33">sonrieres</p><p class="dp33">sonriere</p><p
                                                                        class="dp33">sonriéremos</p><p class="dp33">sonriereis</p><p
                                                                        class="dp33">sonrieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">sonríe (sonreí), sonreíd</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">sonreír</p></td><td class="dp26"
                                                                                                         coslpan="3"><p
                                                                        class="dp33">sonreído</p></td><td class="dp42"
                                                                                                          colspan="2"><p
                                                                        class="dp33">sonriendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n56">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>56. TAÑER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">taño</p><p
                                                                        class="dp33">tañes (tañés)</p><p class="dp33">tañe</p><p
                                                                        class="dp33">tañemos</p><p
                                                                        class="dp33">tañéis</p><p class="dp33">tañen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">tañía</p><p class="dp33">tañías</p><p
                                                                        class="dp33">tañía</p><p
                                                                        class="dp33">tañíamos</p><p
                                                                        class="dp33">tañíais</p><p
                                                                        class="dp33">tañían</p></td><td class="dp39"><p
                                                                        class="dp33">tañí</p><p class="dp33">tañiste</p><p
                                                                        class="dp33">tañó</p><p class="dp33">tañimos</p><p
                                                                        class="dp33">tañisteis</p><p class="dp33">tañeron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">tañeré</p><p
                                                                        class="dp33">tañerás</p><p
                                                                        class="dp33">tañerá</p><p
                                                                        class="dp33">tañeremos</p><p class="dp33">tañeréis</p><p
                                                                        class="dp33">tañerán</p></td><td class="dp39"><p
                                                                        class="dp33">tañería</p><p
                                                                        class="dp33">tañerías</p><p
                                                                        class="dp33">tañería</p><p class="dp33">tañeríamos</p><p
                                                                        class="dp33">tañeríais</p><p class="dp33">tañerían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">taña</p><p class="dp33">tañas</p><p
                                                                        class="dp33">taña</p><p class="dp33">tañamos</p><p
                                                                        class="dp33">tañáis</p><p class="dp33">tañan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">tañera <i>o</i> tañese</p><p
                                                                        class="dp33">tañeras <i>o</i> tañeses</p><p
                                                                        class="dp33">tañera <i>o</i> tañese</p><p
                                                                        class="dp33">tañéramos <i>o</i> tañésemos</p><p
                                                                        class="dp33">tañerais <i>o</i> tañeseis</p><p
                                                                        class="dp33">tañeran <i>o</i> tañesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">tañere</p><p
                                                                        class="dp33">tañeres</p><p
                                                                        class="dp33">tañere</p><p
                                                                        class="dp33">tañéremos</p><p class="dp33">tañereis</p><p
                                                                        class="dp33">tañeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">tañe (tañé), tañed</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">tañer</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">tañido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">tañendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n57">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>57. TENER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">tengo</p><p
                                                                        class="dp33">tienes (tenés)</p><p class="dp33">tiene</p><p
                                                                        class="dp33">tenemos</p><p
                                                                        class="dp33">tenéis</p><p
                                                                        class="dp33">tienen</p></td><td class="dp23"
                                                                                                        colspan="3"><p
                                                                        class="dp33">tenía</p><p class="dp33">tenías</p><p
                                                                        class="dp33">tenía</p><p
                                                                        class="dp33">teníamos</p><p
                                                                        class="dp33">teníais</p><p
                                                                        class="dp33">tenían</p></td><td class="dp39"><p
                                                                        class="dp33">tuve</p><p class="dp33">tuviste</p><p
                                                                        class="dp33">tuvo</p><p class="dp33">tuvimos</p><p
                                                                        class="dp33">tuvisteis</p><p class="dp33">tuvieron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">tendré</p><p
                                                                        class="dp33">tendrás</p><p
                                                                        class="dp33">tendrá</p><p
                                                                        class="dp33">tendremos</p><p class="dp33">tendréis</p><p
                                                                        class="dp33">tendrán</p></td><td class="dp39"><p
                                                                        class="dp33">tendría</p><p
                                                                        class="dp33">tendrías</p><p
                                                                        class="dp33">tendría</p><p class="dp33">tendríamos</p><p
                                                                        class="dp33">tendríais</p><p class="dp33">tendrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">tenga</p><p class="dp33">tengas</p><p
                                                                        class="dp33">tenga</p><p
                                                                        class="dp33">tengamos</p><p
                                                                        class="dp33">tengáis</p><p
                                                                        class="dp33">tengan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">tuviera <i>o</i> tuviese</p><p
                                                                        class="dp33">tuvieras <i>o</i> tuvieses</p><p
                                                                        class="dp33">tuviera <i>o</i> tuviese</p><p
                                                                        class="dp33">tuviéramos <i>o</i> tuviésemos</p><p
                                                                        class="dp33">tuvierais <i>o</i> tuvieseis</p><p
                                                                        class="dp33">tuvieran <i>o</i> tuviesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">tuviere</p><p
                                                                        class="dp33">tuvieres</p><p
                                                                        class="dp33">tuviere</p><p class="dp33">tuviéremos</p><p
                                                                        class="dp33">tuviereis</p><p class="dp33">tuvieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">ten (tené), tened</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">tener</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">tenido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">teniendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n58">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>58. TRAER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">traigo</p><p
                                                                        class="dp33">traes (traés)</p><p class="dp33">trae</p><p
                                                                        class="dp33">traemos</p><p
                                                                        class="dp33">traéis</p><p class="dp33">traen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">traía</p><p class="dp33">traías</p><p
                                                                        class="dp33">traía</p><p
                                                                        class="dp33">traíamos</p><p
                                                                        class="dp33">traíais</p><p
                                                                        class="dp33">traían</p></td><td class="dp39"><p
                                                                        class="dp33">traje</p><p
                                                                        class="dp33">trajiste</p><p
                                                                        class="dp33">trajo</p><p
                                                                        class="dp33">trajimos</p><p class="dp33">trajisteis</p><p
                                                                        class="dp33">trajeron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">traeré</p><p
                                                                        class="dp33">traerás</p><p
                                                                        class="dp33">traerá</p><p
                                                                        class="dp33">traeremos</p><p class="dp33">traeréis</p><p
                                                                        class="dp33">traerán</p></td><td class="dp39"><p
                                                                        class="dp33">traería</p><p
                                                                        class="dp33">traerías</p><p
                                                                        class="dp33">traería</p><p class="dp33">traeríamos</p><p
                                                                        class="dp33">traeríais</p><p class="dp33">traerían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">traiga</p><p
                                                                        class="dp33">traigas</p><p
                                                                        class="dp33">traiga</p><p
                                                                        class="dp33">traigamos</p><p class="dp33">traigáis</p><p
                                                                        class="dp33">traigan</p></td><td class="dp22"
                                                                                                         colspan="4"><p
                                                                        class="dp33">trajera <i>o</i> trajese</p><p
                                                                        class="dp33">trajeras <i>o</i> trajeses</p><p
                                                                        class="dp33">trajera <i>o</i> trajese</p><p
                                                                        class="dp33">trajéramos <i>o</i> trajésemos</p><p
                                                                        class="dp33">trajerais <i>o</i> trajeseis</p><p
                                                                        class="dp33">trajeran <i>o</i> trajesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">trajere</p><p
                                                                        class="dp33">trajeres</p><p
                                                                        class="dp33">trajere</p><p class="dp33">trajéremos</p><p
                                                                        class="dp33">trajereis</p><p class="dp33">trajeren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">trae (traé), traed</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">traer</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">traído</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">trayendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n59">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>59. VALER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">valgo</p><p
                                                                        class="dp33">vales (valés)</p><p class="dp33">vale</p><p
                                                                        class="dp33">valemos</p><p
                                                                        class="dp33">valéis</p><p class="dp33">valen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">valía</p><p class="dp33">valías</p><p
                                                                        class="dp33">valía</p><p
                                                                        class="dp33">valíamos</p><p
                                                                        class="dp33">valíais</p><p
                                                                        class="dp33">valían</p></td><td class="dp39"><p
                                                                        class="dp33">valí</p><p class="dp33">valiste</p><p
                                                                        class="dp33">valió</p><p
                                                                        class="dp33">valimos</p><p class="dp33">valisteis</p><p
                                                                        class="dp33">valieron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">valdré</p><p
                                                                        class="dp33">valdrás</p><p
                                                                        class="dp33">valdrá</p><p
                                                                        class="dp33">valdremos</p><p class="dp33">valdréis</p><p
                                                                        class="dp33">valdrán</p></td><td class="dp39"><p
                                                                        class="dp33">valdría</p><p
                                                                        class="dp33">valdrías</p><p
                                                                        class="dp33">valdría</p><p class="dp33">valdríamos</p><p
                                                                        class="dp33">valdríais</p><p class="dp33">valdrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">valga</p><p class="dp33">valgas</p><p
                                                                        class="dp33">valga</p><p
                                                                        class="dp33">valgamos</p><p
                                                                        class="dp33">valgáis</p><p
                                                                        class="dp33">valgan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">valiera <i>o</i> valiese</p><p
                                                                        class="dp33">valieras <i>o</i> valieses</p><p
                                                                        class="dp33">valiera <i>o</i> valiese</p><p
                                                                        class="dp33">valiéramos <i>o</i> valiésemos</p><p
                                                                        class="dp33">valierais <i>o</i> valieseis</p><p
                                                                        class="dp33">valieran <i>o</i> valiesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">valiere</p><p
                                                                        class="dp33">valieres</p><p
                                                                        class="dp33">valiere</p><p class="dp33">valiéremos</p><p
                                                                        class="dp33">valiereis</p><p class="dp33">valieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">vale (valé), valed</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">valer</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">valido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">valiendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n60">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>60. VENIR</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">vengo</p><p
                                                                        class="dp33">vienes (venís)</p><p class="dp33">viene</p><p
                                                                        class="dp33">venimos</p><p
                                                                        class="dp33">venís</p><p class="dp33">vienen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">venía</p><p class="dp33">venías</p><p
                                                                        class="dp33">venía</p><p
                                                                        class="dp33">veníamos</p><p
                                                                        class="dp33">veníais</p><p
                                                                        class="dp33">venían</p></td><td class="dp39"><p
                                                                        class="dp33">vine</p><p class="dp33">viniste</p><p
                                                                        class="dp33">vino</p><p class="dp33">vinimos</p><p
                                                                        class="dp33">vinisteis</p><p class="dp33">vinieron</p></td><td
                                                                        class="dp40" colspan="2"><p
                                                                        class="dp33">vendré</p><p
                                                                        class="dp33">vendrás</p><p
                                                                        class="dp33">vendrá</p><p
                                                                        class="dp33">vendremos</p><p class="dp33">vendréis</p><p
                                                                        class="dp33">vendrán</p></td><td class="dp39"><p
                                                                        class="dp33">vendría</p><p
                                                                        class="dp33">vendrías</p><p
                                                                        class="dp33">vendría</p><p class="dp33">vendríamos</p><p
                                                                        class="dp33">vendríais</p><p class="dp33">vendrían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">venga</p><p class="dp33">vengas</p><p
                                                                        class="dp33">venga</p><p
                                                                        class="dp33">vengamos</p><p
                                                                        class="dp33">vengáis</p><p
                                                                        class="dp33">vengan</p></td><td class="dp22"
                                                                                                        colspan="4"><p
                                                                        class="dp33">viniera <i>o</i> viniese</p><p
                                                                        class="dp33">vinieras <i>o</i> vinieses</p><p
                                                                        class="dp33">viniera <i>o</i> viniese</p><p
                                                                        class="dp33">viniéramos <i>o</i> viniésemos</p><p
                                                                        class="dp33">vinierais <i>o</i> vinieseis</p><p
                                                                        class="dp33">vinieran <i>o</i> viniesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">viniere</p><p
                                                                        class="dp33">vinieres</p><p
                                                                        class="dp33">viniere</p><p class="dp33">viniéremos</p><p
                                                                        class="dp33">viniereis</p><p class="dp33">vinieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">ven (vení), venid</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">venir</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">venido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">viniendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n61">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>61. VER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">veo</p><p
                                                                        class="dp33">ves (ves)</p><p class="dp33">ve</p><p
                                                                        class="dp33">vemos</p><p class="dp33">veis</p><p
                                                                        class="dp33">ven</p></td><td class="dp23"
                                                                                                     colspan="3"><p
                                                                        class="dp33">veía</p><p class="dp33">veías</p><p
                                                                        class="dp33">veía</p><p class="dp33">veíamos</p><p
                                                                        class="dp33">veíais</p><p class="dp33">veían</p></td><td
                                                                        class="dp39"><p class="dp33">vi</p><p
                                                                        class="dp33">viste</p><p class="dp33">vio</p><p
                                                                        class="dp33">vimos</p><p
                                                                        class="dp33">visteis</p><p
                                                                        class="dp33">vieron</p></td><td class="dp40"
                                                                                                        colspan="2"><p
                                                                        class="dp33">veré</p><p class="dp33">verás</p><p
                                                                        class="dp33">verá</p><p class="dp33">veremos</p><p
                                                                        class="dp33">veréis</p><p class="dp33">verán</p></td><td
                                                                        class="dp39"><p class="dp33">vería</p><p
                                                                        class="dp33">verías</p><p class="dp33">vería</p><p
                                                                        class="dp33">veríamos</p><p
                                                                        class="dp33">veríais</p><p
                                                                        class="dp33">verían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33">vea</p><p
                                                                        class="dp33">veas</p><p class="dp33">vea</p><p
                                                                        class="dp33">veamos</p><p class="dp33">veáis</p><p
                                                                        class="dp33">vean</p></td><td class="dp22"
                                                                                                      colspan="4"><p
                                                                        class="dp33">viera <i>o</i> viese</p><p
                                                                        class="dp33">vieras <i>o</i> vieses</p><p
                                                                        class="dp33">viera <i>o</i> viese</p><p
                                                                        class="dp33">viéramos <i>o</i> viésemos</p><p
                                                                        class="dp33">vierais <i>o</i> vieseis</p><p
                                                                        class="dp33">vieran <i>o</i> viesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">viere</p><p class="dp33">vieres</p><p
                                                                        class="dp33">viere</p><p
                                                                        class="dp33">viéremos</p><p
                                                                        class="dp33">viereis</p><p
                                                                        class="dp33">vieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p class="dp33">ve (ve), ved</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33">ver</p></td><td
                                                                        class="dp26" coslpan="3"><p
                                                                        class="dp33">visto</p></td><td class="dp42"
                                                                                                       colspan="2"><p
                                                                        class="dp33">viendo</p></td></tr></tbody></table><p
                                                                        class="dp33"></p><table class="dp55"><tbody><tr><td><p
                                                                        class="dp36"><a name="n62">&nbsp;</a><a
                                                                        href="#advertencias">[Advertencias]</a></p></td></tr></tbody></table><table
                                                                        class="dp56"><tbody><tr><td class="dp43"
                                                                                                    colspan="8"><b>62. YACER</b></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>INDICATIVO</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp23" colspan="3"><p class="dp33"><b>pret. imperfecto</b> /<br><b>copretérito</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>pret. perfecto</b><br><b>simple</b> / <b>pretérito</b></p></td><td
                                                                        class="dp40" colspan="2"><p class="dp33"><b>futuro simple</b> /<br><b>futuro</b></p></td><td
                                                                        class="dp39"><p
                                                                        class="dp33"><b>condicional</b><br><b>simple</b> /<br><b>pospretérito</b></p></td></tr><tr><td
                                                                        class="dp39"><p class="dp33">yazco <i>o</i> yazgo <i>o</i> yago</p><p
                                                                        class="dp33">yaces (yacés)</p><p class="dp33">yace</p><p
                                                                        class="dp33">yacemos</p><p
                                                                        class="dp33">yacéis</p><p class="dp33">yacen</p></td><td
                                                                        class="dp23" colspan="3"><p
                                                                        class="dp33">yacía</p><p class="dp33">yacías</p><p
                                                                        class="dp33">yacía</p><p
                                                                        class="dp33">yacíamos</p><p
                                                                        class="dp33">yacíais</p><p
                                                                        class="dp33">yacían</p></td><td class="dp39"><p
                                                                        class="dp33">yací</p><p class="dp33">yaciste</p><p
                                                                        class="dp33">yació</p><p
                                                                        class="dp33">yacimos</p><p class="dp33">yacisteis</p><p
                                                                        class="dp33">yacieron</p></td><td class="dp40"
                                                                                                          colspan="2"><p
                                                                        class="dp33">yaceré</p><p
                                                                        class="dp33">yacerás</p><p
                                                                        class="dp33">yacerá</p><p
                                                                        class="dp33">yaceremos</p><p class="dp33">yaceréis</p><p
                                                                        class="dp33">yacerán</p></td><td class="dp39"><p
                                                                        class="dp33">yacería</p><p
                                                                        class="dp33">yacerías</p><p
                                                                        class="dp33">yacería</p><p class="dp33">yaceríamos</p><p
                                                                        class="dp33">yaceríais</p><p class="dp33">yacerían</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>SUBJUNTIVO</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p class="dp33"><b>presente</b></p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33"><b>pret. imperfecto</b> / <b>pretérito</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>futuro simple</b> / <b>futuro</b></p></td></tr><tr><td
                                                                        class="dp25" colspan="2"><p
                                                                        class="dp33">yazca <i>o</i> yazga <i>o</i> yaga</p><p
                                                                        class="dp33">yazcas <i>o</i> yazgas <i>o</i> yagas</p><p
                                                                        class="dp33">yazca <i>o</i> yazga <i>o</i> yaga</p><p
                                                                        class="dp33">yazcamos <i>o</i> yazgamos <i>o</i> yagamos</p><p
                                                                        class="dp33">yazcáis <i>o</i> yazgáis <i>o</i> yagáis</p><p
                                                                        class="dp33">yazcan <i>o</i> yazgan <i>o</i> yagan</p></td><td
                                                                        class="dp22" colspan="4"><p class="dp33">yaciera <i>o</i> yaciese</p><p
                                                                        class="dp33">yacieras <i>o</i> yacieses</p><p
                                                                        class="dp33">yaciera <i>o</i> yaciese</p><p
                                                                        class="dp33">yaciéramos <i>o</i> yaciésemos</p><p
                                                                        class="dp33">yacierais <i>o</i> yacieseis</p><p
                                                                        class="dp33">yacieran <i>o</i> yaciesen</p></td><td
                                                                        class="dp42" colspan="2"><p
                                                                        class="dp33">yaciere</p><p
                                                                        class="dp33">yacieres</p><p
                                                                        class="dp33">yaciere</p><p class="dp33">yaciéremos</p><p
                                                                        class="dp33">yaciereis</p><p class="dp33">yacieren</p></td></tr><tr><td
                                                                        class="dp41" colspan="8"><p class="dp33"><b>IMPERATIVO</b></p></td></tr><tr><td
                                                                        class="dp44" colspan="8"><p
                                                                        class="dp33">yace <i>o</i> yaz (yacé), yaced</p></td></tr><tr><td
                                                                        class="dp43" colspan="8"><p class="dp33"><b>FORMAS NO PERSONALES</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p class="dp33"><b>infinitivo</b></p></td><td
                                                                        class="dp26" coslpan="3"><p class="dp33"><b>participio</b></p></td><td
                                                                        class="dp42" colspan="2"><p class="dp33"><b>gerundio</b></p></td></tr><tr><td
                                                                        class="dp24" colspan="3"><p
                                                                        class="dp33">yacer</p></td><td class="dp26"
                                                                                                       coslpan="3"><p
                                                                        class="dp33">yacido</p></td><td class="dp42"
                                                                                                        colspan="2"><p
                                                                        class="dp33">yaciendo</p></td></tr></tbody></table></div></div></span>
                                                                </div>
                                                            </div>
                                                        </div>


                                                    </div>
                                                    <div class="view-node-navigation">
                                                        <div class="view view-nodes-view-a view-id-nodes_view_a view-display-id-block view-dom-id-d5cb53852604165c71724098d07f6e45">


                                                        </div>
                                                    </div>
                                                </div>
                                            </div>
                                        </div>

                                        <div class="clearfix"></div>
                                    </div>
                                </div>
                            </div>
                        </div>
                        <div class="block block-block block-81 block-block-81 even block-without-title"
                             id="block-block-81">
                            <div class="block-inner clearfix">

                                <div class="content clearfix">
                                    <div class="left">Con el patrocinio de</div>
                                    <div class="right"><img alt="" src="/sites/all/common/images/logos/telefonica.png">
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
</div>
</body>
</html>